/********************************************************************
 bbox.c --
        Bounding box of mesh points, and cuboid operations. 
 
 Roman Putanowicz <romek@sect.mce.hw.ac.uk>
 SECT Research Group, Heriot-Watt University, Edinburgh    
 ********************************************************************/

/* $Id: bbox.h,v 1.7 2002/08/03 18:32:43 peteri Exp $ */

#ifndef BBOX_H
#define BBOX_H

#ifdef __cplusplus
extern "C"
{
#endif

#include "e_lib.h"
#include "vertex.h"
#include "mesh.h"


/* factor of which mesh bounding box is enlarged */
#define CUBOID_SCALE  1.5

void BoundingBox (int nvertex, Vertex **vertex, double *low, double *high);
void MakeCuboid (TetrahMesh *tetMesh, double *low, double *high);

void ExtractMeshFromCuboid (MeshGenData *mgds);
void IdentifyOuterTetFlood (MeshGenData *mgds);
void DeleteOuterTetrah (TetrahMesh *tetMesh);
void RemoveUnconnectedVertices(TetrahMesh *tetMesh);

#ifdef __cplusplus
}
#endif


#endif /* BBOX_H */

/* Modifications:
 * $Log: bbox.h,v $
 * Revision 1.7  2002/08/03 18:32:43  peteri
 * - smaller cuboid
 *
 * Revision 1.6  2002/07/26 14:59:44  peteri
 * - mesh data is separated from boundary data
 * - now it is possible to setup the boundary data separately
 *   and refine that boundary (without having a MESH)
 *
 * Revision 1.5  2002/03/18 15:53:46  peteri
 * Some renaming and cleaning
 *
 * Revision 1.4  2002/02/16 18:55:51  romek
 * Doc fix.
 *
 */
