%{
/********************************************************************
 Roman Putanowicz <romek@sect.mce.hw.ac.uk>
 SECT Research Group, Heriot-Watt University, Edinburgh    

 Changes: for log of changes see the end of the file    
 ********************************************************************/

/* $Id: ctr_parser.y,v 1.6 2002/07/28 16:57:00 peteri Exp $ */

# include <stdio.h>
# include <stdlib.h>
# include "ctr_guard.h"
# include "ctr_read.h"

extern void yyerror ( );
extern int  yylex  ( );

static int fileCounter;
static int optMethod = 0;


/* pointer to  object to store parsed data */
#define YYPARSE_PARAM mgCtrl
%}

%union {
    int       i;
    double    d;
    char     *s;
    char      c;
}

%token NMESHES ALPHA BETA
%token REFINE_METHOD 
%token OUTPUT_MESH_FILE
%token INPUT_MESH_FILES	
%token DATAEND
%token INTEGER DOUBLE NAME STRING
%token LOG_LEVEL NO_LOG MINIMAL_LOG NORMAL_LOG FULL_LOG
%token EDGE_LEN
%token OPTIMIZATION INSERTING SWAPPING LAPLACE COLLAPSING

%type	<i> INTEGER 
%type	<d> DOUBLE 
%type	<s> NAME 
%type   <s> STRING
%type   <i> log_level_value refine_method_value
%%

specification
        : rdf_specification DATAEND
          {
             return 1;
          }
	;
rdf_specification 
        : rdf_mandatory rdf_optional
        ;

rdf_mandatory
        : nmeshes input_mesh_files output_mesh_file refine_method alpha beta
        ;

nmeshes 
        : NMESHES INTEGER  
          {
            if (MGCSet_nmeshes ((MeshGenCtrl*) mgCtrl, $2) != 1)
                yyerror("");
          }
        ;
input_mesh_files 
        : input_mesh_start input_mesh_list
         {
           if (fileCounter != ((MeshGenCtrl *)mgCtrl)->nmeshes)
              yyerror("Expecting names of input files");
         }
        ;

input_mesh_start 
        : INPUT_MESH_FILES
          {
            fileCounter = 0;
          }  
        ;
input_mesh_list
        : input_mesh_file input_mesh_list
        | /* empty */ 
        ;

input_mesh_file 
        : STRING
          {
            if (MGCSet_inputFileNames ((MeshGenCtrl*) mgCtrl, fileCounter, $1) != 1)
                yyerror("");
            fileCounter++;
          }
        ;

output_mesh_file
        : OUTPUT_MESH_FILE STRING
          {
            if (MGCSet_outputFileName ((MeshGenCtrl*) mgCtrl, $2) != 1)
                yyerror("");
          }
        ;


refine_method
        : REFINE_METHOD refine_method_value 
          {
            if (MGCSet_refineMethod ((MeshGenCtrl*) mgCtrl, $2) != 1)
                yyerror("");
          }
        ;

refine_method_value
        : EDGE_LEN 
          {
            $$ = VM_REFINE_EDGE_LEN;
          }
        ; 

alpha   
        : ALPHA DOUBLE
          {
            if (MGCSet_alpha ((MeshGenCtrl*) mgCtrl, $2) != 1)
                yyerror("");
          }
        ;
beta    
        : BETA DOUBLE
          {
            if (MGCSet_beta ((MeshGenCtrl*) mgCtrl, $2) != 1)
                yyerror("");
          }
        ;

rdf_optional 
        : /* empty */
        | rdf_optional optional_specification
        ;

optional_specification
        : log_level
        | optimization
        ;

log_level
        : LOG_LEVEL log_level_value
          {
            if (MGCSet_logLevel ((MeshGenCtrl*) mgCtrl, $2) != 1)
                yyerror("");
          }

log_level_value
        : NO_LOG
          {
            $$ = VM_NO_LOG;
          }
        | MINIMAL_LOG
          {
            $$ = VM_MINIMAL_LOG;
          }
        | NORMAL_LOG
          {
            $$ = VM_NORMAL_LOG;
          }
        | FULL_LOG
          { 
            $$ = VM_FULL_LOG;
          }
        ; 

optimization
        : OPTIMIZATION opt_method
          {
            if (MGCSet_optimization ((MeshGenCtrl*) mgCtrl, optMethod) != 1)
                yyerror("");
          }
        ;
              
opt_method
        : /* empty */ 
        | opt_method opt_specification
        ;

opt_specification
        : INSERTING
          {
            optMethod |= OPT_INSERTING;
          }
        | SWAPPING
          {
            optMethod |= OPT_SWAPPING;
          }
        | LAPLACE
          {
            optMethod |= OPT_LAPLACE;
          }
        | COLLAPSING
          {
            optMethod |= OPT_COLLAPSING;
          }
        ;

%%

/* Modifications:
 * $Log: ctr_parser.y,v $
 * Revision 1.6  2002/07/28 16:57:00  peteri
 * - the required optimization method is specified in the file
 *   OPTIMIZATION INSERTING LAPLACE SWAPPING COLLAPSING
 *
 * Revision 1.5  2002/07/26 17:53:33  peteri
 * - Internal point is not needed for flooding any more
 *
 * Revision 1.4  2002/07/26 13:06:28  peteri
 * CENTR_VERT_LEN refinement method is removed
 *
 * Revision 1.3  2002/01/31 23:31:02  peteri
 * - waitingPoints member is removed,
 * - related functions are updated, kernel needs only the tetMesh
 * - other minor editing, changes
 *
 * Revision 1.2  2002/01/31 22:37:38  peteri
 * Minor modifications, some renaming.
 *
 * Revision 1.1.1.1  2002/01/30 10:20:55  peteri
 * Imported using TkCVS
 *
 */

