/* $Id: edge.h,v 1.7 2002/07/26 11:39:08 peteri Exp $ */

#ifndef EDGE_H
#define EDGE_H

#ifdef __cplusplus
extern "C"
{
#endif


#include "mstack.h"
#include "vertex.h"
#include "face.h"

/* macros to access of structure members */
#define EDGE_VERTEX(e, i)      ((e)->vertex[i])
#define EDGE_VCOORDS(e, i)     ((e)->vertex[i]->coords)
#define EDGE_NEIGHBOUR(e, i)   ((e)->neighbour[i])
#define EDGE_FLAG(e)           ((e)->flag)
#define EDGE_NEXT(e)           ((e)->next)
#define EDSGE_PREV(e)          ((e)->prev)

/* unit of allocation, when incrementing the edge table */
#define EDGE_ALLOC_INCR  1024

/* Edges flags */


typedef struct edge_
{
  Vertex          *vertex[2];    /* edge vertices */
  struct face_    *neighbour[2]; /* neighbouring faces */
  Tos_MStack       splitEdges;
  int              flag;
  int              mesh;         /* which boundary mesh contains it */
  
  int              domain;
  int              number;
  int              subdomain;
  unsigned int     id;        /* unique id number (used only for debugging) */
  struct edge_    *prev;      /* pointer to previous element in the list 
                                 (previously added) */
  struct edge_    *next;      /* pointer to next element in the list */
}
Edge;

void  EdgeMemPoolInit(void);
Edge *EdgeCreate (void);
void  EdgeDelete (Edge *edge);

#ifdef __cplusplus
}
#endif

#endif /* EDGE_H */

/* Modifications:
 * $Log: edge.h,v $
 * Revision 1.7  2002/07/26 11:39:08  peteri
 * - edge stores split edges and not splitting points
 * - edge stores the mesh number to which it belongs to
 *
 * Revision 1.6  2002/03/18 15:54:54  peteri
 * multiple adjacency flag is removed
 *
 * Revision 1.5  2002/02/16 16:54:57  romek
 * Doc fix.
 *
 */
