
#ifndef FIND_H
#define FIND_H

#ifdef __cplusplus
extern "C"
{
#endif

#include "mstack.h"
#include "vertex.h"
#include "edge.h"
#include "face.h"
#include "tetrah.h"
#include "mesh.h"


/* tollerance for tetst of orient3d */
#define POSITION_TOL 1.0e-14

/* tolerance for insphere test */
#define COSPHER_TRESHOLD 1.0e-13

/* possible point location in tetrahedron */
enum PointLocation {OUTSIDE=0, INSIDE, ONFACE, ONEDGE, ONVERTEX};


int WhereInTetrah (Tetrah *tet, Vertex *point,
                   int *faceNum, int *faceSum);
void FindSharedEdge (Tetrah *tet, int face1, int face2, Edge *e);
int FindBaseSearchAll(Tetrah *st, Vertex *point, Tos_MStack base);
int FindBaseWalk (Tetrah *st, Vertex *point, Tos_MStack base);
int FindCavity (Vertex *point, Tos_MStack base, int force, Tos_MStack cavity,
                Tos_MStack outcavStack);
void FindConnectedPoints (Vertex* point, Tos_MStack connectedPoints);
void FindEdgesIntersectingFacePlane (Tetrah *tet, Face *face,
                                     int *nodes, int cnodes,
                                     Tos_MStack intersectingEdges);
int FindEdgeShell (Edge *edge, Tos_MStack shell);
int FindEdgeEquator (Edge *edge, Tetrah *startTet, Tos_MStack equator);
int FindFaceShell (Face *face, Tetrah **shell);
Tetrah *FindInnerTetrah (MeshGenData *mgds);
void FindMissingEdges (MeshGenData *mgds, Tos_MStack missingEdges);
void FindMissingFaces (MeshGenData *mgds, Tos_MStack missingFaces);
int FindPointsOnFaceEdges (Face *face, Tos_MStack pointsOnEdges);
void FindPointsTetrahAdjSet (Tos_MStack pointsStack, Tos_MStack tetrahStack);
int FindSharedNodes (Tetrah *tet, Tos_MStack pointsStack, int *nodes);
Tetrah *FindSubdomainTetrah (MeshGenData *mgds, int subdomain);


/* find_alpha.c */
double FindAlphaParam (MeshGenData *mgds, double *min_elength, 
                       double *max_elength);


#ifdef __cplusplus
}
#endif

#endif /* FIND_H */

