#include "eg_lib.h"
#include "e_lib.h"
#include "vertex.h"
#include "mesh.h"


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 FindAlphaParam --
        Find alpha parameter required to agree edge lengths and 
        vertex mesh parameters. 
        Additionaly if min_elength or max_elength are not NULL
        they will be set to the minimum and maximum edge length
        respectively.
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
double FindAlphaParam (MeshGenData *mgds, double *min_elength, 
                       double *max_elength)
{
  Edge     *edge;
  Vertex   *v1, *v2;
  int      i, iedge;
  double   alpha = 1.0;
  double   loc_alpha;
  double   min_el = HUGE_VAL;
  double   max_el = -1.0;
  double   elength;

  for (i = 0; i < mgds->NMeshes; i++)
  {
    /* prepare to traverse boundary edges table */
    for(iedge = 0; iedge < mgds->eTableLength[i]; iedge++)
    {
      edge = mgds->eTable[i][iedge];
      
      v1 = EDGE_VERTEX (edge, 0);
      v2 = EDGE_VERTEX (edge, 1);

      elength = eg_PointDist3D (v1->coords, v2->coords);
      
      /* check edge length against mesh parameter at vertices */
      loc_alpha = elength/v1->meshParam;
      if (loc_alpha > alpha)
      {
        alpha = loc_alpha;
      }

      loc_alpha = elength/v2->meshParam;
      if (loc_alpha > alpha)
      {
        alpha = loc_alpha;
      }

      /* calculate minimal and maximal edge length */
      if (elength < min_el) 
        min_el = elength;
      if (elength > max_el) 
        max_el = elength;
    } /* end of loop over edges */
  } /* end of loop over meshes */

  if (min_elength != NULL)
    *min_elength = min_el;
  if (max_elength != NULL)
    *max_elength = max_el;

  return alpha;
} /* end of FindAlphaParam */

