/* $Id */

#include <assert.h>
#include "eg_lib.h"
#include "mstack.h"
#include "find.h"
#include "basic_op.h"
#include "mesh_op.h"
#include "measures.h"
#include "flip.h"


#define GetIJElem(matrix, n, m, i, j, k) (matrix)[(m)*3*(i)+3*(j)+(k)]

/* this is to identify that a minimal measure of the configureation has 
 * been selected and it must be larger than 4*PI 
 */
#define MIN_MEASURE_INIT   1000.0

static int TriangConfig_3[1][1][3] =
{
    { { 0, 1, 2 } }
};


static int TriangConfig_4[2][2][3] =
{
    { { 2, 0, 1 },
      { 0, 2, 3 }  },
    { { 1, 3, 0 },
      { 3, 1, 2 }  }
};

static int TriangConfig_5[5][3][3] =
{
    { { 0, 1, 2 },
      { 0, 2, 3 },
      { 0, 3, 4 }  },
    { { 1, 4, 0 },
      { 1, 3, 4 },
      { 1, 2, 3 }  },
    { { 2, 0, 1 },
      { 2, 4, 0 },
      { 2, 3, 4 }  },
    { { 3, 1, 2 },
      { 3, 0, 1 },
      { 3, 4, 0 }  },
    { { 4, 0, 1 },
      { 4, 1, 2 },
      { 4, 2, 3 }  }
};


static int TriangConfig_6[14][4][3] =
{
    { { 0, 1, 2 },      /* 1 */
      { 0, 2, 3 },
      { 0, 3, 4 },
      { 0, 4, 5 }  },
    { { 1, 2, 3 },      /* 2 */
      { 1, 3, 4 },
      { 1, 4, 5 },
      { 1, 5, 0 }  },
    { { 2, 3, 4 },      /* 3 */
      { 2, 4, 5 },
      { 2, 5, 0 },
      { 2, 0, 1 }  },
    { { 3, 4, 5 },      /* 4 */
      { 3, 5, 0 },
      { 3, 0, 1 },
      { 3, 1, 2 }  },
    { { 4, 5, 0 },      /* 5 */
      { 4, 0, 1 },
      { 4, 1, 2 },
      { 4, 2, 3 }  },
    { { 5, 0, 1 },      /* 6 */
      { 5, 1, 2 },
      { 5, 2, 3 },
      { 5, 3, 4 }  },

    { { 2, 0, 1 },      /* 7 */
      { 3, 0, 2 },
      { 3, 5, 0 },
      { 5, 3, 4 }  },
    { { 3, 1, 2 },      /* 8 */
      { 4, 1, 3 },
      { 4, 0, 1 },
      { 0, 4, 5 }  },
    { { 1, 5, 0 },      /* 9 */
      { 2, 5, 1 },
      { 2, 4, 5 },
      { 4, 2, 3 }  },

    { { 2, 0, 1 },      /* 10 */
      { 2, 5, 0 },
      { 3, 5, 2 },
      { 5, 3, 4 }  },
    { { 3, 1, 2 },      /* 11 */
      { 3, 0, 1 },
      { 4, 0, 3 },
      { 0, 4, 5 }  },
    { { 1, 5, 0 },      /* 12 */
      { 1, 4, 5 },
      { 2, 4, 1 },
      { 4, 2, 3 }  },

    { { 2, 0, 1 },      /* 13 */
      { 4, 2, 3 },
      { 0, 4, 5 },
      { 0, 2, 4 }  },
    { { 3, 1, 2 },      /* 14 */
      { 5, 3, 4 },
      { 1, 5, 0 },
      { 3, 5, 1 }  }

};

static int TriangConfig_7[42][5][3] =
{
    { { 0, 1, 2 },      /* 1 */
      { 0, 2, 3 },
      { 0, 3, 4 },
      { 0, 4, 5 },
      { 0, 5, 6 }  },
    { { 1, 2, 3 },      /* 2 */
      { 1, 3, 4 },
      { 1, 4, 5 },
      { 1, 5, 6 },
      { 1, 6, 0 }  },
    { { 2, 3, 4 },      /* 3 */
      { 2, 4, 5 },
      { 2, 5, 6 },
      { 2, 6, 0 },
      { 2, 0, 1 }  },
    { { 3, 4, 5 },      /* 4 */
      { 3, 5, 6 },
      { 3, 6, 0 },
      { 3, 0, 1 },
      { 3, 1, 2 }  },
    { { 4, 5, 6 },      /* 5 */
      { 4, 6, 0 },
      { 4, 0, 1 },
      { 4, 1, 2 },
      { 4, 2, 3 }  },
    { { 5, 6, 0 },      /* 6 */
      { 5, 0, 1 },
      { 5, 1, 2 },
      { 5, 2, 3 },
      { 5, 3, 4 }  },
    { { 6, 0, 1 },      /* 7 */
      { 6, 1, 2 },
      { 6, 2, 3 },
      { 6, 3, 4 },
      { 6, 4, 5 }  },

    { { 2, 0, 1 },      /* 8 */
      { 3, 0, 2 },
      { 3, 6, 0 },
      { 4, 6, 3 },
      { 6, 4, 5 }  },
    { { 3, 1, 2 },      /* 9 */
      { 4, 1, 3 },
      { 4, 0, 1 },
      { 5, 0, 4 },
      { 0, 5, 6 }  },
    { { 4, 2, 3 },      /* 10 */
      { 5, 2, 4 },
      { 5, 1, 2 },
      { 6, 1, 5 },
      { 1, 6, 0 }  },
    { { 2, 0, 1 },      /* 11 */
      { 2, 6, 0 },
      { 3, 6, 2 },
      { 3, 5, 6 },
      { 5, 3, 4 }  },
    { { 3, 1, 2 },      /* 12 */
      { 3, 0, 1 },
      { 4, 0, 3 },
      { 4, 6, 0 },
      { 6, 4, 5 }  },
    { { 4, 2, 3 },      /* 13 */
      { 4, 1, 2 },
      { 5, 1, 4 },
      { 5, 0, 1 },
      { 0, 5, 6 }  },
    { { 1, 6, 0 },      /* 14 */
      { 2, 6, 1 },
      { 2, 5, 6 },
      { 3, 5, 2 },
      { 5, 3, 4 }  },

    { { 2, 0, 1 },      /* 15 */
      { 3, 0, 2 },
      { 4, 0, 3 },
      { 4, 6, 0 },
      { 6, 4, 5 }  },
    { { 3, 1, 2 },      /* 16 */
      { 4, 1, 3 },
      { 5, 1, 4 },
      { 5, 0, 1 },
      { 0, 5, 6 }  },
    { { 4, 2, 3 },      /* 17 */
      { 5, 2, 4 },
      { 6, 2, 5 },
      { 6, 1, 2 },
      { 1, 6, 0 }  },
    { { 5, 3, 4 },      /* 18 */
      { 6, 3, 5 },
      { 0, 3, 6 },
      { 0, 2, 3 },
      { 2, 0, 1 }  },
    { { 3, 1, 2 },      /* 19 */
      { 4, 1, 3 },
      { 4, 0, 1 },
      { 4, 6, 0 },
      { 6, 4, 5 }  },
    { { 4, 2, 3 },      /* 20 */
      { 5, 2, 4 },
      { 5, 1, 2 },
      { 5, 0, 1 },
      { 0, 5, 6 }  },
    { { 5, 3, 4 },      /* 21 */
      { 6, 3, 5 },
      { 6, 2, 3 },
      { 6, 1, 2 },
      { 1, 6, 0 }  },

    { { 3, 1, 2 },      /* 22 */
      { 3, 0, 1 },
      { 4, 0, 3 },
      { 5, 0, 4 },
      { 0, 5, 6 }  },
    { { 4, 2, 3 },      /* 23 */
      { 4, 1, 2 },
      { 5, 1, 4 },
      { 6, 1, 5 },
      { 1, 6, 0 }  },
    { { 2, 0, 1 },      /* 24 */
      { 2, 6, 0 },
      { 2, 5, 6 },
      { 3, 5, 2 },
      { 5, 3, 4 }  },
    { { 3, 1, 2 },      /* 25 */
      { 3, 0, 1 },
      { 3, 6, 0 },
      { 4, 6, 3 },
      { 6, 4, 5 }  },
    { { 4, 2, 3 },      /* 26 */
      { 4, 1, 2 },
      { 4, 0, 1 },
      { 5, 0, 4 },
      { 0, 5, 6 }  },
    { { 1, 6, 0 },      /* 27 */
      { 1, 5, 6 },
      { 2, 5, 1 },
      { 3, 5, 2 },
      { 5, 3, 4 }  },
    { { 2, 0, 1 },      /* 28 */
      { 2, 6, 0 },
      { 3, 6, 2 },
      { 4, 6, 3 },
      { 6, 4, 5 }  },

    { { 2, 0, 1 },      /* 29 */
      { 4, 2, 3 },
      { 4, 0, 2 },
      { 5, 0, 4 },
      { 0, 5, 6 }  },
    { { 3, 1, 2 },      /* 30 */
      { 5, 3, 4 },
      { 5, 1, 3 },
      { 6, 1, 5 },
      { 1, 6, 0 }  },
    { { 4, 2, 3 },      /* 31 */
      { 6, 4, 5 },
      { 6, 2, 4 },
      { 0, 2, 6 },
      { 2, 0, 1 }  },
    { { 5, 3, 4 },      /* 32 */
      { 0, 5, 6 },
      { 0, 3, 5 },
      { 1, 3, 0 },
      { 3, 1, 2 }  },
    { { 6, 4, 5 },      /* 33 */
      { 1, 6, 0 },
      { 1, 4, 6 },
      { 2, 4, 1 },
      { 4, 2, 3 }  },
    { { 0, 5, 6 },      /* 34 */
      { 2, 0, 1 },
      { 2, 5, 0 },
      { 3, 5, 2 },
      { 5, 3, 4 }  },
    { { 1, 6, 0 },      /* 35 */
      { 3, 1, 2 },
      { 3, 6, 1 },
      { 4, 6, 3 },
      { 6, 4, 5 }  },

    { { 5, 3, 4 },      /* 36 */
      { 0, 5, 6 },
      { 0, 3, 5 },
      { 0, 2, 3 },
      { 2, 0, 1 }  },
    { { 6, 4, 5 },      /* 37 */
      { 1, 6, 0 },
      { 1, 4, 6 },
      { 1, 3, 4 },
      { 3, 1, 2 }  },
    { { 0, 5, 6 },      /* 38 */
      { 2, 0, 1 },
      { 2, 5, 0 },
      { 2, 4, 5 },
      { 4, 2, 3 }  },
    { { 1, 6, 0 },      /* 39 */
      { 3, 1, 2 },
      { 3, 6, 1 },
      { 3, 5, 6 },
      { 5, 3, 4 }  },
    { { 2, 0, 1 },      /* 40 */
      { 4, 2, 3 },
      { 4, 0, 2 },
      { 4, 6, 0 },
      { 6, 4, 5 }  },
    { { 3, 1, 2 },      /* 41 */
      { 5, 3, 4 },
      { 5, 1, 3 },
      { 5, 0, 1 },
      { 0, 5, 6 }  },
    { { 4, 2, 3 },      /* 42 */
      { 6, 4, 5 },
      { 6, 2, 4 },
      { 6, 1, 2 },
      { 1, 6, 0 }  }
};

static int CheckQualityMeasure (enum MeasureType measure, double minMeasure, 
                                double maxMeasure, double minTargetMeasure, 
                                double maxTargetMeasure, double alpha); 


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/****f*
 * NAME
 *   CheckQualityMeasure 
 * DESCRIPTION
 *   For given measure type and minimal and maximal value of that measure
 *   (or pseudo measure as dihedra angle or solid angle) check if 
 *   the values can be accepted comparing to minimal target measure and
 *   maximal target measure. Two coefficients alpha and beta allow to relax
 *   bounds set by target measures. If the minimal and maximal measure are
 *   within bounds of target measures then function returns 1.
 *   If both are outside target range function returns 0. 
 *   If one measure is within range and the second is outside but not more
 *   than alpha * (tartet measure) and is far enough from the absolute limits
 *   of the measure then function returns 1. In any other case function returns
 *   0.
 *   The limits measure cannot exces are:
 *       DIHEDRAL_MEASURE  10 degrees -- 160 degrees
 *       SOLID_MEASURE     5 degress  -- 340 degrees
 *       RADIUS_RATIO_MEASURE 0.1 -- not applicable
 *       MEAN_RATIO_MEASURE  0.4641 -- not applicable
 *       SIGMA_MEASURE  0.3162 -- not applicable
 * ARGUMENTS
 *   measure -- type of the measure
 *   minMeasure -- minimal value of measure
 *   maxMeasure -- maximal value of measure
 *   minTargetMeasure -- minimal target measure
 *   maxTargetMeasure -- maximal target measure
 *   alpha -- relaxation coefficient (0.0 : 1.0)
 * RETURN VALUE
 *   1 if minMeasure and maxMeasure are accepted 0 otherwise 
 ******/
static int CheckQualityMeasure (enum MeasureType measure, double minMeasure, 
                                double maxMeasure, double minTargetMeasure, 
                                double maxTargetMeasure, double alpha) 
{
  double minLimitMeasure;
  double maxLimitMeasure;

  /* set up measure limit */
  switch (measure)
  {
    case RADIUS_RATIO_MEASURE:
      minLimitMeasure = 0.1000;
      maxLimitMeasure = 1.0;
      break;
    case MEAN_RATIO_MEASURE:
      minLimitMeasure = 0.4641;
      maxLimitMeasure = 1.0;
      break;
    case SIGMA_MEASURE:
      minLimitMeasure = 0.3162; 
      maxLimitMeasure = 1.0;
      break;
    default:
      e_GeneralError ("CheckQualityMeasure: unknown measure type");
  }

  /* only when minimum is increased !!! */
  if (minMeasure > minTargetMeasure)
  {
    return 1;
  }
  
#if 0
  if (minMeasure > minTargetMeasure && maxMeasure < maxTargetMeasure)
  {
    return 1;
  }
  else if (minMeasure <= minTargetMeasure && maxMeasure >= maxTargetMeasure)
  {
    return 0; 
  }
  else if (minMeasure > minTargetMeasure && maxMeasure >= maxTargetMeasure)
  {
    if (   maxMeasure <= (1+alpha) * maxTargetMeasure
        && maxMeasure <= maxLimitMeasure)
    {
      return 1;
    }
  }
  else if (minMeasure <= minTargetMeasure && maxMeasure < maxTargetMeasure)
  {
    if (   minMeasure >= minLimitMeasure 
        && minMeasure >= (1-alpha) * minTargetMeasure)
    {
      return 1;
    }
  }
#endif
  
  return 0;
} /* end of CheckQualityMeasure */
 

static int CheckFlipEdges(int NConf, int NTriang, int *EquTriang,
                          enum MeasureType measure, double minTargetMeasure,
                          double maxTargetMeasure, 
                          Vertex *nodeA, Vertex *nodeB,
                          Vertex *np, Vertex *sp, int NEqPoint,
                          Tos_MStack equator, double oldVolume)
{
  Vertex *peq[7];
  Tetrah  tet;
  int     must_recover;
  int     can_recover;
  int     max_index, max_conf;
  int     i, j;
  double  minMeasure, maxMeasure, minCurrentMeasure, maxCurrentMeasure;
  int     mcheck; 
  double  alpha = 0.1;
  double  newVolume;
  
  /* do we want to recover the edge ? */
  if(nodeA != NULL && nodeB != NULL)
      must_recover = 1;
  else
      must_recover = 0;

  max_conf  = -1;
  maxMeasure = 0.0;
  max_index = -1;

  /* extracts equator vertices */
  Tos_MStackTraversalInit (equator);
  for(i = 0; i < NEqPoint; i++)
  {
    peq[i] = Tos_MStackTraverse (equator);
    assert(peq[i] != NULL);
    /* check projection of equator points,
       not really necessary, volume calculation should take care of it */
    if(eg_PointProjInsideSegment(np->coords, sp->coords, peq[i]->coords) == 0)
        return(max_conf);
  }

  /* go through all configurations */
  for(i = 0; i < NConf; i++)
  {
    can_recover = 0;
    minMeasure  = MIN_MEASURE_INIT;
    newVolume   = 0.0;
    /* go through all triangles at the equator */
    for(j = 0; j < NTriang; j++)
    {
      /* form north tetrahedron */
      tet.vertex[0] = peq[GetIJElem(EquTriang, NConf, NTriang, i, j, 0)];
      tet.vertex[1] = peq[GetIJElem(EquTriang, NConf, NTriang, i, j, 1)];
      tet.vertex[2] = peq[GetIJElem(EquTriang, NConf, NTriang, i, j, 2)];
      tet.vertex[3] = np;
      /* check that the diagonal will recover the edge,
         the triangles are provided in the list in such a way that
         each diagonal can be encountered at least once as the first
         face of a triangle,
         only checked once, since south tetrahedron has the same
         base triangle at the equator */
      if(   (   (tet.vertex[0] == nodeA)
             && (tet.vertex[1] == nodeB))
         || (   (tet.vertex[1] == nodeA)
             && (tet.vertex[0] == nodeB)) )
      {
        can_recover = 1;
      }

      newVolume += fabs(eg_Orient3D (tet.vertex[0]->coords,  
                                     tet.vertex[1]->coords, 
                                     tet.vertex[2]->coords, 
                                     tet.vertex[3]->coords));

      /* calculate quality measure of tetrahedron */
      minCurrentMeasure = maxCurrentMeasure = Tetrah_Measure(&tet, measure);
      
      /* check quality of north tetrahedron */
      mcheck = CheckQualityMeasure (measure, minCurrentMeasure, maxCurrentMeasure,
                                    minTargetMeasure, maxTargetMeasure, alpha);

      if (mcheck != 1 && must_recover == 0)
      {
        break;
      }
      if (minMeasure > minCurrentMeasure)
      {
        minMeasure = minCurrentMeasure; 
      }

      /* form south tetrahedron */
      tet.vertex[0] = peq[GetIJElem(EquTriang, NConf, NTriang, i, j, 0)];
      tet.vertex[1] = peq[GetIJElem(EquTriang, NConf, NTriang, i, j, 1)];
      tet.vertex[2] = peq[GetIJElem(EquTriang, NConf, NTriang, i, j, 2)];
      tet.vertex[3] = sp;

      newVolume += fabs(eg_Orient3D (tet.vertex[0]->coords,  
                                     tet.vertex[1]->coords, 
                                     tet.vertex[2]->coords, 
                                     tet.vertex[3]->coords));
      
      /* calculate quality measure of the south tetrahedron */
      minCurrentMeasure = maxCurrentMeasure=Tetrah_Measure (&tet, measure);
      
      /* check quality of the south tetrahedron */
      mcheck = CheckQualityMeasure(measure,
                                   minCurrentMeasure, maxCurrentMeasure,
                                   minTargetMeasure, maxTargetMeasure,
                                   alpha);
  
      if (mcheck != 1 && must_recover == 0)
      {
        break;
      }
      if (minMeasure > minCurrentMeasure)
      {
        minMeasure = minCurrentMeasure; 
      }
    }
    
    /* all formed tetrahedron has been checked */
    if(   j == NTriang
       && fabs(oldVolume-newVolume) < 1e-8)
    {
      /* edge recovery is necessary */
      if(must_recover == 1)
      {
        if(   (can_recover == 1) /* can recover the edge */
           && (maxMeasure < minMeasure) )
        {
          maxMeasure = minMeasure;
          max_conf  = i;
        }
      }
      /* only optimization, no edge recovery */
      else
      {
        if(maxMeasure < minMeasure )
        {
          maxMeasure = minMeasure;
          max_conf  = i;
        }
      }
    }
  }

  return(max_conf);
}

static int DoEdgeFlip(TetrahMesh *tetMesh, int Conf,
                      int NConf, int NTriang, int *EquTriang,
                      Vertex *np, Vertex *sp, int NEqPoint, Tos_MStack equator)
{
  Edge           dummyEdge; /* to have automatic edge variable */
  Edge          *edge; 
  Tetrah        *tet, *newtet;
  Tos_MStack     edgeShell; 
  Vertex        *peq[7];
  int            i;
  

  if(Conf < 0)
      return FALSE;
  
  /* extracts equator vertices */
  Tos_MStackTraversalInit (equator);
  for(i = 0; i < NEqPoint; i++)
  {
    peq[i] = Tos_MStackTraverse (equator);
    assert(peq[i] != NULL);
  }

  /* lets do the edge flipping */
  edge = &dummyEdge;
  EDGE_VERTEX (edge, 0) = sp; 
  EDGE_VERTEX (edge, 1) = np;

  /* allocate memory for edge shell */
  edgeShell = Tos_MStackNew ();
  /* find edge shell */
  FindEdgeShell (edge, edgeShell);
  
  /* delete the original tetrahedrons */
  Tos_MStackTraversalInit (edgeShell);
  while (Tos_MStackCount (edgeShell) > 0)
  {
    tet = (Tetrah *) Tos_MStackPop (edgeShell);
    RemoveTetFromMesh (tetMesh, tet);
    TetrahDelete (tet);
  }

  /* add the new tetrahedra */
  for(i = 0; i < NTriang; i++)
  {
    newtet = TetrahCreate ();
    TET_VERTEX (newtet, 0) = peq[GetIJElem(EquTriang, NConf, NTriang,
                                           Conf, i, 0)];
    TET_VERTEX (newtet, 1) = peq[GetIJElem(EquTriang, NConf, NTriang,
                                           Conf, i, 1)];
    TET_VERTEX (newtet, 2) = peq[GetIJElem(EquTriang, NConf, NTriang,
                                           Conf, i, 2)];
    TET_VERTEX (newtet, 3) = sp;
    AddTetToMesh (tetMesh, newtet);

    newtet = TetrahCreate ();
    TET_VERTEX (newtet, 0) = peq[GetIJElem(EquTriang, NConf, NTriang,
                                           Conf, i, 0)];
    TET_VERTEX (newtet, 1) = peq[GetIJElem(EquTriang, NConf, NTriang,
                                           Conf, i, 1)];
    TET_VERTEX (newtet, 2) = peq[GetIJElem(EquTriang, NConf, NTriang,
                                           Conf, i, 2)];
    TET_VERTEX (newtet, 3) = np;
    AddTetToMesh (tetMesh, newtet);
  }
  
  return(TRUE);
}


int BruteForceFlipEdges(TetrahMesh *tetMesh, enum MeasureType measure, 
                        double minMeasure, double maxMeasure,  
                        Vertex *nodeA, Vertex *nodeB,
                        Vertex *np, Vertex *sp, int N,
                        Tos_MStack equator)
{
  int conf;
  Edge        edge;
  Tos_MStack  edgeShell;
  Tetrah     *tet;
  double      oldVolume;

  EDGE_VERTEX (&edge, 0) = np;
  EDGE_VERTEX (&edge, 1) = sp;

  edgeShell = Tos_MStackNew ();

  /* find edge shell */
  FindEdgeShell (&edge, edgeShell);
  /* calculate the volume around the edge */
  oldVolume = 0.0;
  while(Tos_MStackCount(edgeShell))
  {
    tet = Tos_MStackPop (edgeShell);
    oldVolume += fabs(eg_Orient3D (tet->vertex[0]->coords,  
                                   tet->vertex[1]->coords, 
                                   tet->vertex[2]->coords, 
                                   tet->vertex[3]->coords));
  }
  Tos_MStackFree(&edgeShell);
  
  /* check whether flipping edges is beneficial or possible,
     then do edge flipping */
  switch(N)
  {
    case 3:
        conf = CheckFlipEdges(1, 1, &(TriangConfig_3[0][0][0]),
                              measure, minMeasure, maxMeasure, nodeA, nodeB,
                              np, sp, N, equator, oldVolume);
        return(DoEdgeFlip(tetMesh, conf, 1, 1, &(TriangConfig_3[0][0][0]),
                          np, sp, N, equator));
        break;
    case 4:
        conf = CheckFlipEdges(2, 2, &(TriangConfig_4[0][0][0]),
                              measure, minMeasure, maxMeasure, nodeA, nodeB,
                              np, sp, N, equator, oldVolume);
        return(DoEdgeFlip(tetMesh, conf, 2, 2, &(TriangConfig_4[0][0][0]),
                          np, sp, N, equator));
        break;
    case 5:
        conf = CheckFlipEdges(5, 3, &(TriangConfig_5[0][0][0]),
                              measure, minMeasure, maxMeasure, nodeA, nodeB,
                              np, sp, N, equator, oldVolume);
        return(DoEdgeFlip(tetMesh, conf, 5, 3, &(TriangConfig_5[0][0][0]),
                          np, sp, N, equator));
        break;
    case 6:
        conf = CheckFlipEdges(14, 4, &(TriangConfig_6[0][0][0]),
                              measure, minMeasure, maxMeasure, nodeA, nodeB,
                              np, sp, N, equator, oldVolume);
        return(DoEdgeFlip(tetMesh, conf, 14, 4, &(TriangConfig_6[0][0][0]),
                          np, sp, N, equator));
        break;
    case 7:
        conf = CheckFlipEdges(42, 5, &(TriangConfig_7[0][0][0]),
                              measure, minMeasure, maxMeasure, nodeA, nodeB,
                              np, sp, N, equator, oldVolume);
        return(DoEdgeFlip(tetMesh, conf, 42, 5, &(TriangConfig_7[0][0][0]),
                          np, sp, N, equator));
        break;
    default:
        return FALSE;
  }
}


/****f*
 * NAME
 *   SwapNEdgeConf
 * DESCRIPTION
 *   Tries to recover edge between 'nodeA' and 'nodeB' by swaping
 *   edges.
 * ARGUMENTS
 *   tetMesh - The pointer to the tetrahedra mesh
 *   nodeA - the start node of an edge (IN)
 *   nodeB - the end node of an edge (IN)
 *   tetA - one of the tetrahedra enclosing edge and having vertex 'nodeA' (IN)
 *   verticesA - array of indices of vertices which form edge (IN)
 * RETURN VALUE
 *   If edge was recovered returns TRUE otherwise returns FALSE
 ******/
int SwapNEdgeConf(TetrahMesh *tetMesh, Vertex *nodeA, Vertex *nodeB,
                  Tetrah *tetA, int *vertices)
{
  Edge        edge;
  Vertex     *v1, *v2;
  int         n;
  Tos_MStack  equator;
  int         flipped;

  equator = Tos_MStackNew();
  v1 = TET_VERTEX(tetA, vertices[0]); 
  v2 = TET_VERTEX(tetA, vertices[1]);

  EDGE_VERTEX (&edge, 0) = v1;
  EDGE_VERTEX (&edge, 1) = v2;

  /* create equator, collect all points on it */
  n = FindEdgeEquator (&edge, tetA, equator);
  
  /* FIXME - I have arbitrary chosen SIGMA_MEASURE to select best
   *  recovery configuration - investigate the influence of different
   *  measures 
   */
  flipped = BruteForceFlipEdges(tetMesh, SIGMA_MEASURE, 0, 1, nodeA, nodeB,
                                v1, v2, n, equator);

  Tos_MStackFlush (equator); 
  Tos_MStackFree (&equator);
  
  return(flipped);
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/****f*
 * NAME
 *   SwapEdgeForOptimization
 * DESCRIPTION
 *   Tries to swap edge in order to improve quality measure of the 
 *   edge shell.
 * ARGUMENTS
 *   tetMesh - The pointer to the tetrahedra mesh
 *   v1 - the start node of an edge (IN)
 *   v2 - the end node of an edge (IN)
 *   measure - type of the quality measure used to evaluate possible swap
 *             configurations.
 * RETURN VALUE
 *   1 if it was possible to swap edge and improve quality otherwise 0. 
 ******/
int SwapEdgeForOptimization(TetrahMesh *tetMesh, Vertex *v1, Vertex *v2,
                            enum MeasureType measure)
{
  double minMeasure, maxMeasure;
  Edge        edge;
  int         swapped = 0;
  Tos_MStack  st;
  Tetrah     *tet;
  int         n;
  
  EDGE_VERTEX (&edge, 0) = v1;
  EDGE_VERTEX (&edge, 1) = v2;

  st = Tos_MStackNew ();

  /* find edge shell and store it in 'edgeShell' */
  FindEdgeShell (&edge, st);
  
  /* find limits of the measure of tetrah in edge shell */
  FindMeasureLimits (st, measure, &minMeasure, &maxMeasure);
  tet = (Tetrah *) Tos_MStackPop (st);
  /* flush st to prepare it to use as edge equator holder */
  Tos_MStackFlush (st);
  
  n = FindEdgeEquator (&edge, tet, st);

  swapped = BruteForceFlipEdges(tetMesh, measure, minMeasure, maxMeasure, 
                                NULL, NULL, v1, v2, n, st);
  Tos_MStackFlush (st);
  Tos_MStackFree (&st);

  return (swapped); 
} /* end of SwapEdgeForOptimization */

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/****f*
 * NAME
 *   CheckEdgeSwap2To3
 * DESCRIPTION
 *   Check if it is possible to recover edge by swaping faces.
 *   Recovery by swaping is possible if the missing edge is
 *   completly enclosed in two tetrahedra adjacent by faces.
 * ARGUMENTS
 *   tetB - tetrahedron containing the start point of the edge (IN)
 *   nodeB - the end point of the edge (IN)
 *   verticesA - array of indices of 'tetA' vertices of the face
 *                intersected by missing edge (IN)
 *   tetB - pointer to the tetrahedron containing the point 'nodeB' (OUT)
 *   verticesB - indices of vertices of intersected face in tetrahedron
 *               'tetB' (OUT)
 * RETURN VALUE
 *   If it is possible to recover edge by swapping returns 1 and 
 *   sets 'tetB' to point to the tetrahderon containing 'nodeB'
 *   otherwise returns 0 and sets 'tetB' to NULL.
 *   The array 'verticesB' is filled with indices of vertices of 
 *   'tetB' which corrsponds to vertices indicated by array 'verticesA'
 ******/
static int CheckEdgeSwap2To3 (Tetrah *tetA, Vertex *nodeB, int *verticesA,
                              Tetrah **tetB, int *verticesB)
{
  Tetrah *n;
  int foundNodeB = FALSE;
  int sanityCheck = 0;
  int faceNum;
  int i;

  assert (tetA != NULL);
  assert (nodeB != NULL);

  faceNum = 6 - verticesA[0] - verticesA[1] - verticesA[2];
  *tetB = NULL;
  n = TET_NEIGHBOUR (tetA, faceNum);
  
  for (i=0; i<4; i++)
  {
    if (TET_VERTEX (n, i) == nodeB)  
    {
      foundNodeB=TRUE;
      *tetB = n;
    }
    if (TET_VERTEX (n, i) == TET_VERTEX(tetA, verticesA[0]))
    {
      sanityCheck++;
      verticesB[0] = i;
    }
    if (TET_VERTEX (n, i) == TET_VERTEX(tetA, verticesA[1]))
    {
      sanityCheck++;
      verticesB[1] = i;
    }
    if (TET_VERTEX (n, i) == TET_VERTEX(tetA, verticesA[2]))
    {
      sanityCheck++;
      verticesB[2] = i;
    }
  }
  /* check if the 'neighbour' has all vertices of the intersected face.
   * This checks if mesh is not corrupted.
   */
  if (sanityCheck != 3)
  {
    e_GeneralError 
        ("CheckEdgeSwap2To3: Invalid adjacency relations (sanityCheck=%d)", 
         sanityCheck);
  }
  return foundNodeB;
} /* end of CheckEdgeSwap2To3 */


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/****f*
 * NAME
 *   Swap2To3EdgeRecovery
 * DESCRIPTION
 *   Tries to recover edge between 'nodeA' and 'nodeB' by swaping
 *   faces. The edge recovery by swaping faces is performed only
 *   when edge is completly enclosed in two face adjacent tetrahedra.
 *   Recovery by swaping will be only performed in case of 'intersectType'
 *   is EG_INSIDE.
 * ARGUMENTS
 *   nodeA - the start node of an edge (IN)
 *   nodeB - the end node of an edge (IN)
 *   tetA - one of the tetrahedra enclosing edge and having vertex 'nodeA' (IN)
 *   intersectType - type of edge intersection (IN) 
 *   verticesA - array of indices of vertices which form face (edge or vertec) 
 *               of tetrahedron 'tetA' intersected by the edge (IN)
 * RETURN VALUE
 *   If edge was recovered returns TRUE otherwise returns FALSE
 ******/
int Swap2To3EdgeRecovery (TetrahMesh *tetMesh, Vertex *nodeA, Vertex *nodeB,
                          Tetrah *tetA, int *verticesA)
{
  int result;

  Tetrah *newTetrah[3];
  Vertex *vertices[5];
  int verticesB[3];
  Tetrah *tetB;

  result = CheckEdgeSwap2To3 (tetA, nodeB, verticesA, &tetB, verticesB);
  if (result == FALSE)
  {
    return result;
  }

  /* set vertices */
  vertices[0] = nodeA;
  vertices[1] = TET_VERTEX (tetA, verticesA[0]);
  vertices[2] = TET_VERTEX (tetA, verticesA[1]);
  vertices[3] = TET_VERTEX (tetA, verticesA[2]);
  vertices[4] = nodeB;
  /* create new tetrahedra */
  newTetrah[0] = TetrahCreate();
  newTetrah[1] = TetrahCreate();
  newTetrah[2] = TetrahCreate();
  /* set vertices of the new tetrahedra */
  TET_VERTEX(newTetrah[0], 0) = vertices[0];
  TET_VERTEX(newTetrah[0], 1) = vertices[1];
  TET_VERTEX(newTetrah[0], 2) = vertices[4];
  TET_VERTEX(newTetrah[0], 3) = vertices[3];
  /*---------------------*/
  TET_VERTEX(newTetrah[1], 0) = vertices[0];
  TET_VERTEX(newTetrah[1], 1) = vertices[4];
  TET_VERTEX(newTetrah[1], 2) = vertices[2];
  TET_VERTEX(newTetrah[1], 3) = vertices[3];
  /*---------------------*/
  TET_VERTEX(newTetrah[2], 0) = vertices[0];
  TET_VERTEX(newTetrah[2], 1) = vertices[1];
  TET_VERTEX(newTetrah[2], 2) = vertices[4];
  TET_VERTEX(newTetrah[2], 3) = vertices[2];
  
  RemoveTetFromMesh (tetMesh, tetA);
  TetrahDelete (tetA);
  RemoveTetFromMesh (tetMesh, tetB);
  TetrahDelete (tetB);
  AddTetToMesh (tetMesh, newTetrah[0]);
  AddTetToMesh (tetMesh, newTetrah[1]);
  AddTetToMesh (tetMesh, newTetrah[2]);
  
  return TRUE;
} /* end of Swap2To3EdgeRecovery */


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/****f*
 * NAME
 *   TrySwapEdgeRecovery 
 * DESCRIPTION
 *   Tries to recover edge without creating additinal points by
 *   various swapings.
 * ARGUMENTS
 *   nodeA - the start node of an edge (IN)
 *   nodeB - the end node of an edge (IN)
 *   tetA - one of the tetrahedra enclosing edge and having vertex 'nodeA' (IN)
 *   intersectionType - type of the intersection of missing edge with
 *                      other entities in the mesh.
 *   verticesA - array of indices of vertices which form part  
 *               of tetrahedron 'tetA' intersected by the edge (IN)
 * RETURN VALUE
 *   If edge was recovered returns TRUE otherwise returns FALSE
 ******/
int TrySwapEdgeRecovery (TetrahMesh *tetMesh, Vertex *nodeA, Vertex *nodeB,
                         Tetrah *tetA, int intersectionType, int *vertices)
{
  int result;

  if (intersectionType == EG_INSIDE)
  {
    result = Swap2To3EdgeRecovery (tetMesh, nodeA, nodeB, tetA, vertices);
  }
  else if (intersectionType == EG_ONEDGE)
  {
    result = SwapNEdgeConf(tetMesh, nodeA, nodeB, tetA, vertices);
  }
  else
  {
    result = FALSE;
  }
  return result;
} /* end of TrySwapEdgeRecovery */

/* Modifications:
 * $Log: flip.c,v $
 * Revision 1.15  2002/07/26 18:54:17  peteri
 * - editing of error messages
 *
 * Revision 1.14  2002/07/19 20:56:25  peteri
 * Swap for optimization only when minimum is increased
 *
 * Revision 1.13  2002/07/03 21:24:11  peteri
 * Minor formatting corrections, spell-checking
 *
 * Revision 1.12  2002/03/18 15:56:45  peteri
 * Simplification, algorithm adjustment.
 *
 * Revision 1.11  2002/03/08 13:06:38  peteri
 * The volume of the old configuration and the new configuration must
 * be equal. This checks for non-convex boundaries in al directions.
 *
 * Revision 1.10  2002/02/24 19:04:10  romek
 * *** empty log message ***
 *
 */
