
#include <assert.h>
#include "list.h"
#include "e_lib.h"

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 CreateTetList --
        Allocates memory for list structure and returns a
        pointer to it.
 Arguments:       
        line -- number of line in which this function is called
        filename -- name of file in which this function is called
 Return value:
        On succes returns pointer to memory allocated for the list structure.
        If memory has not been allocated succesfully write error message
        using 'line' and 'filename' arguments and exit (by call to
        e_GenerallError).
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define CREATE_LIST_BODY(type)                                             \
  type *list=NULL;                                                         \
  list = (type *) e_calloc (1, sizeof (type));                             \
  if (list == NULL)                                                        \
  {                                                                        \
    e_GeneralError ("Cannot allocate memory for list");                    \
  }                                                                        \
  return list;


TetList *CreateTetList ()
{
  CREATE_LIST_BODY(TetList)
}

FaceList *CreateFaceList ()
{
  CREATE_LIST_BODY(FaceList)
}

EdgeList *CreateEdgeList ()
{
  CREATE_LIST_BODY(EdgeList)
}

VertexList *CreateVertexList ()
{
  CREATE_LIST_BODY(VertexList)
}

#undef CREATE_LIST_BODY


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 Put<*>OnListHead --
        Adds new element to the list. The list is
        double linked.
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define PUT_ON_LIST_HEAD         \
  if (list->head == NULL)        \
  {                              \
    item->prev = NULL;           \
    item->next = NULL;           \
    list->head = item;           \
    list->tail = item;           \
  }                              \
  else                           \
  {                              \
    item->prev = list->head;     \
    item->next = NULL;           \
    list->head->next = item;     \
    list->head = item;           \
  }


void PutTetOnListHead(TetList *list, Tetrah *item)
{
  PUT_ON_LIST_HEAD
}

void PutFaceOnListHead(FaceList *list, Face *item)
{
  PUT_ON_LIST_HEAD
}

void PutEdgeOnListHead(EdgeList *list, Edge *item)
{
  PUT_ON_LIST_HEAD
}

void PutVertexOnListHead(VertexList *list, Vertex *item)
{
  PUT_ON_LIST_HEAD
}

#undef PUT_ON_LIST_HEAD


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 Put<*>OnListTail --
        Adds new element to the tail of the list. The list is
        double linked.
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define PUT_ON_LIST_TAIL         \
  if (list->tail == NULL)        \
  {                              \
    item->prev = NULL;           \
    item->next = NULL;           \
    list->head = item;           \
    list->tail = item;           \
  }                              \
  else                           \
  {                              \
    item->next = list->tail;     \
    item->prev = NULL;           \
    list->tail->prev = item;     \
    list->tail = item;           \
  }


void PutTetOnListTail(TetList *list, Tetrah *item)
{
  PUT_ON_LIST_TAIL
}

void PutFaceOnListTail(FaceList *list, Face *item)
{
  PUT_ON_LIST_TAIL
}

void PutEdgeOnListTail(EdgeList *list, Edge *item)
{
  PUT_ON_LIST_TAIL
}

void PutVertexOnListTail(VertexList *list, Vertex *item)
{
  PUT_ON_LIST_TAIL
}

#undef PUT_ON_LIST_TAIL


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 RemoveTetFromList --
        Removes element from the list.
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* As I have to write four identical functions I put the body of them
 * into a macro 
 */  
#define REMOVE_FROM_LIST_BODY              \
  {                                        \
    assert (list->head != NULL);           \
    assert (list->tail != NULL);           \
    /* check prev connection */            \
    if (item->prev != NULL)                \
    { /* item is not tail */               \
      (item->prev)->next = item->next;     \
    }                                      \
    else                                   \
    { /* item is tail */                   \
      list->tail = item->next;             \
    }                                      \
    /* check next connection */            \
    if (item->next == NULL)                \
    { /* item is head */                   \
      list->head = item->prev;             \
    }                                      \
    else                                   \
    { /* item is not head */               \
      (item->next)->prev = item->prev;     \
    }                                      \
                                           \
    item->prev = NULL;                     \
    item->next = NULL;                     \
  }

void RemoveTetFromList (TetList *list, Tetrah *item)
{
  REMOVE_FROM_LIST_BODY
}

void RemoveFaceFromList (FaceList *list, Face *item)
{
  REMOVE_FROM_LIST_BODY
}

void RemoveEdgeFromList (EdgeList *list, Edge *item)
{
  REMOVE_FROM_LIST_BODY
}

void RemoveVertexFromList (VertexList *list, Vertex *item)
{
  REMOVE_FROM_LIST_BODY
}

#undef REMOVE_FROM_LIST_BODY



/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 TetListTraverse --
        Return subsequent elements from the list.
        Direction can have values: TAIL_HEAD or HEAD_TAIL
 Prior to this function TetListTravarsalInit shoud be called.
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define LIST_TRAVERSE_BODY                                       \
  if (direction == HEAD_TAIL)                                    \
  {                                                              \
    if (list->nextItem != NULL)                                  \
    {                                                            \
      list->nextItem = list->nextItem->prev;                     \
    }                                                            \
  }                                                              \
  else if (direction == TAIL_HEAD)                               \
  {                                                              \
    if (list->nextItem != NULL)                                  \
    {                                                            \
      list->nextItem = list->nextItem->next;                     \
    }                                                            \
  }                                                              \
  else                                                           \
  {                                                              \
    e_GeneralError ("Unknown list traversal direction (%d)\n",   \
                   direction);                                   \
  }


Tetrah *TetListTraverse (TetList *list, int direction)
{
  Tetrah *tet;
  tet = list->nextItem;
  LIST_TRAVERSE_BODY 
  return tet;
}

Edge *EdgeListTraverse (EdgeList *list, int direction)
{
  Edge *edge;
  edge = list->nextItem;
  LIST_TRAVERSE_BODY
  return edge;
} 

Face *FaceListTraverse (FaceList *list, int direction)
{
  Face *face;
  face = list->nextItem;
  LIST_TRAVERSE_BODY
  return face;
}

Vertex *VertexListTraverse (VertexList *list, int direction)
{
  Vertex *point;
  point = list->nextItem;
  LIST_TRAVERSE_BODY
  return point;
}

#undef LIST_TRAVERSE_BODY


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 TetListTraversalInit --
        Prepare list to be traversed
        Direction can have values: TAIL_HEAD or HEAD_TAIL
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define LIST_TRAVERSAL_INIT_BODY                                  \
  if (direction == HEAD_TAIL)                                     \
  {                                                               \
    list->nextItem = list->head;                                  \
  }                                                               \
  else if (direction == TAIL_HEAD)                                \
  {                                                               \
    list->nextItem = list->tail;                                  \
  }                                                               \
  else                                                            \
  {                                                               \
    e_GeneralError ("Unknown list traversal direction (%d)\n",    \
                   direction);                                    \
  }


void TetListTraversalInit (TetList *list, int direction)
{
  LIST_TRAVERSAL_INIT_BODY 
  return;
}

void EdgeListTraversalInit (EdgeList *list, int direction)
{
  LIST_TRAVERSAL_INIT_BODY 
  return;
}

void FaceListTraversalInit (FaceList *list, int direction)
{
  LIST_TRAVERSAL_INIT_BODY 
  return;
}

void VertexListTraversalInit (VertexList *list, int direction)
{
  LIST_TRAVERSAL_INIT_BODY 
  return;
}
#undef LIST_TRAVERSAL_INIT_BODY
