/********************************************************************
 mstack.h -- 
        Tos_MStack  -- stack based on Tos_MemPool memory alocator.        

 Roman Putanowicz <romek@sect.mce.hw.ac.uk>
 SECT Research Group, Heriot-Watt University, Edinburgh    
 ********************************************************************/

/* $Id: mstack.h,v 1.4 2002/07/19 20:43:51 peteri Exp $ */
#ifndef MSTACK_H
#define MSTACK_H
    
/* Stack data structure 
 * Stack is implemented as a linked list and elements on stack
 * are void pointers
 * Memory for stack elements is allocated from memory pool specified
 * during stack initialization
 */ 
struct Tos_MStack
{
  unsigned int count;    /* number of elements on stack */
  struct elem 
  {
    void *x;
    struct elem *next;
  } *head;
  struct elem *pathItem; /* next item on stack to be traversed */
};

typedef struct Tos_MStack *Tos_MStack;

#define Tos_MStackMEMREQ (sizeof (struct elem))

void MStackMemPoolInit(void);

/* Initialize new stack */
Tos_MStack Tos_MStackNew (void);

/* Check if there are items on stack. If yes return 1, otherwise return 0 */
int   Tos_MStackEmpty (Tos_MStack stk);

/* Push element on the stack */
void  Tos_MStackPush (Tos_MStack stk, void *x);

/* Pop element from the stack */
void* Tos_MStackPop (Tos_MStack stk);

/* Free stack */ 
void  Tos_MStackFree (Tos_MStack *stk);

/* Prepare stack to be traversed */
void  Tos_MStackTraversalInit (Tos_MStack stk);

/* Traverse the stack without destroying it */
void* Tos_MStackTraverse (const Tos_MStack stk); 

/* Search for element on the stack */
int Tos_MStackSearch (Tos_MStack stk, void *x);

/* Search for number of occurences of an element on the stack */
int Tos_MStackSearchN (Tos_MStack stk, void *x);

/* Remove element from the stack */
int Tos_MStackRemove (Tos_MStack stk, void *x); 

/* Remove all elements from stack */
void Tos_MStackFlush (Tos_MStack stk);


/* Return number of elements on stack */
#define Tos_MStackCount(stk)  ((stk)->count)

/* Return the head of stack */
#define Tos_MStackHead(stk) (((stk)->head)->x)


#endif /* MSTACK_H */

/* Modifications:
 * $Log: mstack.h,v $
 * Revision 1.4  2002/07/19 20:43:51  peteri
 * Mempool removed for simplicity
 *
 * Revision 1.3  2002/02/14 10:25:11  romek
 * Added function Tos_MStackSearchN.
 *
 * Revision 1.2  2002/01/31 21:27:55  peteri
 * Major reorganization in the code.
 * - It is split into smaller modules.
 * - Some types, functions are renamed
 * - Proper flipping of edges is implemented
 *
 * Revision 1.1.1.1  2002/01/30 10:20:55  peteri
 * Imported using TkCVS
 *
 * Revision 1.3  2001/08/29 12:41:39  romek
 * *** empty log message ***
 *
 * Revision 1.2  2001/07/14 17:42:17  romek
 * last working version.
 *
 * Revision 1.4  2000/06/14 11:25:46  romek
 * (Tos_MStackSearch) : New function
 *
 * Revision 1.3  2000/02/10 14:51:02  romek
 * Macros Tos_MStackHead and Tos_MStackCount added
 *
 * Revision 1.2  2000/02/10 11:24:21  romek
 * Tos_MStackCount added
 *
 * Revision 1.1  2000/02/06 17:15:01  romek
 * Initial revision
 *
 */ 
            
