#include "basic_op.h"
#include "find.h"
#include "mesh_op.h"
#include "subdomain.h"
#include "debug.h"

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 IdentifySubdomainTetFlood --
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void IdentifySubdomainTetFlood (MeshGenData *mgds, int subdomain)
{
  Tetrah    *tet, *innerTet, *neighbour;
  int        nface;
  Tos_MStack checkStack;
  Face       face;

  innerTet = FindSubdomainTetrah (mgds, subdomain);

  checkStack = Tos_MStackNew ();
  Tos_MStackPush (checkStack, (void *) innerTet);

  while (Tos_MStackCount (checkStack) > 0)
  {
    tet = (Tetrah *) Tos_MStackPop (checkStack);
    TET_SUBDOMAIN (tet) = subdomain;
    for (nface=4; nface--;)
    {
      neighbour = TET_NEIGHBOUR (tet, nface);
      GetFaceFromTet (tet, nface, &face);
      if (   neighbour == NULL
          || TET_SUBDOMAIN(neighbour) <= subdomain
          || IsFaceOnSubdomainBoundaries (mgds, &face) == 1)
      {
        continue;
      }
      else
      {
        Tos_MStackPush (checkStack, (void *) neighbour);
      }
    }
  }

  Tos_MStackFree (&checkStack);
  return; 
} /* end of IdentifySubdomainTetFlood */



/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 IsFaceOnSubdomainBoundaries --
    Check if surface is on one of boundaries of subdomains 
 Arguments:
        face -- face under consideration 
        
 Return value:
        1 - if face is present in the surface mesh
        0 - otherwise
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
int IsFaceOnSubdomainBoundaries (MeshGenData *mgds, Face *face)
{
  int k, surfMeshNum;
  
  /* do the quick check of flags of nodes */
  if (IsFaceOnBoundary(face, POINT_BOUNDARY) == 0)
  {
    return 0;
  }
  
  /* Scan subdomain meshes */
  for (k = 0; k < mgds->mgCtrl->nsubdomains; k++)
  {
    surfMeshNum = mgds->mgCtrl->subdomainMeshes[k]-1;
    if(IsFaceOnMeshBoundary (mgds, surfMeshNum, face, POINT_BOUNDARY) == 1)
        return 1;
  }
  return 0; 
} /* end of IsFaceOnSubdomainBoundaries  */


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 RecoverSubdomains --
        Set subdomains flags in elements
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void RecoverSubdomains (MeshGenData *mgds)
{
  /*register int nface;*/
  Tetrah *tet;
  int nsubdomains, i; 

  if(vm_verbose_level >= 2)
  {
    printf("+ debug 2 -- RecoverSubdomains\n");
  }
  
  nsubdomains = mgds->mgCtrl->nsubdomains;
  for (i=1; i<=nsubdomains; i++)
  {
     IdentifySubdomainTetFlood (mgds, i);
  }
  tet = LIST_TAIL((mgds->tetMesh).tList);

  while (tet != NULL)
  {
    if (TET_SUBDOMAIN(tet) == MAX_SUBDOMAIN_NUM) 
    {
       TET_SUBDOMAIN(tet) = nsubdomains+1;
    }  
    tet = TET_NEXT(tet);  
  }

} /* end of RecoverSubdomains */



