
#ifndef TETRAH_H
#define TETRAH_H

#ifdef __cplusplus
extern "C"
{
#endif

#include "mstack.h"
#include "vertex.h"


/* This is initial setting of subdomain number in each tetrah */
#define MAX_SUBDOMAIN_NUM  99999


/* macros to access of structure members */
#define TET_NEIGHBOUR(tet, i) ((tet)->neighbour[i])
#define TET_VERTEX(tet, i)    ((tet)->vertex[i])
#define TET_VCOORDS(tet, i)   ((tet)->vertex[i]->coords)
#define TET_VNUMBER(tet, i)   ((tet)->vertex[i]->number)
#define TET_VFLAG(tet, i)     ((tet)->vertex[i]->flag)
#define TET_NUMBER(tet)       ((tet)->number)
#define TET_FLAG(tet)         ((tet)->flag)
#define TET_DOMAIN(tet)       ((tet)->domain)
#define TET_SUBDOMAIN(tet)    ((tet)->subdomain)
#define TET_NEXT(tet)         ((tet)->next)
#define TET_PREV(tet)         ((tet)->prev)

/* set the flag in tetrahedra, putting tetrahedron on specified stack */
#define TET_SET_FLAG(tet, stack, flag)        \
      TET_FLAG((tet)) |= (flag);              \
      Tos_MStackPush (stack, (void *) (tet));


/* Tetrahedron flags */
#define TET_IN_DOMAIN     (1<<0)  /* flag set in tetrah. inside domain */
#define TET_OUT_DOMAIN    (1<<1)  /* flag set in tetrah. inside domain */
#define TET_PTAS          (1<<2)  /* flag set in FindPointsTetrahAdjSet */
#define TET_VISITED       (1<<3)  /* flag set in visited tetrahedra */
#define TET_OUTCAVITY     (1<<4)  /* flag set in tetrah. outside cavity */

#define TET_FINISHED      (1<<5)  /* no further refinement is necessary */

#define TET_ROUND         (1<<21)
#define TET_NEEDLE        (1<<22)
#define TET_WEDGE         (1<<23)
#define TET_SPINDLE       (1<<24)
#define TET_SLIVER        (1<<25)
#define TET_CAP           (1<<26)
#define TET_UNKNOWN_TYPE  (1<<27)


extern const int faceDef[4][3];
extern const int edgeDef[6][2];
extern const int sideDef[3][2];


typedef struct tetrah_
{
  struct tetrah_  *neighbour[4]; /* pointers to neighbouring tetrahedra */
  Vertex          *vertex[4];    /* tetrahedron vertices */
  int              flag;
  int              domain;
  int              number;
  int              subdomain;
  unsigned int     id;        /* unique id number (used only for debuging) */
  struct tetrah_  *prev;      /* pointer to previous element in the list 
                                 (previously added) */
  struct tetrah_  *next;      /* pointer to next element in the list */
}
Tetrah;

void TetrahMemPoolInit(void);
Tetrah *TetrahCreate (void);
void TetrahDelete (Tetrah *tet);
void TetrahClearFlag (Tos_MStack stk, int flag);

#ifdef __cplusplus
}
#endif

#endif /* TETRAH_H */


