

#ifndef VERTEX_H
#define VERTEX_H

#ifdef __cplusplus
extern "C"
{
#endif

#include "mstack.h"


/* macros to access of structure members */
#define VERTEX_FLAG(v) ((v)->flag)
#define VERTEX_NEXT(v) ((v)->next)
#define VERTEX_PREV(v) ((v)->prev)
#define VERTEX_COORDS(v) ((v)->coords)
/* add new element to point adjacency stack */
#define POINT_ADD_ADJ(point, tet) \
      Tos_MStackPush ((point)->adjElem, (void *)tet)

/* delete element from point adjacency stack */
#  define POINT_REMOVE_ADJ(point, tet) \
      Tos_MStackRemove ((point)->adjElem, (void *)tet)  


/* Point flags */
#define POINT_INT_BOUNDARY (1<<0) /* flag for internal boundary points */
#define POINT_EXT_BOUNDARY (1<<1) /* flag for external boundary points */
#define POINT_BOUNDARY     (1<<2) /* flag for all boundary points */
#define POINT_ON_EDGE      (1<<3) /* flag for points recovering an edge */
#define POINT_ON_FACE      (1<<4) /* flag for points recovering a face */
#define POINT_CUBOID       (1<<5) /* flag of cuboid points */
#define POINT_CONNECTED    (1<<6) /* flag used in FindConnectedPoints */
#define POINT_SHIELD       (1<<7) /* flag points in cuboid shield  */


typedef struct vertex
{
  double         coords[3];
  double         meshParam;
  Tos_MStack     adjElem; /* stack of adjacent tetrahedra */
  int            number;
  int            flag;
  unsigned int   id;      /* unique id number (used only for debuging) */
  struct vertex *prev;    /* pointer to previous element in the list 
                           (previously added) */
  struct vertex *next;    /* pointer to next element in the list */
} Vertex; 


void    VertexMemPoolInit(void);
Vertex *VertexCreate (void);
void    VertexDelete (Vertex *point);





#ifdef __cplusplus
}
#endif

#endif /* VERTEX_H */


