/********************************************************************
 write_log.c --     
        Functions for writing log messages. 
 
 Roman Putanowicz <romek@sect.mce.hw.ac.uk>
 SECT Research Group, Heriot-Watt University Edinburgh    
 ********************************************************************/

/* $Id: write_log.c,v 1.11 2002/07/26 18:55:20 peteri Exp $ */

#include <stdio.h>
#include <time.h>
#include "e_lib.h"
#include "eg_lib.h"
#include "measures.h"
#include "mesh.h"
#include "write_log.h"

/* Some error messages uses gcc __FUNCTION__ macro which 
 * is not defined in Sun cc  
 */
#ifndef __GNUC__
#   define __FUNCTION__ __FILE__
#endif

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/****f*
 * NAME
 *   Tos_LogHeader 
 * DESCRIPTION
 * ARGUMENTS
 * RETURN VALUE
 *   NONE
 ******/
void Tos_LogHeader (MeshGenData *mgds) 
{
  time_t logtime;
  FILE *log = mgds->mgCtrl->logFile;
  if (mgds->mgCtrl->logLevel >= VM_MINIMAL_LOG)
  {
     fprintf (log, "# Log file for output mesh: %s\n", 
              mgds->mgCtrl->outputFileName);
     logtime = time(NULL);
     fprintf (log, "# Created: %s\n", ctime(&logtime));
  }
} /* end of Tos_LogHeader */

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/****f*
 * NAME
 *   Tos_LogNInputMeshes 
 * DESCRIPTION
 * ARGUMENTS
 * RETURN VALUE
 *   NONE
 ******/
void Tos_LogNInputMeshes (MeshGenData *mgds) 
{
  FILE *log = mgds->mgCtrl->logFile;
  if (mgds->mgCtrl->logLevel >= VM_MINIMAL_LOG)
  {
     fprintf (log, "#1 Number of input meshes\n\t%d\n", 
              mgds->mgCtrl->nmeshes);
  }
} /* end of Tos_LogNInputMeshes */


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/****f*
 * NAME
 *   Tos_LogNInputMeshNames 
 * DESCRIPTION
 * ARGUMENTS
 * RETURN VALUE
 *   NONE
 ******/
void Tos_LogNInputMeshNames (MeshGenData *mgds) 
{
  FILE *log = mgds->mgCtrl->logFile;
  int i;
  if (mgds->mgCtrl->logLevel >= VM_MINIMAL_LOG)
  {
     fprintf (log, "#2 Input mesh files\n");
     for (i=0; i<mgds->mgCtrl->nmeshes; i++)
     {
        fprintf (log, "\t%s.mgd\n", mgds->mgCtrl->inputFileNames[i]);
     } 
  }
} /* end of Tos_LogNInputMeshNames */

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/****f*
 * NAME
 *   Tos_LogNInputNEF 
 * DESCRIPTION
 * ARGUMENTS
 * RETURN VALUE
 *   NONE
 ******/
void Tos_LogNInputNEF (MeshGenData *mgds) 
{
  FILE *log = mgds->mgCtrl->logFile;
  int i;
  if (mgds->mgCtrl->logLevel >= VM_NORMAL_LOG)
  {
     fprintf (log, "#3 Components of input meshes\n");
     fprintf (log, "\t%s\t\t%s\t%s\t%s\n",
                    "mesh", "num. of nodes", "num. of edges", 
                    "num. of faces");
     for (i=0; i<mgds->mgCtrl->nmeshes; i++)
     {
        fprintf (log, "\t%s\t\t\t%d\t\t%d\t\t%d\n",
                 mgds->mgCtrl->inputFileNames[i],
                 mgds->pTableLength[i],
                 mgds->eTableLength[i],
                 mgds->fTableLength[i]);
     } 
  }
} /* end of Tos_LogNInputNEF */

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/****f*
 * NAME
 *   Tos_LogNTotalInputNEF 
 * DESCRIPTION
 * ARGUMENTS
 * RETURN VALUE
 *   NONE
 ******/
void Tos_LogNTotalInputNEF (MeshGenData *mgds) 
{
  FILE *log = mgds->mgCtrl->logFile;
  int i;
  int totalPoints=0, totalEdges=0, totalFaces=0;
  if (mgds->mgCtrl->logLevel >= VM_NORMAL_LOG)
  {
     fprintf (log, "#4 Total number of input components\n");
     fprintf (log, "\t%s\t%s\t%s\n",
                    "num. of nodes", "num. of edges", 
                    "num. of faces");
     for (i=0; i<mgds->mgCtrl->nmeshes; i++)
     {
        totalPoints += mgds->pTableLength[i];
        totalEdges  += mgds->eTableLength[i];
        totalFaces  += mgds->fTableLength[i];
     } 
     fprintf (log, "\t\t%d\t\t%d\t\t%d\n", totalPoints, totalEdges, totalFaces);
  }
} /* end of Tos_LogNInputNEF */


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/****f*
 * NAME
 *   Tos_LogInputMeshParam 
 * DESCRIPTION
 * ARGUMENTS
 * RETURN VALUE
 *   NONE
 ******/
void Tos_LogInputMeshParam (MeshGenData *mgds) 
{
  FILE *log = mgds->mgCtrl->logFile;
  double minMeshParam, maxMeshParam, mp;
  int i, j;
  if(mgds->mgCtrl->logLevel >= VM_FULL_LOG)
  {
    fprintf (log, "#5 Minimum and maximum of nodal mesh parameter\n");
    minMeshParam = maxMeshParam = mgds->pTable[0][0]->meshParam;
    for(i = 0; i < mgds->mgCtrl->nmeshes; i++)
    {
      for(j = 0; j < mgds->pTableLength[i]; j++)
      {
        mp =  mgds->pTable[i][j]->meshParam;
        if (minMeshParam > mp) minMeshParam = mp;
        if (maxMeshParam < mp) maxMeshParam = mp; 
      }
    } 
    fprintf (log, "\t%f\t%f\n", minMeshParam, maxMeshParam);
  }
} /* end of Tos_LogInputMeshParam */

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/****f*
 * NAME
 *   Tos_InputEdgeLengthDistriburion 
 * DESCRIPTION
 * ARGUMENTS
 * RETURN VALUE
 *   NONE
 ******/
void Tos_InputEdgeLengthDistribution (MeshGenData *mgds) 
{
  FILE   *log = mgds->mgCtrl->logFile;
  FILE   *distributionFile;
  Edge   *edge;
  double *v1, *v2;
  char   *fileName;
  char   *nameSuffix = "_intput_edge_length_dist.dis";  
  int     i, iedge;
  
  if (mgds->mgCtrl->logLevel >= VM_FULL_LOG)
  {
     fileName = (char *) e_alloc_c_vect (strlen(mgds->mgCtrl->outputFileName)
                     + strlen(nameSuffix)+ 1);
     strcpy (fileName, mgds->mgCtrl->outputFileName);
     strcat (fileName, nameSuffix);
     fprintf (log, 
      "#5.1 Distribution of length of input edges saved in file\n\t%s\n",
              fileName);
     distributionFile = fopen (fileName, "w");
     if (distributionFile == NULL)
     {
       e_GeneralError ("Cannot open file %s for writing", fileName);
     }
     fprintf (distributionFile, 
              "# Distribution of length of input edges\n");
     fprintf (distributionFile, "#\n");
     fprintf (distributionFile, "# edge length\n");
     /* for each input mesh */
     for (i = 0; i < mgds->NMeshes; i++)
     {
       for(iedge = 0; iedge < mgds->eTableLength[i]; iedge++)
       {
         edge = mgds->eTable[i][iedge];
         v1 = EDGE_VCOORDS(edge,0);
         v2 = EDGE_VCOORDS(edge,1); 
         fprintf (distributionFile, "\t%f\n",eg_PointDist3D(v1,v2));
       }
     }
     fclose (distributionFile);
     e_free (fileName);
 }
} /* end of Tos_InputEdgeLengthDistribution*/



/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/****f*
 * NAME
 *   Tos_LogBoundingBox 
 * DESCRIPTION
 * ARGUMENTS
 * RETURN VALUE
 *   NONE
 ******/
void Tos_LogBoundingBox (MeshGenData *mgds, double *low, double *high) 
{
  FILE *log = mgds->mgCtrl->logFile;
  if (mgds->mgCtrl->logLevel >= VM_NORMAL_LOG)
  {
     fprintf (log, "#6 Input meshes bounding box\n");
     fprintf (log, "\t%f\t%f\t%f\n", low[0], low[1], low[2]);
     fprintf (log, "\t%f\t%f\t%f\n", high[0], high[1], high[2]);
  }
} /* end of Tos_LogBoundingBox */

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/****f*
 * NAME
 *   Tos_LogNTetEmptyTriang 
 * DESCRIPTION
 * ARGUMENTS
 * RETURN VALUE
 *   NONE
 ******/
void Tos_LogNTetEmptyTriang (MeshGenData *mgds) 
{
  FILE *log = mgds->mgCtrl->logFile;
  if (mgds->mgCtrl->logLevel >= VM_NORMAL_LOG)
  {
     fprintf (log, "#7 Number of tetrahedra in `empty' triangulation\n");
     fprintf (log, "\t%d\n", mgds->tetMesh.NTetrah);
  }
} /* end of Tos_LogNTetEmptyTriang */


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/****f*
 * NAME
 *   Tos_Log 
 * DESCRIPTION
 * ARGUMENTS
 * RETURN VALUE
 *   NONE
 ******/
void Tos_LogShapeEmptyTriang (MeshGenData *mgds) 
{
  FILE *log = mgds->mgCtrl->logFile;
  FILE *shapeFile;
  Tetrah *tet;
  double sigma, eta, rho;
  char *fileName;
  char *nameSuffix = "_empty_shape.shp";
  if (mgds->mgCtrl->logLevel >= VM_FULL_LOG)
  {
     fileName = (char *) e_alloc_c_vect (strlen(mgds->mgCtrl->outputFileName)
                     + strlen(nameSuffix)+ 1);
     strcpy (fileName, mgds->mgCtrl->outputFileName);
     strcat (fileName, nameSuffix);
     fprintf (log, 
              "#8 Shape measures of `empty' triangulation saved in file\n\t%s\n",
              fileName);
     shapeFile = fopen (fileName, "w");
     if (shapeFile == NULL)
     {
       e_GeneralError ("Cannot open file %s for writing", fileName);
     }
     tet = LIST_TAIL(mgds->tetMesh.tList);

     fprintf (shapeFile, "# Shape measure of `empty' triangulation\n");
     fprintf (shapeFile, "# of final mesh from `%s'\n",  
                         mgds->mgCtrl->outputFileName);
     fprintf (shapeFile, "# Min. solid angle | Radius ratio  | Mean Ratio\n");
     while (tet != NULL)
     {
       int dummy;
       sigma = Tetrah_MinSigmaMeasure (tet, &dummy);
       rho = Tetrah_RadiusRatio (tet);
       eta = Tetrah_MeanRatio (tet);
       fprintf (shapeFile, "\t%f\t%f\t%f\n", sigma, rho, eta);
       tet = TET_NEXT(tet);  
     }  
     fclose (shapeFile);
     e_free (fileName);
  }
} /* end of Tos_LogShapeEmpytTriang */


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/****f*
 * NAME
 *   Tos_LogShapeCoreEmpytTriang  
 * DESCRIPTION
 * ARGUMENTS
 * RETURN VALUE
 *   NONE
 ******/
void Tos_LogShapeCoreEmptyTriang (MeshGenData *mgds) 
{
  FILE *log = mgds->mgCtrl->logFile;
  FILE *shapeFile;
  Tetrah *tet;
  double sigma, eta, rho;
  char *fileName;
  char *nameSuffix = "_core_empty_shape.shp";
  int i, skip;
  if (mgds->mgCtrl->logLevel >= VM_FULL_LOG)
  {
     fileName = (char *) e_alloc_c_vect (strlen(mgds->mgCtrl->outputFileName)
                     + strlen(nameSuffix)+ 1);
     strcpy (fileName, mgds->mgCtrl->outputFileName);
     strcat (fileName, nameSuffix);
     fprintf (log, 
      "#9 Shape measures of core of `empty' triangulation saved in file\n\t%s\n",
              fileName);
     shapeFile = fopen (fileName, "w");
     if (shapeFile == NULL)
     {
       e_GeneralError ("Cannot open file %s for writing", fileName);
     }
     tet = LIST_TAIL(mgds->tetMesh.tList);

     fprintf (shapeFile, "# Shape measure of core of `empty' triangulation\n");
     fprintf (shapeFile, "# of final mesh from `%s'\n",  
                         mgds->mgCtrl->outputFileName);
     fprintf (shapeFile, "# Min. solid angle | Radius ratio  | Mean Ratio\n");
     while (tet != NULL)
     {
       skip = 0;
       for (i=0; i<4; i++)
       {
         if ((TET_VFLAG(tet,i) & POINT_CUBOID) == POINT_CUBOID ||
             (TET_VFLAG(tet,i) & POINT_SHIELD) == POINT_SHIELD)
         {
           skip = 1;
           break;
         }
       }
       if (skip != 1)
       {
         int dummy;
         sigma = Tetrah_MinSigmaMeasure (tet, &dummy);
         rho = Tetrah_RadiusRatio (tet);
         eta = Tetrah_MeanRatio (tet);
         fprintf (shapeFile, "\t%f\t%f\t%f\n", sigma, rho, eta);
       }
       tet = TET_NEXT(tet);  
     }  
     fclose (shapeFile);
     e_free (fileName);
  }
} /* end of Tos_LogShapeCoreEmpytTriang */

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/****f*
 * NAME
 *   Tos_LogNTetCoreEmptyTriang 
 * DESCRIPTION
 * ARGUMENTS
 * RETURN VALUE
 *   NONE
 ******/
void Tos_LogNTetCoreEmptyTriang (MeshGenData *mgds) 
{
  FILE *log = mgds->mgCtrl->logFile;
  Tetrah *tet;
  int counter;
  int i, skip;
  if (mgds->mgCtrl->logLevel >= VM_FULL_LOG)
  {
     tet = LIST_TAIL(mgds->tetMesh.tList);
     counter = 0;
     while (tet != NULL)
     {
       skip = 0;
       for (i=0; i<4; i++)
       {
         if ((TET_VFLAG(tet,i) & POINT_CUBOID) == POINT_CUBOID ||
             (TET_VFLAG(tet,i) & POINT_SHIELD) == POINT_SHIELD)
         {
           skip = 1;
           break;
         }
       }
       if (skip != 1)
       {
         counter++;
       }
       tet = TET_NEXT(tet);  
     }  

     fprintf (log, 
       "#10 Number of tetrahedra in core of `empty' triangulation\n");
     fprintf (log, "\t%d\n", counter);
  }
} /* end of Tos_LogNTetCoreEmptyTriang */

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/****f*
 * NAME
 *   Tos_LogNPointsVolumeTriang 
 * DESCRIPTION
 * ARGUMENTS
 * RETURN VALUE
 *   NONE
 ******/
void Tos_LogNPointsVolumeTriang (MeshGenData *mgds, int counter) 
{
  FILE *log = mgds->mgCtrl->logFile;
  if (mgds->mgCtrl->logLevel >= VM_MINIMAL_LOG)
  {
    fprintf (log, 
      "#11 Number of points added during refinement phase\n");
    fprintf (log, "\t%d\n", counter);
  }
} /* end of Tos_LogNPointsVolumeTriang */

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/****f*
 * NAME
 *   Tos_LogNTetVolumeTriang 
 * DESCRIPTION
 * ARGUMENTS
 * RETURN VALUE
 *   NONE
 ******/
void Tos_LogNTetVolumeTriang (MeshGenData *mgds) 
{
  FILE *log = mgds->mgCtrl->logFile;
  if (mgds->mgCtrl->logLevel >= VM_NORMAL_LOG)
  {
     fprintf (log, "#12 Number of tetrahedra after triangulation refinement\n");
     fprintf (log, "\t%d\n", mgds->tetMesh.NTetrah);
  }
} /* end of Tos_LogNTetVolumeTriang */

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/****f*
 * NAME
 *   Tos_LogNTetCoreVolumeTriang 
 * DESCRIPTION
 * ARGUMENTS
 * RETURN VALUE
 *   NONE
 ******/
void Tos_LogNTetCoreVolumeTriang (MeshGenData *mgds) 
{
  FILE *log = mgds->mgCtrl->logFile;
  Tetrah *tet;
  int counter;
  int i, skip;
  if (mgds->mgCtrl->logLevel >= VM_FULL_LOG)
  {
     tet = LIST_TAIL(mgds->tetMesh.tList);
     counter = 0;
     while (tet != NULL)
     {
       skip = 0;
       for (i=0; i<4; i++)
       {
         if ((TET_VFLAG(tet,i) & POINT_CUBOID) == POINT_CUBOID ||
             (TET_VFLAG(tet,i) & POINT_SHIELD) == POINT_SHIELD)
         {
           skip = 1;
           break;
         }
       }
       if (skip != 1)
       {
         counter++;
       }
       tet = TET_NEXT(tet);  
     }  

     fprintf (log, 
       "#13 Number of tetrahedra in core after triangulation refinement\n");
     fprintf (log, "\t%d\n", counter);
  }
} /* end of Tos_LogNTetCoreVolumeTriang */


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/****f*
 * NAME
 *   Tos_LogShapeVolumeTriang 
 * DESCRIPTION
 * ARGUMENTS
 * RETURN VALUE
 *   NONE
 ******/
void Tos_LogShapeVolumeTriang (MeshGenData *mgds) 
{
  FILE *log = mgds->mgCtrl->logFile;
  FILE *shapeFile;
  Tetrah *tet;
  double sigma, eta, rho;
  char *fileName;
  char *nameSuffix = "_volume_shape.shp";
  if (mgds->mgCtrl->logLevel >= VM_FULL_LOG)
  {
     fileName = (char *) e_alloc_c_vect (strlen(mgds->mgCtrl->outputFileName)
                     + strlen(nameSuffix)+ 1);
     strcpy (fileName, mgds->mgCtrl->outputFileName);
     strcat (fileName, nameSuffix);
     fprintf (log, 
        "#14 Shape measures after triangulation refinement saved in file\n\t%s\n",
              fileName);
     shapeFile = fopen (fileName, "w");
     if (shapeFile == NULL)
     {
       e_GeneralError ("Cannot open file %s for writing", fileName);
     }
     tet = LIST_TAIL(mgds->tetMesh.tList);

     fprintf (shapeFile, "# Shape measure of refined triangulation\n");
     fprintf (shapeFile, "# of final mesh from `%s'\n",  
                         mgds->mgCtrl->outputFileName);
     fprintf (shapeFile, "# Min. solid angle | Radius ratio  | Mean Ratio\n");
     while (tet != NULL)
     {
       int dummy; 
       sigma = Tetrah_MinSigmaMeasure (tet, &dummy);
       rho = Tetrah_RadiusRatio (tet);
       eta = Tetrah_MeanRatio (tet);
       fprintf (shapeFile, "\t%f\t%f\t%f\n", sigma, rho, eta);
       tet = TET_NEXT(tet);  
     }  
     fclose (shapeFile);
     e_free (fileName);
  }
} /* end of Tos_LogShapeVolumeTriang */


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/****f*
 * NAME
 *   Tos_LogShapeCoreVolumeTriang  
 * DESCRIPTION
 * ARGUMENTS
 * RETURN VALUE
 *   NONE
 ******/
void Tos_LogShapeCoreVolumeTriang (MeshGenData *mgds) 
{
  FILE *log = mgds->mgCtrl->logFile;
  FILE *shapeFile;
  Tetrah *tet;
  double sigma, eta, rho;
  char *fileName;
  char *nameSuffix = "_core_volume_shape.shp";  
  int i, skip;
  if (mgds->mgCtrl->logLevel >= VM_FULL_LOG)
  {
     fileName = (char *) e_alloc_c_vect (strlen(mgds->mgCtrl->outputFileName)
                     + strlen(nameSuffix)+ 1);
     strcpy (fileName, mgds->mgCtrl->outputFileName);
     strcat (fileName, nameSuffix);
     fprintf (log, 
     "#15 Shape measures of core of refinded triangulation saved in file\n\t%s\n",
              fileName);
     shapeFile = fopen (fileName, "w");
     if (shapeFile == NULL)
     {
       e_GeneralError ("Cannot open file %s for writing", fileName);
     }
     tet = LIST_TAIL(mgds->tetMesh.tList);

     fprintf (shapeFile, "# Shape measure of core of refined triangulation\n");
     fprintf (shapeFile, "# of final mesh from `%s'\n",  
                         mgds->mgCtrl->outputFileName);
     fprintf (shapeFile, "# Min. solid angle | Radius ratio  | Mean Ratio\n");
     while (tet != NULL)
     {
       skip = 0;
       for (i=0; i<4; i++)
       {
         if ((TET_VFLAG(tet,i) & POINT_CUBOID) == POINT_CUBOID ||
             (TET_VFLAG(tet,i) & POINT_SHIELD) == POINT_SHIELD)
         {
           skip = 1;
           break;
         }
       }
       if (skip != 1)
       {
         int dummy;
         sigma = Tetrah_MinSigmaMeasure (tet, &dummy);
         rho = Tetrah_RadiusRatio (tet);
         eta = Tetrah_MeanRatio (tet);
         fprintf (shapeFile, "\t%f\t%f\t%f\n", sigma, rho, eta);
       }
       tet = TET_NEXT(tet);  
     }  
     fclose (shapeFile);
     e_free (fileName);
  }
} /* end of Tos_LogShapeCoreVolumeTriang */


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/****f*
 * NAME
 *   Tos_LogNMissingEdges 
 * DESCRIPTION
 * ARGUMENTS
 * RETURN VALUE
 *   NONE
 ******/
void Tos_LogNMissingEdges (MeshGenData *mgds, int counter) 
{
  FILE *log = mgds->mgCtrl->logFile;
  if (mgds->mgCtrl->logLevel >= VM_NORMAL_LOG)
  {
    fprintf (log, 
      "#16 Number of missing edges\n");
    fprintf (log, "\t%d\n", counter);
  }
} /* end of Tos_LogNMissingEdges */


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/****f*
 * NAME
 *   Tos_LogSwap2To3EdgeRecovery 
 * DESCRIPTION
 * ARGUMENTS
 * RETURN VALUE
 *   NONE
 ******/
void Tos_LogSwap2To3EdgeRecovery (MeshGenData *mgds, int counter) 
{
  FILE *log = mgds->mgCtrl->logFile;
  if (mgds->mgCtrl->logLevel >= VM_FULL_LOG)
  {
    fprintf (log, 
      "#17 Number of edges recovered by Swap2To3\n");
    fprintf (log, "\t%d\n", counter);
  }
} /* end of Tos_LogSwap2To3EdgeRecovery*/

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/****f*
 * NAME
 *   Tos_LogSwap3To4EdgeRecovery 
 * DESCRIPTION
 * ARGUMENTS
 * RETURN VALUE
 *   NONE
 ******/
void Tos_LogSwap3To4EdgeRecovery (MeshGenData *mgds, int counter) 
{
  FILE *log = mgds->mgCtrl->logFile;
  if (mgds->mgCtrl->logLevel >= VM_FULL_LOG)
  {
    fprintf (log, 
      "#18 Number of edges recovered by Swap3To4\n");
    fprintf (log, "\t%d\n", counter);
  }
} /* end of Tos_LogSwap3To4EdgeRecovery*/

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/****f*
 * NAME
 *   Tos_LogCollapseEdgeRecovery 
 * DESCRIPTION
 * ARGUMENTS
 * RETURN VALUE
 *   NONE
 ******/
void Tos_LogCollapseEdgeRecovery (MeshGenData *mgds, int counter) 
{
  FILE *log = mgds->mgCtrl->logFile;
  if (mgds->mgCtrl->logLevel >= VM_FULL_LOG)
  {
    fprintf (log, 
      "#19 Number of edges recovered by Collapse\n");
    fprintf (log, "\t%d\n", counter);
  }
} /* end of Tos_LogCollapseEdgeRecovery*/

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/****f*
 * NAME
 *   Tos_LogSplitEdgeRecovery 
 * DESCRIPTION
 * ARGUMENTS
 * RETURN VALUE
 *   NONE
 ******/
void Tos_LogSplitEdgeRecovery (MeshGenData *mgds, int counter) 
{
  FILE *log = mgds->mgCtrl->logFile;
  if (mgds->mgCtrl->logLevel >= VM_FULL_LOG)
  {
    fprintf (log, 
      "#20 Number of edges recovered by Split\n");
    fprintf (log, "\t%d\n", counter);
  }
} /* end of Tos_LogSplitEdgeRecovery*/


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/****f*
 * NAME
 *   Tos_LogSplitPointsDistriburion 
 * DESCRIPTION
 * ARGUMENTS
 * RETURN VALUE
 *   NONE
 ******/
void Tos_LogSplitPointsDistribution (MeshGenData *mgds) 
{
  FILE *log = mgds->mgCtrl->logFile;
  FILE *distributionFile;
  Edge *edge;
  char *fileName;
  char *nameSuffix = "_edge_split_dist.dis";  
  int   i, iedge;
  
  if (mgds->mgCtrl->logLevel >= VM_FULL_LOG)
  {
     fileName = (char *) e_alloc_c_vect (strlen(mgds->mgCtrl->outputFileName)
                 + strlen(nameSuffix)+ 1);
     strcpy (fileName, mgds->mgCtrl->outputFileName);
     strcat (fileName, nameSuffix);
     fprintf (log, 
              "#21 Distribution of edge split points saved in file\n\t%s\n",
              fileName);
     distributionFile = fopen (fileName, "w");
     if (distributionFile == NULL)
     {
       e_GeneralError ("Cannot open file %s for writing", fileName);
     }
     fprintf (distributionFile, 
              "# Distribution of edge split points\n");
     fprintf (distributionFile, 
              "#\n# split points\n");
     /* for each input mesh */
     for (i=0; i<mgds->NMeshes; i++)
     {
       for(iedge = 0; iedge < mgds->eTableLength[i]; iedge++)
       {
         edge = mgds->eTable[i][iedge];
         fprintf (distributionFile, "\t%d\n",
                  Tos_MStackCount(edge->splitEdges));
       }
     }
     fclose (distributionFile);
     e_free (fileName);
 }
} /* end of Tos_LogSplitPointsDistribution*/

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/****f*
 * NAME
 *   Tos_LogNMissingFaces 
 * DESCRIPTION
 * ARGUMENTS
 * RETURN VALUE
 *   NONE
 ******/
void Tos_LogNMissingFaces (MeshGenData *mgds, int counter) 
{
  FILE *log = mgds->mgCtrl->logFile;
  if (mgds->mgCtrl->logLevel >= VM_NORMAL_LOG)
  {
    fprintf (log, 
      "#22 Number of missing faces\n");
    fprintf (log, "\t%d\n", counter);
  }
} /* end of Tos_LogNMissingFaces */


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/****f*
 * NAME
 *   Tos_LogMaxFaceSplit 
 * DESCRIPTION
 * ARGUMENTS
 * RETURN VALUE
 *   NONE
 ******/
void Tos_LogMaxFaceSplit (MeshGenData *mgds, int counter) 
{
  FILE *log = mgds->mgCtrl->logFile;
  if (mgds->mgCtrl->logLevel >= VM_NORMAL_LOG)
  {
    fprintf (log, 
      "#23 Maximal face split\n");
    fprintf (log, "\t%d\n", counter);
  }
} /* end of Tos_LogMaxFaceSplit */



/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/****f*
 * NAME
 *   Tos_LogUnsplitedEdges 
 * DESCRIPTION
 * ARGUMENTS
 * RETURN VALUE
 *   NONE
 ******/
void Tos_LogUnsplitedEdges (MeshGenData *mgds, int counter) 
{
  FILE *log = mgds->mgCtrl->logFile;
  if (mgds->mgCtrl->logLevel >= VM_FULL_LOG)
  {
    fprintf (log, 
      "#24 Number of successfully unsplited edges\n");
    fprintf (log, "\t%d\n", counter);
  }
} /* end of Tos_LogUnsplitedEdges */

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/****f*
 * NAME
 *   Tos_LogUnsplitedFaces 
 * DESCRIPTION
 * ARGUMENTS
 * RETURN VALUE
 *   NONE
 ******/
void Tos_LogUnsplitedFaces (MeshGenData *mgds, int counter) 
{
  FILE *log = mgds->mgCtrl->logFile;
  if (mgds->mgCtrl->logLevel >= VM_FULL_LOG)
  {
    fprintf (log, 
      "#25 Number of sucessfully unsplited faces\n");
    fprintf (log, "\t%d\n", counter);
  }
} /* end of Tos_LogUnsplitedFaces */

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/****f*
 * NAME
 *   Tos_LogSplitedFaces 
 * DESCRIPTION
 * ARGUMENTS
 * RETURN VALUE
 *   NONE
 ******/
void Tos_LogSplitedFaces (MeshGenData *mgds) 
{
  FILE *log = mgds->mgCtrl->logFile;
  int counter=0;
  int i, meshNum;
  Face *face;
  if (mgds->mgCtrl->logLevel >= VM_FULL_LOG)
  {
    for (meshNum = 0; meshNum < mgds->NMeshes; meshNum++)
    {
      for (i=0; i<mgds->fTableLength[meshNum]; i++)
      {
        face = mgds->fTable[meshNum][i];
        if(Tos_MStackCount(face->splitFaces) > 0)
            counter += Tos_MStackCount(face->splitFaces);
        else
            counter++;
      }
    }   
    fprintf (log, 
      "#25.1 Number of faces with split points on them\n");
    fprintf (log, "\t%d\n", counter);
  }
} /* end of Tos_LogSplitedFaces */


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/****f*
 * NAME
 *   Tos_LogNTotalTetrah 
 * DESCRIPTION
 * ARGUMENTS
 * RETURN VALUE
 *   NONE
 ******/
void Tos_LogNTotalTetrah (MeshGenData *mgds) 
{
  FILE *log = mgds->mgCtrl->logFile;
  if (mgds->mgCtrl->logLevel >= VM_MINIMAL_LOG)
  {
    fprintf (log, 
      "#26 Total number of tetrahedra in mesh\n");
    fprintf (log, "\t%d\n", mgds->tetMesh.NTetrah);
  }
} /* end of Tos_LogNTotalTetrah */

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/****f*
 * NAME
 *   Tos_LogNTotalMeshPoints 
 * DESCRIPTION
 * ARGUMENTS
 * RETURN VALUE
 *   NONE
 ******/
void Tos_LogNTotalMeshPoints (MeshGenData *mgds) 
{
  FILE *log = mgds->mgCtrl->logFile;
  if (mgds->mgCtrl->logLevel >= VM_MINIMAL_LOG)
  {
    fprintf (log, 
      "#27 Total number of mesh points in mesh\n");
    fprintf (log, "\t%d\n", mgds->tetMesh.NMeshPoints);
  }
} /* end of Tos_LogNTotalMeshPoints */

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/****f*
 * NAME
 *   Tos_LogNAddedPointsMeshPoints 
 * DESCRIPTION
 * ARGUMENTS
 * RETURN VALUE
 *   NONE
 ******/
void Tos_LogNAddedMeshPoints (MeshGenData *mgds) 
{
  FILE *log = mgds->mgCtrl->logFile;
  Vertex *vertex;
  int counterEdgePoints=0;
  int counterFacePoints=0;
  int counterBoundaryPoints=0;
  int i=0; 
  if (mgds->mgCtrl->logLevel >= VM_NORMAL_LOG)
  {
    vertex = LIST_TAIL(mgds->tetMesh.pList);
    while (vertex != NULL)
    {
       i++;
       if ((VERTEX_FLAG(vertex) & POINT_ON_EDGE) == POINT_ON_EDGE)
       {
          counterEdgePoints++;
       }
       if ((VERTEX_FLAG(vertex) & POINT_ON_FACE) == POINT_ON_FACE)
       {
          counterFacePoints++;
       }
       if ((VERTEX_FLAG(vertex) & POINT_BOUNDARY) == POINT_BOUNDARY)
       {
          counterBoundaryPoints++;
       }
       vertex = VERTEX_NEXT(vertex);  
    }
    fprintf (log, 
      "#27.1 Total number of boundary mesh points\n");
    fprintf (log, "\t%d\n", counterBoundaryPoints);

    fprintf (log, 
      "#27.2 Total number of mesh points added on edges\n");
    fprintf (log, "\t%d\n", counterEdgePoints);

    fprintf (log, 
      "#27.3 Total number of mesh points added on faces\n");
    fprintf (log, "\t%d\n", counterFacePoints);
  }
} /* end of Tos_LogNAddedMeshPoints */


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/****f*
 * NAME
 *   Tos_LogShapeFinalTriang  
 * DESCRIPTION
 * ARGUMENTS
 * RETURN VALUE
 *   NONE
 ******/
void Tos_LogShapeFinalTriang (MeshGenData *mgds) 
{
  FILE *log = mgds->mgCtrl->logFile;
  FILE *shapeFile;
  Tetrah *tet;
  double sigma, eta, rho;
  char *fileName;
  char *nameSuffix = "_final_shape.shp";  
  int i, skip;
  if (mgds->mgCtrl->logLevel >= VM_NORMAL_LOG)
  {
     ClassifyTetrah (&(mgds->tetMesh));
    
     fileName = (char *) e_alloc_c_vect (strlen(mgds->mgCtrl->outputFileName)
                     + strlen(nameSuffix)+ 1);
     strcpy (fileName, mgds->mgCtrl->outputFileName);
     strcat (fileName, nameSuffix);
     fprintf (log, 
      "#28 Shape measures of final triangulation saved in file\n\t%s\n",
              fileName);
     shapeFile = fopen (fileName, "w");
     if (shapeFile == NULL)
     {
       e_GeneralError ("Cannot open file %s for writing", fileName);
     }
     tet = LIST_TAIL(mgds->tetMesh.tList);

     fprintf (shapeFile, "# Shape measure of final triangulation\n");
     fprintf (shapeFile, "# of final mesh from `%s'\n",  
                         mgds->mgCtrl->outputFileName);
     fprintf (shapeFile, "# Min. solid angle | Radius ratio  | Mean Ratio\n");
     while (tet != NULL)
     {
       skip = 0;
       for (i=0; i<4; i++)
       {
         if ((TET_VFLAG(tet,i) & POINT_CUBOID) == POINT_CUBOID ||
             (TET_VFLAG(tet,i) & POINT_SHIELD) == POINT_SHIELD    )
         {
           skip = 1;
           break;
         }
       }
       if (skip != 1)
       {
         int dummy;
         sigma = Tetrah_MinSigmaMeasure (tet, &dummy);
         rho = Tetrah_RadiusRatio (tet);
         eta = Tetrah_MeanRatio (tet);
         fprintf (shapeFile, "\t%f\t%f\t%f\n", sigma, rho, eta);
       }
       tet = TET_NEXT(tet);  
     }  
     fclose (shapeFile);
     e_free (fileName);
  }
} /* end of Tos_LogShapeFinalTriang */

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/****f*
 * NAME
 *   Tos_LogFinalVolumeDistriburion 
 * DESCRIPTION
 * ARGUMENTS
 * RETURN VALUE
 *   NONE
 ******/
void Tos_LogFinalVolumeDistribution (MeshGenData *mgds) 
{
  FILE *log = mgds->mgCtrl->logFile;
  FILE *distributionFile;
  Tetrah *tet;
  char *fileName;
  char *nameSuffix = "_volume_dist.dis";  
  double volume;
  double *vcoords[4];

  if (mgds->mgCtrl->logLevel >= VM_FULL_LOG)
  {
     fileName = (char *) e_alloc_c_vect (strlen(mgds->mgCtrl->outputFileName)
                     + strlen(nameSuffix)+ 1);
     strcpy (fileName, mgds->mgCtrl->outputFileName);
     strcat (fileName, nameSuffix);
     fprintf (log, 
      "#29 Distribution of elements volume saved in file\n\t%s\n",
              fileName);
     distributionFile = fopen (fileName, "w");
     if (distributionFile == NULL)
     {
       e_GeneralError ("Cannot open file %s for writing", fileName);
     }
     tet = LIST_TAIL(mgds->tetMesh.tList);

     fprintf (distributionFile, 
              "# Distribution of element volume\n");
     fprintf (distributionFile, 
              "#\n# element volume\n");

     while (tet != NULL)
     {
        vcoords[0] = TET_VCOORDS(tet,0);
        vcoords[1] = TET_VCOORDS(tet,1);
        vcoords[2] = TET_VCOORDS(tet,2);
        vcoords[3] = TET_VCOORDS(tet,3);
        volume = eg_TetrahVolume (vcoords[0], vcoords[1], 
                                  vcoords[2], vcoords[3]);

       fprintf (distributionFile, "\t%f\n", volume);
       tet = TET_NEXT(tet);  
     }
     fclose (distributionFile);
     e_free (fileName);
 }
} /* end of Tos_LogFinalVolumeDistribution */





/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/****f*
 * NAME
 *   Tos_LogDihedralAngle
 * DESCRIPTION
 * ARGUMENTS
 * RETURN VALUE
 *   NONE
 ******/
void Tos_LogDihedralAngle(MeshGenData *mgds) 
{
  FILE   *log = mgds->mgCtrl->logFile;
  FILE   *shapeFile;
  Tetrah *tet;
  double  angle;
  char   *fileName;
  char   *nameSuffix = "_dihedral_dist.dis";  
  int     i, skip;
  
  if (mgds->mgCtrl->logLevel >= VM_NORMAL_LOG)
  {
    fileName = (char *)
        e_alloc_c_vect (strlen(mgds->mgCtrl->outputFileName) +
                        strlen(nameSuffix) + 1);
    strcpy (fileName, mgds->mgCtrl->outputFileName);
    strcat (fileName, nameSuffix);
    fprintf (log, 
             "#30 Distribution of dihedral angles of final triangulation "
             "saved in file\n\t%s\n",
              fileName);
    
    shapeFile = fopen (fileName, "w");
    if (shapeFile == NULL)
    {
      e_GeneralError ("Cannot open file %s for writing", fileName);
    }
    tet = LIST_TAIL(mgds->tetMesh.tList);

    fprintf (shapeFile, "# Distribution of dihedral angles\n");
    fprintf (shapeFile, "# of final mesh from `%s'\n",
             mgds->mgCtrl->outputFileName);
    fprintf (shapeFile, "# Dihedral angle \n");

    while (tet != NULL)
    {
      skip = 0;
      for (i=0; i<4; i++)
      {
        if ((TET_VFLAG(tet,i) & POINT_CUBOID) == POINT_CUBOID ||
            (TET_VFLAG(tet,i) & POINT_SHIELD) == POINT_SHIELD    )
        {
          skip = 1;
          break;
        }
      }
      if (skip != 1)
      {
        for(i = 0; i < 6; i++)
        {
          angle = Tetrah_DihedralAngle(tet, i);
          fprintf (shapeFile, "\t%f\n", angle * 180 / M_PI);
        }
      }
      tet = TET_NEXT(tet);  
    }
    
    fclose (shapeFile);
    e_free (fileName);
  }
} /* end of Tos_LogDihedralAngle */





/* Modifications:
 * $Log: write_log.c,v $
 * Revision 1.11  2002/07/26 18:55:20  peteri
 * - minor editing
 * - adding a function to print the distribution of dihedral angles
 *
 * Revision 1.10  2002/07/26 14:59:49  peteri
 * - mesh data is separated from boundary data
 * - now it is possible to setup the boundary data separately
 *   and refine that boundary (without having a MESH)
 *
 * Revision 1.9  2002/07/26 11:54:39  peteri
 * - split edges are printed correctly (?)
 *
 * Revision 1.8  2002/07/19 20:57:14  peteri
 * Proper quality is written out (with slivers) !!!
 *
 * Revision 1.7  2002/02/24 19:01:20  romek
 * *** empty log message ***
 *
 * Revision 1.6  2002/02/02 17:18:39  peteri
 * - edge list is transformed into edge table
 * - related code is updated
 * - kernel is simplify a little bit
 * - minor editing and formatting
 *
 * Revision 1.5  2002/02/02 15:19:31  peteri
 * - addedfaceslist member from MeshGenData is removed
 * - most of the functions are updated
 * - sup_domain member is deleted from vertex, edge, face, tetrah structures
 * - some functions are merged, we have less code
 * - some other minor changes, editing
 *
 * Revision 1.4  2002/02/01 15:39:58  peteri
 * - new, separate structures for edges and faces
 * - edge structure contains now the split points
 * - minor adjustments to compile
 * - non-necessary functions are removed
 *
 * Revision 1.3  2002/01/31 22:37:44  peteri
 * Minor modifications, some renaming.
 *
 * Revision 1.2  2002/01/31 21:28:00  peteri
 * Major reorganization in the code.
 * - It is split into smaller modules.
 * - Some types, functions are renamed
 * - Proper flipping of edges is implemented
 *
 * Revision 1.1.1.1  2002/01/30 10:20:56  peteri
 * Imported using TkCVS
 *
 * Revision 1.2  2001/10/27 18:31:22  romek
 * *** empty log message ***
 *
 * Revision 1.1  2001/10/16 11:36:06  romek
 * Initial revision
 *
 * Initial revision
 */
