/********************************************************************
 write_log.h --     
        Functions for writing log messages. 
 
 Roman Putanowicz <romek@sect.mce.hw.ac.uk>
 SECT Research Group, Heriot-Watt University Edinburgh    
 ********************************************************************/

/* $Id: write_log.h,v 1.4 2002/07/26 18:55:21 peteri Exp $ */

#ifndef WRITE_LOG_H
#define WRITE_LOG_H

#include "config.h"
#include "mesh.h"

#ifdef LOG_ENABLED
  void Tos_LogHeader (MeshGenData *mgds);
  void Tos_LogNInputMeshes (MeshGenData *mgds);
  void Tos_LogNInputMeshNames (MeshGenData *mgds); 
  void Tos_LogNInputNEF (MeshGenData *mgds); 
  void Tos_LogNTotalInputNEF (MeshGenData *mgds); 
  void Tos_LogInputMeshParam (MeshGenData *mgds);
  void Tos_InputEdgeLengthDistribution (MeshGenData *mgds); 
  void Tos_LogBoundingBox (MeshGenData *mgds, double *low, double *high);
  void Tos_LogNTetEmptyTriang (MeshGenData *mgds);
  void Tos_LogShapeEmptyTriang (MeshGenData *mgds);
  void Tos_LogShapeCoreEmptyTriang (MeshGenData *mgds);
  void Tos_LogNTetCoreEmptyTriang (MeshGenData *mgds);
  void Tos_LogNPointsVolumeTriang (MeshGenData *mgds, int counter);
  void Tos_LogNTetVolumeTriang (MeshGenData *mgds);
  void Tos_LogNTetCoreVolumeTriang (MeshGenData *mgds);
  void Tos_LogShapeVolumeTriang (MeshGenData *mgds);
  void Tos_LogShapeCoreVolumeTriang (MeshGenData *mgds);
  void Tos_LogNMissingEdges (MeshGenData *mgds, int counter);
  void Tos_LogSwap2To3EdgeRecovery (MeshGenData *mgds, int counter);
  void Tos_LogSwap3To4EdgeRecovery (MeshGenData *mgds, int counter);
  void Tos_LogCollapseEdgeRecovery (MeshGenData *mgds, int counter);
  void Tos_LogSplitEdgeRecovery (MeshGenData *mgds, int counter);
  void Tos_LogSplitPointsDistribution (MeshGenData *mgds);
  void Tos_LogNMissingFaces (MeshGenData *mgds, int counter);
  void Tos_LogMaxFaceSplit (MeshGenData *mgds, int counter);
  void Tos_LogUnsplitedEdges (MeshGenData *mgds, int counter);
  void Tos_LogUnsplitedFaces (MeshGenData *mgds, int counter);
  void Tos_LogSplitedFaces (MeshGenData *mgds); 
  void Tos_LogNTotalTetrah (MeshGenData *mgds);
  void Tos_LogNTotalMeshPoints (MeshGenData *mgds);
  void Tos_LogNAddedMeshPoints (MeshGenData *mgds); 
  void Tos_LogShapeFinalTriang (MeshGenData *mgds);
  void Tos_LogFinalVolumeDistribution (MeshGenData *mgds);
  void Tos_LogDihedralAngle(MeshGenData *mgds) ;
#else
#  define Tos_LogHeader(mgds) /* empty */
#  define Tos_LogNInputMeshes(mgds) /* empty */
#  define Tos_LogNInputMeshNames(mgds) /* empty */ 
#  define Tos_LogNInputNEF(mgds) /* empty */ 
#  define Tos_LogNTotalInputNEF(mgds) /* empty */ 
#  define Tos_LogInputMeshParam(mgds) /* empty */
#  define Tos_InputEdgeLengthDistribution(mgds)  /* empyt */
#  define Tos_LogBoundingBox(mgds, low, high) /* empty */
#  define Tos_LogNTetEmptyTriang(mgds) /* empty */
#  define Tos_LogShapeEmptyTriang(mgds) /* empty */
#  define Tos_LogShapeCoreEmptyTriang(mgds) /* empty */
#  define Tos_LogNTetCoreEmptyTriang(mgds) /* empty */
#  define Tos_LogNPointsVolumeTriang(mgds, counter) /* empty */
#  define Tos_LogNTetVolumeTriang(mgds) /* empty */
#  define Tos_LogNTetCoreVolumeTriang(mgds) /* empty */
#  define Tos_LogShapeVolumeTriang(mgds) /* empty */
#  define Tos_LogShapeCoreVolumeTriang(mgds) /* empty */
#  define Tos_LogNMissingEdges(mgds, counter) /* empty */
#  define Tos_LogSwap2To3EdgeRecovery(mgds, counter) /* empty */
#  define Tos_LogSwap3To4EdgeRecovery(mgds, counter) /* empty */
#  define Tos_LogCollapseEdgeRecovery(mgds, counter) /* empty */
#  define Tos_LogSplitEdgeRecovery(mgds, counter) /* empty */
#  define Tos_LogSplitPointsDistribution(mgds) /* empty */
#  define Tos_LogNMissingFaces(mgds, counter) /* empty */
#  define Tos_LogMaxFaceSplit(mgds, counter) /* empty */
#  define Tos_LogUnsplitedEdges(mgds, counter) /* empty */
#  define Tos_LogUnsplitedFaces(mgds, counter) /* empty */
#  define Tos_LogSplitedFaces(mgds) /* empty */
#  define Tos_LogNTotalTetrah(mgds) /* empty */
#  define Tos_LogNTotalMeshPoints(mgds) /* empty */
#  define Tos_LogNAddedMeshPoints(mgds) /* empty */
#  define Tos_LogShapeFinalTriang(mgds) /* empty */
#  define Tos_LogSplitVolumeDistribution(mgds) /* empty */
#  define Tos_LogFinalVolumeDistribution(mgds) /* empty */
#  define Tos_LogDihedralAngle(mgds) /* empty */
#endif

#endif /* WRITE_LOG_H */
/* Modifications:
 * $Log: write_log.h,v $
 * Revision 1.4  2002/07/26 18:55:21  peteri
 * - minor editing
 * - adding a function to print the distribution of dihedral angles
 *
 * Revision 1.3  2002/02/02 15:19:32  peteri
 * - addedfaceslist member from MeshGenData is removed
 * - most of the functions are updated
 * - sup_domain member is deleted from vertex, edge, face, tetrah structures
 * - some functions are merged, we have less code
 * - some other minor changes, editing
 *
 * Revision 1.2  2002/01/31 21:28:01  peteri
 * Major reorganization in the code.
 * - It is split into smaller modules.
 * - Some types, functions are renamed
 * - Proper flipping of edges is implemented
 *
 * Revision 1.1.1.1  2002/01/30 10:20:56  peteri
 * Imported using TkCVS
 *
 * Revision 1.1  2001/10/27 18:31:57  romek
 * Initial revision
 *
 */
