#include "e_lib.h"
#include "getvertices.h"
#include "adjside.h"

/* returns the 0-based adjacency established over shared sides in
   a two-dimensional planar mesh */
void e_SetAdjacencyAndSides(MESH *mesh, int *no_node_elem, int **node_elem,
                            int ***Adjacency, int ***Adjacent_Side)
{
  int  i, j, k;
  int  s, n1, n2, process, found, element;
  int  no_vertices, no_adj_vertices;
  int *vertices, *adj_vertices;
  int  adj_s;

  if(mesh->NElemsTriang1 +
     mesh->NElemsTriang3 +
     mesh->NElemsTriang4 + 
     mesh->NElemsTriang5 +
     mesh->NElemsQuad1 != mesh->TotalNElems)
  {
    e_GeneralError("e_SetAdjacencyAndSides : Mesh contains unsupported element type");
  }

  
  *Adjacency     = e_init_frag_i_matr(mesh->TotalNElems);
  *Adjacent_Side = e_init_frag_i_matr(mesh->TotalNElems);
  
  /* setup adjacency for all elements */
  for(i = 0; i < mesh->TotalNElems; i++)
  {
    e_GetElementVertices(i, mesh, &no_vertices, &vertices);
    
    /* allocate the memory for the current element */
    if((*Adjacency)[i] == NULL)
    {
      (*Adjacency)[i]     = e_alloc_i_vect(no_vertices);
      (*Adjacent_Side)[i] = e_alloc_i_vect(no_vertices);
    }
    
    /* go through each side */
    for(s = 0; s < no_vertices; s++)
    {
      n1 = vertices[s]-1;  /* vertices of the side */
      n2 = vertices[(s+1)%no_vertices]-1;

      process = 0;
      found  = 0;
      /* check all possible combination of elements */
      for(j = 0; j < no_node_elem[n1]; j++)
      {
        for(k = 0; k < no_node_elem[n2]; k++)
        {
          /* the element contains the two nodes */
          if(   (node_elem[n1][j] == node_elem[n2][k]) /* shared side or two nodes */
             && (node_elem[n1][j] != i+1) )            /* it is not itself and larger ID */
          {
            if(process == 1) /* one side can have only one adjacent element */
            {
              e_GeneralError(
                  "e_SetAdjacencyAndSides : Element %d shares an edge more than once !", 
                  i+1);
            }
            if(node_elem[n1][j] > i+1)
            {
              element = node_elem[n1][j];
              process++;
            }
            found = 1;
          }
        }
      }

      /* there is an adjacent element */
      if(found != 0)
      {
        if(process == 1)
        {
          /* get node information about the adjacent element */
          e_GetElementVertices(element-1, mesh, &no_adj_vertices, &adj_vertices);
          
          if((*Adjacency)[element-1] == NULL)
          {
            (*Adjacency)[element-1]     = e_alloc_i_vect(no_adj_vertices);
            (*Adjacent_Side)[element-1] = e_alloc_i_vect(no_adj_vertices);
          }
          
          /* determine the side of the adjacent element */
          for(adj_s = 0; adj_s < no_adj_vertices; adj_s++)
          {
            if(   (   (adj_vertices[adj_s] == vertices[s])
                   && (adj_vertices[(adj_s+1)%no_adj_vertices] == vertices[(s+1)%no_vertices]))
               || (   (adj_vertices[adj_s] == vertices[(s+1)%no_vertices])
                   && (adj_vertices[(adj_s+1)%no_adj_vertices] == vertices[s])) )
                break;
          }
          if(adj_s == no_adj_vertices)
              e_GeneralError("e_SetAdjacencyAndSides : Could not find side in reverse !");
          
          /* store the information */
          (*Adjacency)[i][s] = element;
          (*Adjacent_Side)[i][s] = adj_s+1;
          
          (*Adjacency)[element-1][adj_s] = i+1;
          (*Adjacent_Side)[element-1][adj_s] = s+1;
        }
      }
      /* this is a boundary */
      else
      {
        (*Adjacency)[i][s] = 0;
        (*Adjacent_Side)[i][s] = 0;
      }
    }
  }
}



