#include <e_lib.h>

/***********************************************************/
/* void e_MeshCopy(MESH *source, MESH **dest)              */
/*   for duplicating mesh structures                       */
/*   limitations:                                          */
/*    - no duplication of NurbsCurves                      */
/*    - no duplication of Geometric Model                  */
/***********************************************************/

void e_MeshCopy(MESH *s, MESH **d)
{
  int i,j,k,l;
  int NLayerMaxTriang2;
  int NLayerMaxQuad3;

  (*d)=(MESH *)e_calloc(1,sizeof(MESH));
  e_NullifyMesh(*d);
  
  (*d)->Title=e_alloc_c_vect(strlen(s->Title)+1);
  strcpy((*d)->Title,s->Title);
  
  (*d)->NMeshPoints=s->NMeshPoints;
  (*d)->NNodes=s->NNodes;
  (*d)->NElemsLink1=s->NElemsLink1;
  (*d)->NElemsLink2=s->NElemsLink2;
  (*d)->NElemsLink3=s->NElemsLink3;
  (*d)->NElemsLink4=s->NElemsLink4;
  (*d)->NElemsLink5=s->NElemsLink5;
  (*d)->NElemsTriang1=s->NElemsTriang1;
  (*d)->NElemsTriang2=s->NElemsTriang2;
  (*d)->NElemsTriang3=s->NElemsTriang3;
  (*d)->NElemsTriang4=s->NElemsTriang4;
  (*d)->NElemsTriang5=s->NElemsTriang5;
  (*d)->NElemsQuad1=s->NElemsQuad1;
  (*d)->NElemsQuad3=s->NElemsQuad3;
  (*d)->NElemsTetrah1=s->NElemsTetrah1;
  (*d)->NElemsBlock1=s->NElemsBlock1;
  (*d)->TotalNElems=s->TotalNElems;
  (*d)->NBCNodes=s->NBCNodes;
  (*d)->NLoadedNodes=s->NLoadedNodes;
  (*d)->NMats=s->NMats;
  (*d)->NMdfMats=s->NMdfMats;
  (*d)->NCompMatsType1=s->NCompMatsType1;
  (*d)->NTimeSteps=s->NTimeSteps;
  (*d)->NInternalTimeSteps=s->NInternalTimeSteps;
  
  for (i=0;i<E_NELEM_TYPES;i++)
    (*d)->FirstNetIndex[i]=s->FirstNetIndex[i];

  (*d)->TimeStep=s->TimeStep;
  (*d)->DampingFactor=s->DampingFactor;
  (*d)->Beta=s->Beta;
  (*d)->Gamma=s->Gamma;
  
  (*d)->NStressPointsLink1=s->NStressPointsLink1;
  (*d)->NStressPointsLink2=s->NStressPointsLink2;
  (*d)->NStressPointsLink3=s->NStressPointsLink3;
  (*d)->NStressPointsLink4=s->NStressPointsLink4;
  (*d)->NStressPointsLink5=s->NStressPointsLink5;
  (*d)->NStressPointsTriang1=s->NStressPointsTriang1;
  (*d)->NStressPointsTriang2=s->NStressPointsTriang2;
  (*d)->NStressPointsTriang3=s->NStressPointsTriang3;
  (*d)->NStressPointsTriang4=s->NStressPointsTriang4;
  (*d)->NStressPointsTriang5=s->NStressPointsTriang5;
  (*d)->NStressPointsQuad1=s->NStressPointsQuad1;
  (*d)->NStressPointsQuad3=s->NStressPointsQuad3;
  (*d)->NStressPointsTetrah1=s->NStressPointsTetrah1;
  (*d)->NStressPointsBlock1=s->NStressPointsBlock1;
  
  (*d)->MeshPointCoords=e_alloc_cont_d_matr((*d)->NMeshPoints,E_NDIMS);
  for (i=0;i<s->NMeshPoints;i++)
    for (j=0;j<E_NDIMS;j++)
      (*d)->MeshPointCoords[i][j]=s->MeshPointCoords[i][j];

  if ((*d)->NElemsLink1)
  {
    (*d)->NodesLink1=e_alloc_cont_i_matr((*d)->NElemsLink1,E_NVPE_LINK1);
    for (i=0;i<(*d)->NElemsLink1;i++)
      for (j=0;j<E_NVPE_LINK1;j++)
        (*d)->NodesLink1[i][j]=s->NodesLink1[i][j];
  }
  if ((*d)->NElemsLink2)
  {
    (*d)->NodesLink2=e_alloc_cont_i_matr((*d)->NElemsLink2,E_NVPE_LINK2);
    for (i=0;i<(*d)->NElemsLink2;i++)
      for (j=0;j<E_NVPE_LINK2;j++)
        (*d)->NodesLink2[i][j]=s->NodesLink2[i][j];
  }
  if ((*d)->NElemsLink3)
  {
    (*d)->NodesLink3=e_alloc_cont_i_matr((*d)->NElemsLink3,E_NVPE_LINK3);
    for (i=0;i<(*d)->NElemsLink3;i++)
      for (j=0;j<E_NVPE_LINK3;j++)
        (*d)->NodesLink3[i][j]=s->NodesLink3[i][j];
  }
  if ((*d)->NElemsLink4)
  {
    (*d)->NodesLink4=e_alloc_cont_i_matr((*d)->NElemsLink4,E_NVPE_LINK4);
    for (i=0;i<(*d)->NElemsLink4;i++)
      for (j=0;j<E_NVPE_LINK4;j++)
        (*d)->NodesLink4[i][j]=s->NodesLink4[i][j];
  }
  if ((*d)->NElemsLink5)
  {
    (*d)->NodesLink5=e_alloc_cont_i_matr((*d)->NElemsLink5,E_NVPE_LINK5);
    for (i=0;i<(*d)->NElemsLink5;i++)
      for (j=0;j<E_NVPE_LINK5;j++)
        (*d)->NodesLink5[i][j]=s->NodesLink5[i][j];
  }
  if ((*d)->NElemsTriang1)
  {
    (*d)->NodesTriang1=e_alloc_cont_i_matr((*d)->NElemsTriang1,E_NVPE_TRIANG1);
    for (i=0;i<(*d)->NElemsTriang1;i++)
      for (j=0;j<E_NVPE_TRIANG1;j++)
        (*d)->NodesTriang1[i][j]=s->NodesTriang1[i][j];
  }
  if ((*d)->NElemsTriang2)
  {
    (*d)->NodesTriang2=e_alloc_cont_i_matr((*d)->NElemsTriang2,E_NVPE_TRIANG2);
    for (i=0;i<(*d)->NElemsTriang2;i++)
      for (j=0;j<E_NVPE_TRIANG2;j++)
        (*d)->NodesTriang2[i][j]=s->NodesTriang2[i][j];
  }
  if ((*d)->NElemsTriang3)
  {
    (*d)->NodesTriang3=e_alloc_cont_i_matr((*d)->NElemsTriang3,E_NVPE_TRIANG3);
    for (i=0;i<(*d)->NElemsTriang3;i++)
      for (j=0;j<E_NVPE_TRIANG3;j++)
        (*d)->NodesTriang3[i][j]=s->NodesTriang3[i][j];
  }
  if ((*d)->NElemsTriang4)
  {
    (*d)->NodesTriang4=e_alloc_cont_i_matr((*d)->NElemsTriang4,E_NVPE_TRIANG4);
    for (i=0;i<(*d)->NElemsTriang4;i++)
      for (j=0;j<E_NVPE_TRIANG4;j++)
        (*d)->NodesTriang4[i][j]=s->NodesTriang4[i][j];
  }
  if ((*d)->NElemsTriang5)
  {
    (*d)->NodesTriang5=e_alloc_cont_i_matr((*d)->NElemsTriang5,E_NVPE_TRIANG5);
    for (i=0;i<(*d)->NElemsTriang5;i++)
      for (j=0;j<E_NVPE_TRIANG5;j++)
        (*d)->NodesTriang5[i][j]=s->NodesTriang5[i][j];
  }

  if ((*d)->NElemsQuad1)
  {
    (*d)->NodesQuad1=e_alloc_cont_i_matr((*d)->NElemsQuad1,E_NVPE_QUAD1);
    for (i=0;i<(*d)->NElemsQuad1;i++)
      for (j=0;j<E_NVPE_QUAD1;j++)
        (*d)->NodesQuad1[i][j]=s->NodesQuad1[i][j];
  }
  if ((*d)->NElemsQuad3)
  {
    (*d)->NodesQuad3=e_alloc_cont_i_matr((*d)->NElemsQuad3,E_NVPE_QUAD3);
    for (i=0;i<(*d)->NElemsQuad3;i++)
      for (j=0;j<E_NVPE_QUAD3;j++)
        (*d)->NodesQuad3[i][j]=s->NodesQuad3[i][j];
  }
  if ((*d)->NElemsTetrah1)
  {
    (*d)->NodesTetrah1=e_alloc_cont_i_matr((*d)->NElemsTetrah1,E_NVPE_TETRAH1);
    for (i=0;i<(*d)->NElemsTetrah1;i++)
      for (j=0;j<E_NVPE_TETRAH1;j++)
        (*d)->NodesTetrah1[i][j]=s->NodesTetrah1[i][j];
  }
  if ((*d)->NElemsBlock1)
  {
    (*d)->NodesBlock1=e_alloc_cont_i_matr((*d)->NElemsBlock1,E_NVPE_BLOCK1);
    for (i=0;i<(*d)->NElemsBlock1;i++)
      for (j=0;j<E_NVPE_BLOCK1;j++)
        (*d)->NodesBlock1[i][j]=s->NodesBlock1[i][j];
  }
  
  (*d)->TotalNodalDOF=s->TotalNodalDOF;
  (*d)->NodalDOF=e_alloc_i_vect((*d)->NNodes);
  for (i=0;i<(*d)->NNodes;i++)
    (*d)->NodalDOF[i]=s->NodalDOF[i];
    
  if ((*d)->NBCNodes)
  {
    (*d)->BCNodes=e_alloc_i_vect((*d)->NBCNodes);
    (*d)->BCTypes=e_alloc_cont_i_matr((*d)->NBCNodes,E_NDIMS);
    (*d)->BCDispls=e_alloc_cont_d_matr((*d)->NBCNodes,E_NDIMS);
    (*d)->BCSpringConsts=e_alloc_cont_d_matr((*d)->NBCNodes,E_NDIMS);
    for (i=0;i<(*d)->NBCNodes;i++)
    {
      (*d)->BCNodes[i]=s->BCNodes[i];
      for (j=0;j<E_NDIMS;j++)
      {
        (*d)->BCTypes[i][j]=s->BCTypes[i][j];
        (*d)->BCDispls[i][j]=s->BCDispls[i][j];
        (*d)->BCSpringConsts[i][j]=s->BCSpringConsts[i][j];
      }
    }
  }
  
  if ((*d)->NLoadedNodes)
  {
    (*d)->LoadedNodes=e_alloc_i_vect((*d)->NLoadedNodes);
    (*d)->Loads=e_alloc_cont_d_matr((*d)->NLoadedNodes,E_NDIMS);
    for (i=0;i<(*d)->NLoadedNodes;i++)
    {
      (*d)->LoadedNodes[i]=s->LoadedNodes[i];
      for (j=0;j<E_NDIMS;j++)
        (*d)->Loads[i][j]=s->Loads[i][j];
    }
  }
  
  (*d)->GNIs=e_alloc_i_vect((*d)->NNodes);
  for (i=0;i<(*d)->NNodes;i++)
    (*d)->GNIs[i]=s->GNIs[i];
  (*d)->GEIs=e_alloc_i_vect((*d)->TotalNElems);
  for (i=0;i<(*d)->TotalNElems;i++)
    (*d)->GEIs[i]=s->GEIs[i];
  
  if ((*d)->NElemsLink1)
    (*d)->GEIsLink1=(*d)->GEIs+(*d)->FirstNetIndex[E_ELEM_TYPE_LINK1]-1;
  if ((*d)->NElemsLink2)
    (*d)->GEIsLink2=(*d)->GEIs+(*d)->FirstNetIndex[E_ELEM_TYPE_LINK2]-1;
  if ((*d)->NElemsLink3)
    (*d)->GEIsLink3=(*d)->GEIs+(*d)->FirstNetIndex[E_ELEM_TYPE_LINK3]-1;
  if ((*d)->NElemsLink4)
    (*d)->GEIsLink4=(*d)->GEIs+(*d)->FirstNetIndex[E_ELEM_TYPE_LINK4]-1;
  if ((*d)->NElemsLink5)
    (*d)->GEIsLink5=(*d)->GEIs+(*d)->FirstNetIndex[E_ELEM_TYPE_LINK5]-1;
  if ((*d)->NElemsTriang1)
    (*d)->GEIsTriang1=(*d)->GEIs+(*d)->FirstNetIndex[E_ELEM_TYPE_TRIANG1]-1;
  if ((*d)->NElemsTriang2)
    (*d)->GEIsTriang2=(*d)->GEIs+(*d)->FirstNetIndex[E_ELEM_TYPE_TRIANG2]-1;
  if ((*d)->NElemsTriang3)
    (*d)->GEIsTriang3=(*d)->GEIs+(*d)->FirstNetIndex[E_ELEM_TYPE_TRIANG3]-1;
  if ((*d)->NElemsTriang4)
    (*d)->GEIsTriang4=(*d)->GEIs+(*d)->FirstNetIndex[E_ELEM_TYPE_TRIANG4]-1;
  if ((*d)->NElemsTriang5)
    (*d)->GEIsTriang5=(*d)->GEIs+(*d)->FirstNetIndex[E_ELEM_TYPE_TRIANG5]-1;
  if ((*d)->NElemsQuad1)
    (*d)->GEIsQuad1=(*d)->GEIs+(*d)->FirstNetIndex[E_ELEM_TYPE_QUAD1]-1;
  if ((*d)->NElemsQuad3)
    (*d)->GEIsQuad3=(*d)->GEIs+(*d)->FirstNetIndex[E_ELEM_TYPE_QUAD3]-1;
  if ((*d)->NElemsTetrah1)
    (*d)->GEIsTetrah1=(*d)->GEIs+(*d)->FirstNetIndex[E_ELEM_TYPE_TETRAH1]-1;
  if ((*d)->NElemsBlock1)
    (*d)->GEIsBlock1=(*d)->GEIs+(*d)->FirstNetIndex[E_ELEM_TYPE_BLOCK1]-1;
  
  for (i=0;i<E_NKEYWORDS;i++)
    (*d)->KeywordSet[i]=s->KeywordSet[i];
  for (i=0;i<E_NSUBSTRUCTS;i++)
    (*d)->SubStructSet[i]=s->SubStructSet[i];
  
  if ((*d)->KeywordSet[E_KW_NUM_NMATS])
  {
    (*d)->Props=(MAT *)e_calloc((*d)->NMats,sizeof(MAT));
    for (i=0;i<(*d)->NMats;i++)
    {
      (*d)->Props[i].Name=e_alloc_c_vect(strlen(s->Props[i].Name)+1);
      strcpy((*d)->Props[i].Name,s->Props[i].Name);
      (*d)->Props[i].MatType=s->Props[i].MatType;
      (*d)->Props[i].Density=s->Props[i].Density;
      (*d)->Props[i].YMod=s->Props[i].YMod;
      (*d)->Props[i].YMod_x=s->Props[i].YMod_x;
      (*d)->Props[i].YMod_y=s->Props[i].YMod_y;
      (*d)->Props[i].YMod_z=s->Props[i].YMod_z;
      (*d)->Props[i].PoissonsRatio=s->Props[i].PoissonsRatio;
      (*d)->Props[i].Thickness=s->Props[i].Thickness;
      (*d)->Props[i].iParam1=s->Props[i].iParam1;
      (*d)->Props[i].iParam2=s->Props[i].iParam2;
      (*d)->Props[i].iParam3=s->Props[i].iParam3;
      (*d)->Props[i].iParam4=s->Props[i].iParam4;
      (*d)->Props[i].iParam5=s->Props[i].iParam5;
      (*d)->Props[i].iParam6=s->Props[i].iParam6;
      (*d)->Props[i].dParam1=s->Props[i].dParam1;
      (*d)->Props[i].dParam2=s->Props[i].dParam2;
      (*d)->Props[i].dParam3=s->Props[i].dParam3;
      (*d)->Props[i].dParam4=s->Props[i].dParam4;
      (*d)->Props[i].dParam5=s->Props[i].dParam5;
      (*d)->Props[i].dParam6=s->Props[i].dParam6;
      (*d)->Props[i].dParam7=s->Props[i].dParam7;
      (*d)->Props[i].dParam8=s->Props[i].dParam8;
      (*d)->Props[i].dParam9=s->Props[i].dParam9;
      (*d)->Props[i].dParam10=s->Props[i].dParam10;
      (*d)->Props[i].dParam11=s->Props[i].dParam11;
      (*d)->Props[i].dParam12=s->Props[i].dParam12;
      (*d)->Props[i].dParam13=s->Props[i].dParam13;
      (*d)->Props[i].dParam14=s->Props[i].dParam14;
      (*d)->Props[i].dParam15=s->Props[i].dParam15;
      (*d)->Props[i].dParam16=s->Props[i].dParam16;
      (*d)->Props[i].dParam17=s->Props[i].dParam17;
      (*d)->Props[i].dParam18=s->Props[i].dParam18;
      (*d)->Props[i].dParam19=s->Props[i].dParam19;
      (*d)->Props[i].dParam20=s->Props[i].dParam20;
      for (j=0;j<E_NMAT_PROP_KEYWORDS;j++)
        (*d)->Props[i].KeywordSet[j]=s->Props[i].KeywordSet[j];
    }
    (*d)->Mats=e_alloc_vp_vect((*d)->TotalNElems);
    if ((*d)->NElemsLink1)
    {
      (*d)->MatsLink1=
        (MAT **)&((*d)->Mats[(*d)->FirstNetIndex[E_ELEM_TYPE_LINK1]-1]);
      for (i=0;i<(*d)->NElemsLink1;i++)
      {
        for (j=0;j<(*d)->NMats;j++)
          if (strcmp(s->MatsLink1[i]->Name,(*d)->Props[j].Name)==0)
            break;
        (*d)->MatsLink1[i]=&((*d)->Props[j]);
      }
    }
    if ((*d)->NElemsLink2)
    {
      (*d)->MatsLink2=
        (MAT **)&((*d)->Mats[(*d)->FirstNetIndex[E_ELEM_TYPE_LINK2]-1]);
      for (i=0;i<(*d)->NElemsLink2;i++)
      {
        for (j=0;j<(*d)->NMats;j++)
          if (strcmp(s->MatsLink2[i]->Name,(*d)->Props[j].Name)==0)
            break;
        (*d)->MatsLink2[i]=&((*d)->Props[j]);
      }
    }
    if ((*d)->NElemsLink3)
    {
      (*d)->MatsLink3=
        (MAT **)&((*d)->Mats[(*d)->FirstNetIndex[E_ELEM_TYPE_LINK3]-1]);
      for (i=0;i<(*d)->NElemsLink3;i++)
      {
        for (j=0;j<(*d)->NMats;j++)
          if (strcmp(s->MatsLink3[i]->Name,(*d)->Props[j].Name)==0)
            break;
        (*d)->MatsLink3[i]=&((*d)->Props[j]);
      }
    }
    if ((*d)->NElemsLink4)
    {
      (*d)->MatsLink4=
        (MAT **)&((*d)->Mats[(*d)->FirstNetIndex[E_ELEM_TYPE_LINK4]-1]);
      for (i=0;i<(*d)->NElemsLink4;i++)
      {
        for (j=0;j<(*d)->NMats;j++)
          if (strcmp(s->MatsLink4[i]->Name,(*d)->Props[j].Name)==0)
            break;
        (*d)->MatsLink4[i]=&((*d)->Props[j]);
      }
    }
    if ((*d)->NElemsLink5)
    {
      (*d)->MatsLink5=
        (MAT **)&((*d)->Mats[(*d)->FirstNetIndex[E_ELEM_TYPE_LINK5]-1]);
      for (i=0;i<(*d)->NElemsLink5;i++)
      {
        for (j=0;j<(*d)->NMats;j++)
          if (strcmp(s->MatsLink5[i]->Name,(*d)->Props[j].Name)==0)
            break;
        (*d)->MatsLink5[i]=&((*d)->Props[j]);
      }
    }
    if ((*d)->NElemsTriang1)
    {
      (*d)->MatsTriang1=
        (MAT **)&((*d)->Mats[(*d)->FirstNetIndex[E_ELEM_TYPE_TRIANG1]-1]);
      for (i=0;i<(*d)->NElemsTriang1;i++)
      {
        for (j=0;j<(*d)->NMats;j++)
          if (strcmp(s->MatsTriang1[i]->Name,(*d)->Props[j].Name)==0)
            break;
        (*d)->MatsTriang1[i]=&((*d)->Props[j]);
      }
    }
    if ((*d)->NElemsTriang3)
    {
      (*d)->MatsTriang3=
        (MAT **)&((*d)->Mats[(*d)->FirstNetIndex[E_ELEM_TYPE_TRIANG3]-1]);
      for (i=0;i<(*d)->NElemsTriang3;i++)
      {
        for (j=0;j<(*d)->NMats;j++)
          if (strcmp(s->MatsTriang3[i]->Name,(*d)->Props[j].Name)==0)
            break;
        (*d)->MatsTriang3[i]=&((*d)->Props[j]);
      }
    }
    if ((*d)->NElemsTriang4)
    {
      (*d)->MatsTriang4=
        (MAT **)&((*d)->Mats[(*d)->FirstNetIndex[E_ELEM_TYPE_TRIANG4]-1]);
      for (i=0;i<(*d)->NElemsTriang4;i++)
      {
        for (j=0;j<(*d)->NMats;j++)
          if (strcmp(s->MatsTriang4[i]->Name,(*d)->Props[j].Name)==0)
            break;
        (*d)->MatsTriang4[i]=&((*d)->Props[j]);
      }
    }
    if ((*d)->NElemsTriang5)
    {
      (*d)->MatsTriang5=
        (MAT **)&((*d)->Mats[(*d)->FirstNetIndex[E_ELEM_TYPE_TRIANG5]-1]);
      for (i=0;i<(*d)->NElemsTriang5;i++)
      {
        for (j=0;j<(*d)->NMats;j++)
          if (strcmp(s->MatsTriang5[i]->Name,(*d)->Props[j].Name)==0)
            break;
        (*d)->MatsTriang5[i]=&((*d)->Props[j]);
      }
    }
    if ((*d)->NElemsQuad1)
    {
      (*d)->MatsQuad1=
        (MAT **)&((*d)->Mats[(*d)->FirstNetIndex[E_ELEM_TYPE_QUAD1]-1]);
      for (i=0;i<(*d)->NElemsQuad1;i++)
      {
        for (j=0;j<(*d)->NMats;j++)
          if (strcmp(s->MatsQuad1[i]->Name,(*d)->Props[j].Name)==0)
            break;
        (*d)->MatsQuad1[i]=&((*d)->Props[j]);
      }
    }
    if ((*d)->NElemsTetrah1)
    {
      (*d)->MatsTetrah1=
        (MAT **)&((*d)->Mats[(*d)->FirstNetIndex[E_ELEM_TYPE_TETRAH1]-1]);
      for (i=0;i<(*d)->NElemsTetrah1;i++)
      {
        for (j=0;j<(*d)->NMats;j++)
          if (strcmp(s->MatsTetrah1[i]->Name,(*d)->Props[j].Name)==0)
            break;
        (*d)->MatsTetrah1[i]=&((*d)->Props[j]);
      }
    }
    if ((*d)->NElemsBlock1)
    {
      (*d)->MatsBlock1=
        (MAT **)&((*d)->Mats[(*d)->FirstNetIndex[E_ELEM_TYPE_BLOCK1]-1]);
      for (i=0;i<(*d)->NElemsBlock1;i++)
      {
        for (j=0;j<(*d)->NMats;j++)
          if (strcmp(s->MatsBlock1[i]->Name,(*d)->Props[j].Name)==0)
            break;
        (*d)->MatsBlock1[i]=&((*d)->Props[j]);
      }
    }
  }
  
  if ((*d)->KeywordSet[E_KW_NUM_NCOMP_MATS_TYPE1])
  {
    (*d)->CompPropsType1=(COMPMAT1 *)e_calloc((*d)->NCompMatsType1,sizeof(COMPMAT1));
    for (i=0;i<(*d)->NCompMatsType1;i++)
    {
      (*d)->CompPropsType1[i].Name=e_alloc_c_vect(strlen(s->CompPropsType1[i].Name)+1);
      strcpy((*d)->CompPropsType1[i].Name,s->CompPropsType1[i].Name);
      (*d)->CompPropsType1[i].NLayers=s->CompPropsType1[i].NLayers;
      (*d)->CompPropsType1[i].LayerMats=
        (MAT **)e_calloc((*d)->CompPropsType1[i].NLayers,sizeof(MAT *));
      (*d)->CompPropsType1[i].Thicknesses=
        e_alloc_d_vect((*d)->CompPropsType1[i].NLayers);
      for (l=0;l<(*d)->CompPropsType1[i].NLayers;l++)
      {
        (*d)->CompPropsType1[i].LayerMats[l]->Name=
          e_alloc_c_vect(strlen(s->CompPropsType1[i].LayerMats[l]->Name)+1);
        strcpy((*d)->CompPropsType1[i].LayerMats[l]->Name,
          s->CompPropsType1[i].LayerMats[l]->Name);
        (*d)->CompPropsType1[i].LayerMats[l]->MatType=
          s->CompPropsType1[i].LayerMats[l]->MatType;
        (*d)->CompPropsType1[i].LayerMats[l]->Density=
          s->CompPropsType1[i].LayerMats[l]->Density;
        (*d)->CompPropsType1[i].LayerMats[l]->YMod=
          s->CompPropsType1[i].LayerMats[l]->YMod;
        (*d)->CompPropsType1[i].LayerMats[l]->YMod_x=
          s->CompPropsType1[i].LayerMats[l]->YMod_x;
        (*d)->CompPropsType1[i].LayerMats[l]->YMod_y=
          s->CompPropsType1[i].LayerMats[l]->YMod_y;
        (*d)->CompPropsType1[i].LayerMats[l]->YMod_z=
          s->CompPropsType1[i].LayerMats[l]->YMod_z;
        (*d)->CompPropsType1[i].LayerMats[l]->PoissonsRatio=
          s->CompPropsType1[i].LayerMats[l]->PoissonsRatio;
        (*d)->CompPropsType1[i].LayerMats[l]->Thickness=
          s->CompPropsType1[i].LayerMats[l]->Thickness;
        (*d)->CompPropsType1[i].LayerMats[l]->iParam1=
          s->CompPropsType1[i].LayerMats[l]->iParam1;
        (*d)->CompPropsType1[i].LayerMats[l]->iParam2=
          s->CompPropsType1[i].LayerMats[l]->iParam2;
        (*d)->CompPropsType1[i].LayerMats[l]->iParam3=
          s->CompPropsType1[i].LayerMats[l]->iParam3;
        (*d)->CompPropsType1[i].LayerMats[l]->iParam4=
          s->CompPropsType1[i].LayerMats[l]->iParam4;
        (*d)->CompPropsType1[i].LayerMats[l]->iParam5=
          s->CompPropsType1[i].LayerMats[l]->iParam5;
        (*d)->CompPropsType1[i].LayerMats[l]->iParam6=
          s->CompPropsType1[i].LayerMats[l]->iParam6;
        (*d)->CompPropsType1[i].LayerMats[l]->dParam1=
          s->CompPropsType1[i].LayerMats[l]->dParam1;
        (*d)->CompPropsType1[i].LayerMats[l]->dParam2=
          s->CompPropsType1[i].LayerMats[l]->dParam2;
        (*d)->CompPropsType1[i].LayerMats[l]->dParam3=
          s->CompPropsType1[i].LayerMats[l]->dParam4;
        (*d)->CompPropsType1[i].LayerMats[l]->dParam5=
          s->CompPropsType1[i].LayerMats[l]->dParam5;
        (*d)->CompPropsType1[i].LayerMats[l]->dParam6=
          s->CompPropsType1[i].LayerMats[l]->dParam6;
        (*d)->CompPropsType1[i].LayerMats[l]->dParam7=
          s->CompPropsType1[i].LayerMats[l]->dParam7;
        (*d)->CompPropsType1[i].LayerMats[l]->dParam8=
          s->CompPropsType1[i].LayerMats[l]->dParam8;
        (*d)->CompPropsType1[i].LayerMats[l]->dParam9=
          s->CompPropsType1[i].LayerMats[l]->dParam9;
        (*d)->CompPropsType1[i].LayerMats[l]->dParam10=
          s->CompPropsType1[i].LayerMats[l]->dParam10;
        (*d)->CompPropsType1[i].LayerMats[l]->dParam11=
          s->CompPropsType1[i].LayerMats[l]->dParam11;
        (*d)->CompPropsType1[i].LayerMats[l]->dParam12=
          s->CompPropsType1[i].LayerMats[l]->dParam12;
        (*d)->CompPropsType1[i].LayerMats[l]->dParam13=
          s->CompPropsType1[i].LayerMats[l]->dParam13;
        (*d)->CompPropsType1[i].LayerMats[l]->dParam14=
          s->CompPropsType1[i].LayerMats[l]->dParam14;
        (*d)->CompPropsType1[i].LayerMats[l]->dParam15=
          s->CompPropsType1[i].LayerMats[l]->dParam15;
        (*d)->CompPropsType1[i].LayerMats[l]->dParam16=
          s->CompPropsType1[i].LayerMats[l]->dParam16;
        (*d)->CompPropsType1[i].LayerMats[l]->dParam17=
          s->CompPropsType1[i].LayerMats[l]->dParam17;
        (*d)->CompPropsType1[i].LayerMats[l]->dParam18=
          s->CompPropsType1[i].LayerMats[l]->dParam18;
        (*d)->CompPropsType1[i].LayerMats[l]->dParam19=
          s->CompPropsType1[i].LayerMats[l]->dParam19;
        (*d)->CompPropsType1[i].LayerMats[l]->dParam20=
          s->CompPropsType1[i].LayerMats[l]->dParam20;
        for (j=0;j<E_NMAT_PROP_KEYWORDS;j++)
          (*d)->CompPropsType1[i].LayerMats[l]->KeywordSet[j]=
            s->CompPropsType1[i].LayerMats[l]->KeywordSet[j];
        (*d)->CompPropsType1[i].Thicknesses[l]=s ->CompPropsType1[i].Thicknesses[l];
      }
    }
    NLayerMaxTriang2=0;
    if ((*d)->NElemsTriang2)
    {
      (*d)->CompMatsTriang2=
        (COMPMAT1 **)&((*d)->Mats[(*d)->FirstNetIndex[E_ELEM_TYPE_TRIANG2]-1]);
      for (i=0;i<(*d)->NElemsTriang2;i++)
      {
        for (j=0;j<(*d)->NMats;j++)
          if (strcmp(s->CompMatsTriang2[i]->Name,(*d)->CompPropsType1[j].Name)==0)
            break;
        (*d)->CompMatsTriang2[i]=&((*d)->CompPropsType1[j]);
        if ((*d)->CompMatsTriang2[i]->NLayers>NLayerMaxTriang2)
          NLayerMaxTriang2=(*d)->CompMatsTriang2[i]->NLayers;
      }
    }
    NLayerMaxQuad3=0;
    if ((*d)->NElemsQuad3)
    {
      (*d)->CompMatsQuad3=
        (COMPMAT1 **)&((*d)->Mats[(*d)->FirstNetIndex[E_ELEM_TYPE_QUAD3]-1]);
      for (i=0;i<(*d)->NElemsQuad3;i++)
      {
        for (j=0;j<(*d)->NMats;j++)
          if (strcmp(s->CompMatsQuad3[i]->Name,(*d)->CompPropsType1[j].Name)==0)
            break;
        (*d)->CompMatsQuad3[i]=&((*d)->CompPropsType1[j]);
        if ((*d)->CompMatsQuad3[i]->NLayers>NLayerMaxQuad3)
          NLayerMaxQuad3=(*d)->CompMatsQuad3[i]->NLayers;
      }
    }
  }
  
  if ((*d)->SubStructSet[E_SS_NUM_DECOMP])
  {
    (*d)->Decomp.NSubDoms=s->Decomp.NSubDoms;
    (*d)->Decomp.SubDoms=e_alloc_i_vect((*d)->TotalNElems);
    for (i=0;i<(*d)->TotalNElems;i++)
      (*d)->Decomp.SubDoms[i]=s->Decomp.SubDoms[i];
    if ((*d)->NElemsLink1)
      (*d)->Decomp.SubDomsLink1=
        (*d)->Decomp.SubDoms+(*d)->FirstNetIndex[E_ELEM_TYPE_LINK1]-1;
    if ((*d)->NElemsLink2)
      (*d)->Decomp.SubDomsLink2=
        (*d)->Decomp.SubDoms+(*d)->FirstNetIndex[E_ELEM_TYPE_LINK2]-1;
    if ((*d)->NElemsLink3)
      (*d)->Decomp.SubDomsLink3=
        (*d)->Decomp.SubDoms+(*d)->FirstNetIndex[E_ELEM_TYPE_LINK3]-1;
    if ((*d)->NElemsLink4)
      (*d)->Decomp.SubDomsLink4=
        (*d)->Decomp.SubDoms+(*d)->FirstNetIndex[E_ELEM_TYPE_LINK4]-1;
    if ((*d)->NElemsLink5)
      (*d)->Decomp.SubDomsLink5=
        (*d)->Decomp.SubDoms+(*d)->FirstNetIndex[E_ELEM_TYPE_LINK5]-1;
    if ((*d)->NElemsTriang1)
      (*d)->Decomp.SubDomsTriang1=
        (*d)->Decomp.SubDoms+(*d)->FirstNetIndex[E_ELEM_TYPE_TRIANG1]-1;
    if ((*d)->NElemsTriang2)
      (*d)->Decomp.SubDomsTriang2=
        (*d)->Decomp.SubDoms+(*d)->FirstNetIndex[E_ELEM_TYPE_TRIANG2]-1;
    if ((*d)->NElemsTriang3)
      (*d)->Decomp.SubDomsTriang3=
        (*d)->Decomp.SubDoms+(*d)->FirstNetIndex[E_ELEM_TYPE_TRIANG3]-1;
    if ((*d)->NElemsTriang4)
      (*d)->Decomp.SubDomsTriang4=
        (*d)->Decomp.SubDoms+(*d)->FirstNetIndex[E_ELEM_TYPE_TRIANG4]-1;
    if ((*d)->NElemsTriang5)
      (*d)->Decomp.SubDomsTriang5=
        (*d)->Decomp.SubDoms+(*d)->FirstNetIndex[E_ELEM_TYPE_TRIANG5]-1;
    if ((*d)->NElemsQuad1)
      (*d)->Decomp.SubDomsQuad1=
        (*d)->Decomp.SubDoms+(*d)->FirstNetIndex[E_ELEM_TYPE_QUAD1]-1;
    if ((*d)->NElemsQuad3)
      (*d)->Decomp.SubDomsQuad3=
        (*d)->Decomp.SubDoms+(*d)->FirstNetIndex[E_ELEM_TYPE_QUAD3]-1;
    if ((*d)->NElemsTetrah1)
      (*d)->Decomp.SubDomsTetrah1=
        (*d)->Decomp.SubDoms+(*d)->FirstNetIndex[E_ELEM_TYPE_TETRAH1]-1;
    if ((*d)->NElemsBlock1)
      (*d)->Decomp.SubDomsBlock1=
        (*d)->Decomp.SubDoms+(*d)->FirstNetIndex[E_ELEM_TYPE_BLOCK1]-1;
  }
  
  if ((*d)->SubStructSet[E_SS_NUM_MESHPARAM])
  {
    (*d)->MeshParam.NodalMeshParamsSet=s->MeshParam.NodalMeshParamsSet;
    (*d)->MeshParam.ElemMeshParamsSet=s->MeshParam.ElemMeshParamsSet;
    if ((*d)->MeshParam.NodalMeshParamsSet)
    {
      (*d)->MeshParam.NodalMeshParams=e_alloc_d_vect((*d)->NNodes);
      for (i=0;i<(*d)->NNodes;i++)
        (*d)->MeshParam.NodalMeshParams[i]=
          s->MeshParam.NodalMeshParams[i];
    }
    if ((*d)->MeshParam.ElemMeshParamsSet)
    {
      (*d)->MeshParam.ElemMeshParams=e_alloc_d_vect((*d)->TotalNElems);
      for (i=0;i<(*d)->TotalNElems;i++)
        (*d)->MeshParam.ElemMeshParams[i]=
          s->MeshParam.ElemMeshParams[i];
      if ((*d)->NElemsLink1)
        (*d)->MeshParam.ElemMeshParamsLink1=
          (*d)->MeshParam.ElemMeshParams+(*d)->FirstNetIndex[E_ELEM_TYPE_LINK1]-1;
      if ((*d)->NElemsLink2)
        (*d)->MeshParam.ElemMeshParamsLink2=
          (*d)->MeshParam.ElemMeshParams+(*d)->FirstNetIndex[E_ELEM_TYPE_LINK2]-1;
      if ((*d)->NElemsLink3)
        (*d)->MeshParam.ElemMeshParamsLink3=
          (*d)->MeshParam.ElemMeshParams+(*d)->FirstNetIndex[E_ELEM_TYPE_LINK3]-1;
      if ((*d)->NElemsLink4)
        (*d)->MeshParam.ElemMeshParamsLink4=
          (*d)->MeshParam.ElemMeshParams+(*d)->FirstNetIndex[E_ELEM_TYPE_LINK4]-1;
      if ((*d)->NElemsLink5)
        (*d)->MeshParam.ElemMeshParamsLink5=
          (*d)->MeshParam.ElemMeshParams+(*d)->FirstNetIndex[E_ELEM_TYPE_LINK5]-1;
      if ((*d)->NElemsTriang1)
        (*d)->MeshParam.ElemMeshParamsTriang1=
          (*d)->MeshParam.ElemMeshParams+(*d)->FirstNetIndex[E_ELEM_TYPE_TRIANG1]-1;
      if ((*d)->NElemsTriang2)
        (*d)->MeshParam.ElemMeshParamsTriang2=
          (*d)->MeshParam.ElemMeshParams+(*d)->FirstNetIndex[E_ELEM_TYPE_TRIANG2]-1;
      if ((*d)->NElemsTriang3)
        (*d)->MeshParam.ElemMeshParamsTriang3=
          (*d)->MeshParam.ElemMeshParams+(*d)->FirstNetIndex[E_ELEM_TYPE_TRIANG3]-1;
      if ((*d)->NElemsTriang4)
        (*d)->MeshParam.ElemMeshParamsTriang4=
          (*d)->MeshParam.ElemMeshParams+(*d)->FirstNetIndex[E_ELEM_TYPE_TRIANG4]-1;
      if ((*d)->NElemsTriang5)
        (*d)->MeshParam.ElemMeshParamsTriang5=
          (*d)->MeshParam.ElemMeshParams+(*d)->FirstNetIndex[E_ELEM_TYPE_TRIANG5]-1;
      if ((*d)->NElemsQuad1)
        (*d)->MeshParam.ElemMeshParamsQuad1=
          (*d)->MeshParam.ElemMeshParams+(*d)->FirstNetIndex[E_ELEM_TYPE_QUAD1]-1;
      if ((*d)->NElemsQuad3)
        (*d)->MeshParam.ElemMeshParamsQuad3=
          (*d)->MeshParam.ElemMeshParams+(*d)->FirstNetIndex[E_ELEM_TYPE_QUAD3]-1;
      if ((*d)->NElemsTetrah1)
        (*d)->MeshParam.ElemMeshParamsTetrah1=
          (*d)->MeshParam.ElemMeshParams+(*d)->FirstNetIndex[E_ELEM_TYPE_TETRAH1]-1;
      if ((*d)->NElemsBlock1)
        (*d)->MeshParam.ElemMeshParamsBlock1=
          (*d)->MeshParam.ElemMeshParams+(*d)->FirstNetIndex[E_ELEM_TYPE_BLOCK1]-1;
    }
  }
  
  if ((*d)->SubStructSet[E_SS_NUM_FEERROR])
  {
    (*d)->FEError.FEErrors=e_alloc_d_vect((*d)->TotalNElems);
    for (i=0;i<(*d)->TotalNElems;i++)
      (*d)->FEError.FEErrors[i]=s->FEError.FEErrors[i];
    if ((*d)->NElemsLink1)
      (*d)->FEError.FEErrorsLink1=
        (*d)->FEError.FEErrors+(*d)->FirstNetIndex[E_ELEM_TYPE_LINK1]-1;
    if ((*d)->NElemsLink2)
      (*d)->FEError.FEErrorsLink2=
        (*d)->FEError.FEErrors+(*d)->FirstNetIndex[E_ELEM_TYPE_LINK2]-1;
    if ((*d)->NElemsLink3)
      (*d)->FEError.FEErrorsLink3=
        (*d)->FEError.FEErrors+(*d)->FirstNetIndex[E_ELEM_TYPE_LINK3]-1;
    if ((*d)->NElemsLink4)
      (*d)->FEError.FEErrorsLink4=
        (*d)->FEError.FEErrors+(*d)->FirstNetIndex[E_ELEM_TYPE_LINK4]-1;
    if ((*d)->NElemsLink5)
      (*d)->FEError.FEErrorsLink5=
        (*d)->FEError.FEErrors+(*d)->FirstNetIndex[E_ELEM_TYPE_LINK5]-1;
    if ((*d)->NElemsTriang1)
      (*d)->FEError.FEErrorsTriang1=
        (*d)->FEError.FEErrors+(*d)->FirstNetIndex[E_ELEM_TYPE_TRIANG1]-1;
    if ((*d)->NElemsTriang2)
      (*d)->FEError.FEErrorsTriang2=
        (*d)->FEError.FEErrors+(*d)->FirstNetIndex[E_ELEM_TYPE_TRIANG2]-1;
    if ((*d)->NElemsTriang3)
      (*d)->FEError.FEErrorsTriang3=
        (*d)->FEError.FEErrors+(*d)->FirstNetIndex[E_ELEM_TYPE_TRIANG3]-1;
    if ((*d)->NElemsTriang4)
      (*d)->FEError.FEErrorsTriang4=
        (*d)->FEError.FEErrors+(*d)->FirstNetIndex[E_ELEM_TYPE_TRIANG4]-1;
    if ((*d)->NElemsTriang5)
      (*d)->FEError.FEErrorsTriang5=
        (*d)->FEError.FEErrors+(*d)->FirstNetIndex[E_ELEM_TYPE_TRIANG5]-1;
    if ((*d)->NElemsQuad1)
      (*d)->FEError.FEErrorsQuad1=
        (*d)->FEError.FEErrors+(*d)->FirstNetIndex[E_ELEM_TYPE_QUAD1]-1;
    if ((*d)->NElemsQuad3)
      (*d)->FEError.FEErrorsQuad3=
        (*d)->FEError.FEErrors+(*d)->FirstNetIndex[E_ELEM_TYPE_QUAD3]-1;
    if ((*d)->NElemsTetrah1)
      (*d)->FEError.FEErrorsTetrah1=
        (*d)->FEError.FEErrors+(*d)->FirstNetIndex[E_ELEM_TYPE_TETRAH1]-1;
    if ((*d)->NElemsBlock1)
      (*d)->FEError.FEErrorsBlock1=
        (*d)->FEError.FEErrors+(*d)->FirstNetIndex[E_ELEM_TYPE_BLOCK1]-1;
  }
  
  if ((*d)->SubStructSet[E_SS_NUM_STRESS])
  {
    if ((*d)->NElemsLink1)
    {
      (*d)->Stresses.StressesLink1=e_alloc_cont_d_3D_matr(
        (*d)->NElemsLink1,(*d)->NStressPointsLink1,E_NUM_STRESS_COMPONENTS);
      for (i=0;i<(*d)->NElemsLink1;i++)
        for (j=0;j<(*d)->NStressPointsLink1;j++)
          for (k=0;k<E_NUM_STRESS_COMPONENTS;k++)
            (*d)->Stresses.StressesLink1[i][j][k]=s->Stresses.StressesLink1[i][j][k];
    }
    if ((*d)->NElemsLink2)
    {
      (*d)->Stresses.StressesLink2=e_alloc_cont_d_3D_matr(
        (*d)->NElemsLink2,(*d)->NStressPointsLink2,E_NUM_STRESS_COMPONENTS);
      for (i=0;i<(*d)->NElemsLink2;i++)
        for (j=0;j<(*d)->NStressPointsLink2;j++)
          for (k=0;k<E_NUM_STRESS_COMPONENTS;k++)
            (*d)->Stresses.StressesLink2[i][j][k]=s->Stresses.StressesLink2[i][j][k];
    }
    if ((*d)->NElemsLink3)
    {
      (*d)->Stresses.StressesLink3=e_alloc_cont_d_3D_matr(
        (*d)->NElemsLink3,(*d)->NStressPointsLink3,E_NUM_STRESS_COMPONENTS);
      for (i=0;i<(*d)->NElemsLink3;i++)
        for (j=0;j<(*d)->NStressPointsLink3;j++)
          for (k=0;k<E_NUM_STRESS_COMPONENTS;k++)
            (*d)->Stresses.StressesLink3[i][j][k]=s->Stresses.StressesLink3[i][j][k];
    }
    if ((*d)->NElemsLink4)
    {
      (*d)->Stresses.StressesLink4=e_alloc_cont_d_3D_matr(
        (*d)->NElemsLink4,(*d)->NStressPointsLink4,E_NUM_STRESS_COMPONENTS);
      for (i=0;i<(*d)->NElemsLink4;i++)
        for (j=0;j<(*d)->NStressPointsLink4;j++)
          for (k=0;k<E_NUM_STRESS_COMPONENTS;k++)
            (*d)->Stresses.StressesLink4[i][j][k]=s->Stresses.StressesLink4[i][j][k];
    }
    if ((*d)->NElemsLink5)
    {
      (*d)->Stresses.StressesLink5=e_alloc_cont_d_3D_matr(
        (*d)->NElemsLink5,(*d)->NStressPointsLink5,E_NUM_STRESS_COMPONENTS);
      for (i=0;i<(*d)->NElemsLink5;i++)
        for (j=0;j<(*d)->NStressPointsLink5;j++)
          for (k=0;k<E_NUM_STRESS_COMPONENTS;k++)
            (*d)->Stresses.StressesLink5[i][j][k]=s->Stresses.StressesLink5[i][j][k];
    }
    if ((*d)->NElemsTriang1)
    {
      (*d)->Stresses.StressesTriang1=e_alloc_cont_d_3D_matr(
        (*d)->NElemsTriang1,(*d)->NStressPointsTriang1,E_NUM_STRESS_COMPONENTS);
      for (i=0;i<(*d)->NElemsTriang1;i++)
        for (j=0;j<(*d)->NStressPointsTriang1;j++)
          for (k=0;k<E_NUM_STRESS_COMPONENTS;k++)
            (*d)->Stresses.StressesTriang1[i][j][k]=s->Stresses.StressesTriang1[i][j][k];
    }
    if ((*d)->NElemsTriang2)
    {
      (*d)->Stresses.StressesTriang2=e_alloc_cont_d_4D_matr(
        (*d)->NElemsTriang2,NLayerMaxTriang2,
        (*d)->NStressPointsTriang2,E_NUM_STRESS_COMPONENTS);
      for (i=0;i<(*d)->NElemsTriang2;i++)
        for (l=0;l<(*d)->CompMatsTriang2[i]->NLayers;l++)
          for (j=0;j<(*d)->NStressPointsTriang2;j++)
            for (k=0;k<E_NUM_STRESS_COMPONENTS;k++)
              (*d)->Stresses.StressesTriang2[i][l][j][k]=s->Stresses.StressesTriang2[i][l][j][k];
    }
    if ((*d)->NElemsTriang3)
    {
      (*d)->Stresses.StressesTriang3=e_alloc_cont_d_3D_matr(
        (*d)->NElemsTriang3,(*d)->NStressPointsTriang3,E_NUM_STRESS_COMPONENTS);
      for (i=0;i<(*d)->NElemsTriang3;i++)
        for (j=0;j<(*d)->NStressPointsTriang3;j++)
          for (k=0;k<E_NUM_STRESS_COMPONENTS;k++)
            (*d)->Stresses.StressesTriang3[i][j][k]=s->Stresses.StressesTriang3[i][j][k];
    }
    if ((*d)->NElemsTriang4)
    {
      (*d)->Stresses.StressesTriang4=e_alloc_cont_d_3D_matr(
        (*d)->NElemsTriang4,(*d)->NStressPointsTriang4,E_NUM_STRESS_COMPONENTS);
      for (i=0;i<(*d)->NElemsTriang4;i++)
        for (j=0;j<(*d)->NStressPointsTriang4;j++)
          for (k=0;k<E_NUM_STRESS_COMPONENTS;k++)
            (*d)->Stresses.StressesTriang4[i][j][k]=s->Stresses.StressesTriang4[i][j][k];
    }
    if ((*d)->NElemsTriang5)
    {
      (*d)->Stresses.StressesTriang5=e_alloc_cont_d_3D_matr(
        (*d)->NElemsTriang5,(*d)->NStressPointsTriang5,E_NUM_STRESS_COMPONENTS);
      for (i=0;i<(*d)->NElemsTriang5;i++)
        for (j=0;j<(*d)->NStressPointsTriang5;j++)
          for (k=0;k<E_NUM_STRESS_COMPONENTS;k++)
            (*d)->Stresses.StressesTriang5[i][j][k]=s->Stresses.StressesTriang5[i][j][k];
    }
    if ((*d)->NElemsQuad1)
    {
      (*d)->Stresses.StressesQuad1=e_alloc_cont_d_3D_matr(
        (*d)->NElemsQuad1,(*d)->NStressPointsQuad1,E_NUM_STRESS_COMPONENTS);
      for (i=0;i<(*d)->NElemsQuad1;i++)
        for (j=0;j<(*d)->NStressPointsQuad1;j++)
          for (k=0;k<E_NUM_STRESS_COMPONENTS;k++)
            (*d)->Stresses.StressesQuad1[i][j][k]=s->Stresses.StressesQuad1[i][j][k];
    }
    if ((*d)->NElemsQuad3)
    {
      (*d)->Stresses.StressesQuad3=e_alloc_cont_d_4D_matr(
        (*d)->NElemsQuad3,NLayerMaxQuad3,
        (*d)->NStressPointsQuad3,E_NUM_STRESS_COMPONENTS);
      for (i=0;i<(*d)->NElemsQuad3;i++)
        for (l=0;l<(*d)->CompMatsQuad3[i]->NLayers;l++)
          for (j=0;j<(*d)->NStressPointsQuad3;j++)
            for (k=0;k<E_NUM_STRESS_COMPONENTS;k++)
              (*d)->Stresses.StressesQuad3[i][l][j][k]=s->Stresses.StressesQuad3[i][l][j][k];
    }
    if ((*d)->NElemsTetrah1)
    {
      (*d)->Stresses.StressesTetrah1=e_alloc_cont_d_3D_matr(
        (*d)->NElemsTetrah1,(*d)->NStressPointsTetrah1,E_NUM_STRESS_COMPONENTS);
      for (i=0;i<(*d)->NElemsTetrah1;i++)
        for (j=0;j<(*d)->NStressPointsTetrah1;j++)
          for (k=0;k<E_NUM_STRESS_COMPONENTS;k++)
            (*d)->Stresses.StressesTetrah1[i][j][k]=s->Stresses.StressesTetrah1[i][j][k];
    }
    if ((*d)->NElemsBlock1)
    {
      (*d)->Stresses.StressesBlock1=e_alloc_cont_d_3D_matr(
        (*d)->NElemsBlock1,(*d)->NStressPointsBlock1,E_NUM_STRESS_COMPONENTS);
      for (i=0;i<(*d)->NElemsBlock1;i++)
        for (j=0;j<(*d)->NStressPointsBlock1;j++)
          for (k=0;k<E_NUM_STRESS_COMPONENTS;k++)
            (*d)->Stresses.StressesBlock1[i][j][k]=s->Stresses.StressesBlock1[i][j][k];
    }
  }
}

#ifdef E_COPY_TEST
main(int argc, char **argv)
{
  MESH *s,*d;
  
  e_GetMeshData(&s,argv[1]);
  e_MeshCopy(s,&d);
  e_PutMeshData(d,argv[2]);
  
  e_FreeMeshData(s);
  e_FreeMeshData(d);
}
#endif
