/********************************************************************
  mtrf.h --
        Mapping (and reverse mapping) to coordinate system 
        associated with a triangle.
 
 Jelle Muylle <jelle@sect.mce.hw.ac.uk>
 Roman Putanowicz <romek@sect.mce.hw.ac.uk>
 SECT Research Group, Heriot-Watt University, Edinburgh    
********************************************************************/

#ifndef MTRF_H__
#define MTRF_H__

/* structure which holds base vectors of coordinate system associated
 * with a triangle (only base vectors coplanar with triangle plane
 * are stored) and point vector of coordinate system origin
 */
typedef struct
{
  double u[3];                  /* 'side' base vector */
  double s[3];                  /* 'aside' base vector */
  double r[3];                  /* point vector of system origin */
  double w[3];                  /* normal vector */
  int used;                     /* use the mapping? TRUE/FALSE */
}
MTRF;

/* retrieve the mapping transformation data based
 * on the triangle whose corners coordinates are
 * stored in coords on index n[0], n[1], and n[3]
 *
 * returns TRUE if mtrf is retrieved successfully
 * and FALSE if any problems occur such as collinearity
 */
int get_mtrf(double **coords, int *n, MTRF *mtrf);


/* map the point src into point dest using the
 * mapping transformation based on data from mtrf
 * This mapping can be seen as orthogonal projection 
 * on the coordinate plane associated with a triangle
 */


/* use mtrf_map to transform 3D into 2D */
int mtrf_map(double *src, MTRF *mtrf, double *dest);


/* map the point src into point dest using the
 * reverse mapping transformation based on data  from mtrf
 */
int mtrf_revmap(double *src, MTRF *mtrf, double *dest);


#endif /* MTRF_H__ */



