#include "mgn.h"
#include "mgn_graphics.h"
#include "dtr.h"
#include "mgn_aux.h"

static void add_bnd_node(int node, double **coords, int np, int **tr,
                         int *ntr, int **hull, int *nhull,
                         int **tpnl, int *last, MTRF *mtrf);

void f_connect_bnd_nodes(double **coords, int np, BND * bnd, int ***triangles,
                         int *ntriangles, int **convhull, int *nhullelems,
                         int **tpnl, MTRF *mtrf)
{
  /* This function triangulates the nodes of which coordinates
   * are to be found in coords and are mentioned in the bnd. It initializes triangle matrix triangles.
   * Creates a convex hull, border table and tpnl list
   */

  int b;                        /* boundary node counter */
  int e;                        /* outer node counter */
  int **tr;                     /* triangle matrix */
  int ntr;                      /* number of triangles */

  int i, j;                     /* counters */

  int c0, c1, c2;               /* the nodes of the first triangle */
  int ccw1,ccw2,valid1,valid2;  /* result of CCW tests */
  int cn;                       /* candidate node for first triangle */

  int last;                     /* last found triangle */

  int nhull;                    /* number of points on the convex hull */
  int *hull;                    /* vector containing the node indices that form the hull */

  int bnp;                      /* number of boundary points */
  double **bcrds;

  /* temporary triangle matrix */
  bnp = bnd->oe.nen;
  for (i = 0; i < bnd->nie; i++)
    bnp += bnd->ie[i].nen;
  bcrds = e_alloc_cont_d_matr(bnp, 3);
  b = 0;
  for (i = 0; i < bnd->oe.nen; i++)
  {
    for (j = 0; j < 3; j++)
      bcrds[b][j] = coords[bnd->oe.en[i]][j];
    b++;
  }
  for (e = 0; e < bnd->nie; e++)
    for (i = 0; i < bnd->ie[e].nen; i++)
    {
      for (j = 0; j < 3; j++)
        bcrds[b][j] = coords[bnd->ie[e].en[i]][j];
      b++;
    }

  nhull = f_compute_convex_hull(bcrds, bnp, &hull, mtrf, 1);
  e_free(hull);
  ntr = 2 * (bnp - 1) - nhull;
  if (!(tr = e_alloc_cont_i_matr(ntr, 11)))
    e_GeneralError("memory error\n");
  ntr = 0;
  nhull = 0;

  if (ShowGraphics)
  {
    mgnDrawAll(bnp, bcrds, 0, NULL, 0, NULL, 0, NULL, NULL, "starting bndtr");
  }

  e_free_cont_d_matr(bnp, bcrds);
  /* initialize the first triangle */
  tr[0][0] = bnd->oe.en[0];
  cn = 1;
  ccw1 = FALSE;
  ccw2 = FALSE;
  while ((!ccw1) && (!ccw2))
  {
    cn++;
    ccw1=IsCCW(coords[bnd->oe.en[0]], coords[bnd->oe.en[1]],
               coords[bnd->oe.en[cn]], mtrf, &valid1);
    ccw2=IsCCW(coords[bnd->oe.en[0]], coords[bnd->oe.en[cn]],
               coords[bnd->oe.en[1]], mtrf, &valid2);
  }
  if (ccw1)
  {
    tr[0][1] = bnd->oe.en[1];
    tr[0][2] = bnd->oe.en[cn];
  }
  else
  {
    tr[0][1] = bnd->oe.en[cn];
    tr[0][2] = bnd->oe.en[1];
  }
  tr[0][9] = 1;
  tr[0][10] = 0;
  ntr++;

  /* initialize the hull */
  hull = e_alloc_i_vect(INCR_SIZE);
  nhull = 3;
  c0 = tr[0][0];
  c1 = tr[0][1];
  c2 = tr[0][2];
  hull[0] = c0;
  hull[1] = c1;
  hull[2] = c2;

  /* initialize the tpnl */
  addtr2tpnlist(tr, 0, tpnl);

  /* initialize adjacency info */
  for (i = 0; i < 3; i++)
    tr[0][i + 3] = -1;

  /* last found triangle */
  last = 0;

  /* and add all boundary nodes */
  for (b = 2; b < bnd->oe.nen; b++)
    if (b != cn)
    {
      add_bnd_node(bnd->oe.en[b], coords, np, tr, &ntr,
                   &hull, &nhull, tpnl, &last, mtrf);
    }
  for (e = 0; e < bnd->nie; e++)
    for (b = 0; b < bnd->ie[e].nen; b++)
    {
      add_bnd_node(bnd->ie[e].en[b], coords, np, tr, &ntr,
                   &hull, &nhull, tpnl, &last, mtrf);
    }

  /* pass all computed information */
  (*triangles) = e_alloc_cont_i_matr(ntr, 11);
  (*convhull) = e_alloc_i_vect((int)ceil((double)nhull/(double)INCR_SIZE)*INCR_SIZE);
  for (i = 0; i < ntr; i++)
    for (j = 0; j < 11; j++)
      (*triangles)[i][j] = tr[i][j];
  for (i = 0; i < nhull; i++)
    (*convhull)[i] = hull[i];
  (*ntriangles) = ntr;
  (*nhullelems) = nhull;

  /* free up all consumed memory */
  e_free_cont_i_matr(ntr, tr);
  e_free(hull);
}
static void add_bnd_node(int node, double **coords, int np, int **tr,
                         int *ntr, int **hull, int *nhull,
                         int **tpnl, int *last, MTRF * mtrf)
{
  int sres;
  int t;
  int onside;

  int *efwl;                    /* edge flip wait list */
  int nwl;                      /* length of the wait list */
  int twl;                      /* free spaces in the efwl */

  twl = INCR_SIZE;              /* initialize the edge flip wait list */
  efwl = e_alloc_i_vect(twl);   /* edge flip wait list side 4 to be checked */
  nwl = 0;

  sres = trianglesearch(coords[node], coords, tr, *last, &t, &onside, mtrf);
  if (sres == 1)                /* triangle found */
  {
    *last = t;
    insert_node(node, t, ntr, tr, &efwl, &nwl, &twl, onside,
                hull, nhull, tpnl);
  }
  else if (sres == 0)
  {
    if (DebugOutput)
      printf("\noutside connection !!\n");
    connect_outside_node(node, coords, ntr, tr, &efwl, &nwl, &twl, hull,
                         nhull, tpnl, mtrf);
  }
  edge_flipping(node, coords, &efwl, &nwl, &twl, tr, tpnl, mtrf);

  if (ShowGraphics)
  {
    mgnDrawAll(np, coords, *ntr, tr, 0, NULL, 0, NULL, NULL, "connecting");
  }

  e_free(efwl);
}
