/*

p_0 = (((a_0 - c_0) * (a_0 + c_0) + (a_1 - c_1) * (a_1 + c_1)) / 2 * (b_1 - c_1)
    -  ((b_0 - c_0) * (b_0 + c_0) + (b_1 - c_1) * (b_1 + c_1)) / 2 * (a_1 - c_1))
    / D

p_1 = (((b_0 - c_0) * (b_0 + c_0) + (b_1 - c_1) * (b_1 + c_1)) / 2 * (a_0 - c_0)
    -  ((a_0 - c_0) * (a_0 + c_0) + (a_1 - c_1) * (a_1 + c_1)) / 2 * (b_0 - c_0))
    / D

D = (a_0 - c_0) * (b_1 - c_1) - (b_0 - c_0) * (a_1 - c_1)

r^2 = (c_0 - p_0)^2 + (c_1 - p_1)^2

*/


static double circumcircle(double *cntr, double **crds, int **tr, int t)
{
	double r2,d;

	d=(crds[tr[t][0]][0] - crds[tr[t][2]][0]) *
		(crds[tr[t][1]][1] - crds[tr[t][2]][1]) -
		(crds[tr[t][1]][0] - crds[tr[t][2]][0]) *
		(crds[tr[t][0]][1] - crds[tr[t][2]][1]);

	cntr[0]=(((crds[tr[t][0]][0] - crds[tr[t][2]][0]) *
	          (crds[tr[t][0]][0] + crds[tr[t][2]][0]) +
	          (crds[tr[t][0]][1] - crds[tr[t][2]][1]) *
	          (crds[tr[t][0]][1] + crds[tr[t][2]][1])) / 2 *
	          (crds[tr[t][1]][1] - crds[tr[t][2]][1])
    		 - ((crds[tr[t][1]][0] - crds[tr[t][2]][0]) *
					  (crds[tr[t][1]][0] + crds[tr[t][2]][0]) +
					  (crds[tr[t][1]][1] - crds[tr[t][2]][1]) *
					  (crds[tr[t][1]][1] + crds[tr[t][2]][1])) / 2 *
					  (crds[tr[t][0]][1] - crds[tr[t][2]][1])        )
    		 / d;

	cntr[1]=(((crds[tr[t][1]][0] - crds[tr[t][2]][0]) *
		        (crds[tr[t][1]][0] + crds[tr[t][2]][0]) +
		        (crds[tr[t][1]][1] - crds[tr[t][2]][1]) *
		        (crds[tr[t][1]][1] + crds[tr[t][2]][1])) / 2 *
		        (crds[tr[t][0]][0] - crds[tr[t][2]][0])
	    	 - ((crds[tr[t][0]][0] - crds[tr[t][2]][0]) *
					  (crds[tr[t][0]][0] + crds[tr[t][2]][0]) +
					  (crds[tr[t][0]][1] - crds[tr[t][2]][1]) *
					  (crds[tr[t][0]][1] + crds[tr[t][2]][1])) / 2 *
					  (crds[tr[t][1]][0] - crds[tr[t][2]][0])        )
	    	 / d;

	r2=(crds[tr[t][2]][0] - cntr[0]) *
		 (crds[tr[t][2]][0] - cntr[0]) +
		 (crds[tr[t][2]][1] - cntr[1]) *
		 (crds[tr[t][2]][1] - cntr[1]);

	return sqrt(r2);
}

void delref_chew1(double ***coords, int *np, int *tcs, int ***triangles,
                  int *ntriangles, int *tts, int **convhull,
                  int *nhullelems, int ctrl, double h,
                  int ***tpnl, BND *bnd, MTRF *mtrf)
{
  /* Delaunay mesh refinement based on Chew's 1st alg. */

  int i;                        /* counters */
  int onside;                   /* onside index of the triangle search */
  int t;                        /* index of an existing triangle */
  double *cntr;                 /* centre of an existing triangle */
  int sres;                     /* result of the search operation */
  int refine;                   /* keep on refining? yes/no */
  int *efwl;                    /* edge flip wait list */
  int nwl;                      /* length of the wait list */
  int twl;                      /* total allocated space in the efwl */
  int ct;												/* triangle the candidate point belongs to */
	double cr;										/* circumradius of a triangle */
	double H;											/* mesh size [H,2H] */
  int nsteps;                   /* number of refining steps */

	e_GeneralWarning("experimental Chew 1 delref used");
	if (ctrl!=CTRL_UNIFORM)
		e_GeneralError("only works for uniform meshing");
	if (mtrf->used)
		e_GeneralError("Chew1 is not 3D compliant");

  cntr = e_alloc_d_vect(3);			/* initialize space for circumcentre */
  twl = INCR_SIZE;              /* initialize the edge flip wait list */
  efwl = e_alloc_i_vect(twl);   /* edge flip wait list side 4 to be checked */
  nwl = 0;

	H = h/2;											/* uniform assumed */

  nsteps = 0;
  refine = TRUE;

  /* as long as there are triangles to be refined */
  while (refine)
  {
    nsteps++;
    if (PrintProgress)
    	if (!(nsteps%100))
      printf("number of inserted centers: %d\n", nsteps);

    if (ShowGraphics)           /* a little bit of graphics output */
    {
      mgnDrawAll(*np, *coords, *ntriangles, *triangles, 0, NULL, 0,
                 NULL, NULL, "refining");
		}

    for (t = 0; t < (*ntriangles); t++)
    {
      if (((*triangles)[t][9] == 1) && ((*triangles)[t][10] != -1))
      /* for those triangles that are active and inside the domain */
      {
  			/* first compute circumcircle centre & radius */
				cr=circumcircle(cntr, (*coords), (*triangles), t);
  			/* if radius>H split triangle */
  			if (cr>H)
  			{
  				/* search parent triangle */
  				sres=trianglesearch(cntr, (*coords), (*triangles),t, &ct, &onside, mtrf);
					if (sres == 0)
					  e_GeneralError("refinement node can't be outside the domain!");
					if (sres == -1)
					  e_GeneralError("refinement node is duplicate and so far undetected ?!?");
  				/* add center to coords, mpr, tpnl */
  				while ((*tcs) - (*np) < 1)
      		{
        		(*coords) = e_incr_cont_d_matr((*tcs), 3, (*coords), (*tcs));
        		(*tpnl) = (int **) e_incr_vp_vect((*tcs), (void **) (*tpnl), (*tcs));
        		(*tcs) += (*tcs);
      		}
      		/* update the coordinates array */
        	for (i = 0; i < 3; i++)
        	  (*coords)[(*np)][i] = cntr[i];
        	/* initialize the new entries in the tpnl */
        	(*tpnl)[(*np)] = e_alloc_i_vect(2);
        	(*tpnl)[(*np)][0] = 0;
        	(*tpnl)[(*np)][1] = 0;
        	/* make more space for new triangles (maximum 4 new) */
					if ((*tts) - (*ntriangles) < 4)
					{
					  (*triangles) = e_incr_cont_i_matr((*tts), 11, (*triangles), (*tts));
					  (*tts) += (*tts);
					}
          /* insert centre */
  				insert_node((*np), ct, ntriangles, (*triangles), &efwl, &nwl,
					  &twl, onside, convhull, nhullelems, (*tpnl));
					edge_flipping((*np), (*coords), &efwl, &nwl, &twl, (*triangles),
            (*tpnl), mtrf);
  				/* break */
  				(*np)++;
  				break;
				}
  			/* else set triangle inactive */
  			(*triangles)[t][9]=0;
			}
		}
		if (t==(*ntriangles))
			refine=FALSE;
  }

  /* free up that memory */
  e_free(cntr);
  e_free(efwl);
}
