#include "mgn.h"
#include "dtr.h"


/* dtr specific smoothing function.
   recycles the adjacency information generated by the dtr
   required for nodal avaraging */

void smooth_by_nodal_averaging(double **coords, int np, int **tr, int ntr,
                               int **tpnl, int sm)
{
  int n;                        /* node */
  int t;                        /* triangle */
  int s;                        /* side */
  int smi;                      /* smoothing iteration counter */
  int is_on_bnd;                /* is the node on the boundary ? */
  double pos[3];                /* new position */
  int nni;                      /* number of nodes involved in determing this node's position */

  for (smi = 0; smi < sm; smi++)
  {
    for (n = 0; n < np; n++)
    {
      is_on_bnd = FALSE;
      for (t = 0; t < tpnl[n][0]; t++)
      {
        if (tr[tpnl[n][2 + t]][10] == 0)
          e_GeneralError("zero area codes!");
        if (tr[tpnl[n][2 + t]][10] == -1)
        {
          is_on_bnd = TRUE;
          break;
        }
        for (s = 0; s < 3; s++)
          if (tr[tpnl[n][2 + t]][s] == n)
            break;
        if ((tr[tpnl[n][2 + t]][3 + s] == -1)
            || (tr[tpnl[n][2 + t]][3 + (s + 2) % 3] == -1))
        {
          is_on_bnd = TRUE;
          break;
        }
      }
      if (is_on_bnd)
        continue;
      pos[0] = 0.0;
      pos[1] = 0.0;
      pos[2] = 0.0;
      nni = 0;
      for (t = 0; t < tpnl[n][0]; t++)
      {
        for (s = 0; s < 3; s++)
          if (tr[tpnl[n][2 + t]][s] != n)
          {
            pos[0] += coords[tr[tpnl[n][2 + t]][s]][0];
            pos[1] += coords[tr[tpnl[n][2 + t]][s]][1];
            pos[2] += coords[tr[tpnl[n][2 + t]][s]][2];
            nni++;
          }
      }
      if (nni > 0)
      {
        pos[0] /= (double) nni;
        pos[1] /= (double) nni;
        pos[2] /= (double) nni;
        coords[n][0] = pos[0];
        coords[n][1] = pos[1];
        coords[n][2] = pos[2];
      }
    }
  }
}

/* end of file */
