#include "mgn.h"
#include "dtr.h"

void addtr2tpnlist(int **tr, int t, int **tpnl)
{
  /* add a triangle t to the tpnl */

  int i, j;                     /* counters */
  int already;                  /* the triangle is already in the tpnl */

  for (i = 0; i < 3; i++)
  {
    already = FALSE;
    for (j = 2; j < 2 + tpnl[tr[t][i]][0]; j++)
    {
      if (tpnl[tr[t][i]][j] == t)
      {
        already = TRUE;
        break;
      }
    }
    if (!already)
    {
      if (tpnl[tr[t][i]][1] == tpnl[tr[t][i]][0])
      {
        tpnl[tr[t][i]] =
          e_incr_i_vect(tpnl[tr[t][i]][1] + 2, tpnl[tr[t][i]],
                        tpnl[tr[t][i]][1] + 2);
        tpnl[tr[t][i]][1] += tpnl[tr[t][i]][1] + 2;
      }
      tpnl[tr[t][i]][2 + tpnl[tr[t][i]][0]] = t;
      tpnl[tr[t][i]][0]++;
    }
  }
}

void rmvtrftpnlist(int t, int n, int **tpnl)
{
  /* remove a triangle from the tpnl */

  int i, j;                     /* counters */

  for (i = 0; i < tpnl[n][0]; i++)
    if (tpnl[n][2 + i] == t)
      break;
  if (i == tpnl[n][0])
    e_GeneralWarning
      ("tried to remove triangle from tpnl that was not there\n");
  for (j = i; j < tpnl[n][0] - 1; j++)
    tpnl[n][2 + j] = tpnl[n][j + 2 + 1];
  tpnl[n][0]--;
}

void inittpnl(int ***tpnl, int np)
{
  int i;

  (*tpnl) = (int **) e_calloc(np, sizeof(int *));
  for (i = 0; i < np; i++)
  {
    (*tpnl)[i] = e_alloc_i_vect(2);
    (*tpnl)[i][0] = 0;          /* triangle indices present */
    (*tpnl)[i][1] = 0;          /* space allocated for triangle indices */
  }
}

void createtpnl(int ***tpnl, int **tr, int ntr, int np)
{
  /* create a "triangle per node list"
   * this is a series of vectors, one per node
   * so per node a list of all the triangles that contain this node
   * is kept. the number of triangles is stored in [0] of the vector.
   * the total allocated spaces in the array is stored in [1].
   * the triangle indices are stored in [i] where i>=2.
   */

  int t;

  inittpnl(tpnl, np);
  for (t = 0; t < ntr; t++)
    addtr2tpnlist(tr, t, (*tpnl));
}

void freetpnl(int **tpnl, int np)
{
  int i;

  for (i = 0; i < np; i++)
    e_free(tpnl[i]);
}

/* end of file */
