/* Graphics output for monitoring mgn calculations */

/* ************************************************************************ */

#define MAIN_MGN_GRAPHICS
#include "mgn_graphics.h"
#ifdef HAVE_LIBG2

#include "timing.h"

#define GR_CONT    1
#define GR_REDRAW  2

static int g2Mode;
static double g2ScreenX, g2ScreenY;
static double g2TargetX, g2TargetY;
static double g2Scale;
static double g2Tiling;

/* local counter of the number stops in the program */
static int LocalStep = 0;


static void FindMaxExt(int NNodes, double **Coords,
                       double *Minx, double *Miny,
                       double *Maxx, double *Maxy);

static int UserInput(int NNodes, double **Coords);


/* ------------------------------------------------------------------------ */
static void FindMaxExt(int NNodes, double **Coords,
                       double *Minx, double *Miny, double *Maxx, double *Maxy)
{
  int i;

  *Minx = Coords[0][0];
  *Maxx = Coords[0][0];
  *Miny = Coords[0][1];
  *Maxy = Coords[0][1];
  for (i = 1; i < NNodes; i++)
  {
    if (*Minx > Coords[i][0])
      *Minx = Coords[i][0];
    if (*Maxx < Coords[i][0])
      *Maxx = Coords[i][0];

    if (*Miny > Coords[i][1])
      *Miny = Coords[i][1];
    if (*Maxy < Coords[i][1])
      *Maxy = Coords[i][1];
  }
}


static int UserInput(int NNodes, double **Coords)
{
  int c, w, h;
  double x, y;
  char input[20];

  printf("\nStep %d: Press 'Enter' to continue ('h' for help)\n", 
         LocalStep-1);
  fgets(input, 20, stdin);
  c = input[0];

  switch (c)
  {
  case 'r':
    return (GR_REDRAW);
  case 'h':
    printf("\n---------------------------------------------------------------");
    printf("\nGraphical commands");
    printf("\n\tEnter:\t continue");
    printf("\n\th:\t this help screen");
    printf("\n\tr:\t redraw");
    printf("\n\tw N M:\t new window size (N x M)");
    printf("\n\te:\t toggle the output of element numbers");
    printf("\n\to:\t toggle the output of node numbers");
    printf("\n\tn:\t toggle the output of circles for nodes");
    printf("\n\ts:\t toggle the output of the title string");
    printf("\n\tip:\t increase point size");
    printf("\n\tdp:\t decrease point size");
    printf("\n\tz n:\t zoom in");
    printf("\n\tx n:\t zoom out");
    printf("\n\tt x y:\t set target point of viewing");
    printf("\n\tc n:\t continue without interruption for n steps");
    printf("\n\tf:\t toggle the output of filled elements");
    printf("\n\tg:\t toggle the output of the outline the element boundary");
    printf("\n\ty n:\t set tiling percentage [0.0 <= n <= 0.5]");
    printf("\n\tq\t quit");
    printf("\n---------------------------------------------------------------");
    return (GR_REDRAW);
  case '\n':
    return (GR_CONT);
  case 'c':
    if(sscanf(input + 1, "%d", &mgn_g2Step) != 1)
    {
      mgn_g2Step = 0;
      printf("\nWarning: no parameter specified for command 'c' !");
    }
    return (GR_CONT);
  case 'w':
    if(sscanf(input + 1, "%d %d", &w, &h) != 2)
    {
      printf("\nWarning: not enough parameter specified for command 'w' !");
    }
    if (w != 0 && h != 0)
    {
      mgnCloseGraphics();
      mgnInitGraphics(w, h, NNodes, Coords);
    }
    return (GR_REDRAW);
  case 't':
    if(sscanf(input + 1, "%lf %lf", &x, &y) == 2)
    {
      g2TargetX = x;
      g2TargetY = y;
    }
    else
    {
      printf("\nWarning: not enough parameter specified for command 't' !");
    }
    return (GR_REDRAW);
  case 'z':
    if(sscanf(input + 1, "%lf", &x) == 1)
    {
      g2Scale *= x;
    }
    else
    {
      printf("\nWarning: no parameter specified for command 'z' !");
    }
    return (GR_REDRAW);
  case 'x':
    if(sscanf(input + 1, "%lf", &x) == 1)
    {
      g2Scale /= x;
    }
    else
    {
      printf("\nWarning: no parameter specified for command 'x' !");
    }
    return (GR_REDRAW);
  case 's':
    if(g2Mode & G2_DRAW_TITLE)
    {
      g2Mode = g2Mode & (~G2_DRAW_TITLE);
    }
    else
    {
      g2Mode |= G2_DRAW_TITLE;
    }
    return (GR_REDRAW);
  case 'e':
    if(g2Mode & G2_DRAW_ETEXT)
    {
      g2Mode = g2Mode & (~G2_DRAW_ETEXT);
    }
    else
    {
      g2Mode |= G2_DRAW_ETEXT;
    }
    return (GR_REDRAW);
  case 'o':
    if(g2Mode & G2_DRAW_NTEXT)
    {
      g2Mode = g2Mode & (~G2_DRAW_NTEXT);
    }
    else
    {
      g2Mode |= G2_DRAW_NTEXT;
    }
    return (GR_REDRAW);
  case 'n':
    if(g2Mode & G2_DRAW_NODE)
    {
      g2Mode = g2Mode & (~G2_DRAW_NODE);
    }
    else
    {
      g2Mode |= G2_DRAW_NODE;
    }
    return (GR_REDRAW);
  case 'f':
    if(g2Mode & G2_FILL_ELEM)
    {
      g2Mode = g2Mode & (~G2_FILL_ELEM);
    }
    else
    {
      g2Mode |= G2_FILL_ELEM;
    }
    return (GR_REDRAW);
  case 'g':
    if(g2Mode & G2_OUTLINE_ELEM)
    {
      g2Mode = g2Mode & (~G2_OUTLINE_ELEM);
    }
    else
    {
      g2Mode |= G2_OUTLINE_ELEM;
    }
    return (GR_REDRAW);
  case 'y':
    if(sscanf(input + 1, "%lf", &x) == 1)
    {
      if(   x >= 0.0
         && x <= 0.5  )
      {
        g2Tiling = x;
      }
      else
      {
        printf("\nWarning: invalid parameter specified for command 'y' !");
      }
    }
    else
    {
      printf("\nWarning: no parameter specified for command 'y' !");
    }
    return (GR_REDRAW);
  case 'i':
    if(input[1] == 'p')
    {
      mgn_g2PointSize *= 1.5;
    }
    return (GR_REDRAW);
  case 'd':
    if(input[1] == 'p')
    {
      mgn_g2PointSize /= 1.5;
    }
    return (GR_REDRAW);
  case 'q':
    mgnCloseGraphics();
    printf("\nexecution aborted\n");
    exit(0);
  default:
    printf("\nUnrecognised graphical command: %s", input);
    return (GR_REDRAW);
  }
}

/* ---------------------------------------------------------------------------- */

void MeshToScreen(double x, double y, double *nx, double *ny)
{
  *nx = (x - g2TargetX) * g2Scale + g2ScreenX;
  *ny = (y - g2TargetY) * g2Scale + g2ScreenY;
}



void mgnInitGraphics(int w, int h, int NNodes, double **Coords)
{
  double d, scale;
  double Minx, Miny, Maxx, Maxy;
  
  temptime = p_Timer();

#ifdef WIN32
  mgn_g2Device = g2_open_win32(w, h, "MGN Graphics", 0);
  g2_set_background(mgn_g2Device, 0); /* white background */
  g2_set_auto_flush(mgn_g2Device, 0);
#else
  mgn_g2Device =
      g2_open_X11X(w, h, 100, 100, "MGN Graphics", "MGN", NULL, 20, 23);
      /* g2_open_GIF("output.png", w, h); */
  
  g2_set_background(mgn_g2Device, 1); /* black background */
  g2_set_auto_flush(mgn_g2Device, 1);
#endif

  g2_set_line_width(mgn_g2Device, 1);
  g2_set_font_size(mgn_g2Device, 12);

#ifdef WIN32
  g2_pen(mgn_g2Device, 1);      /* black */
#else
  g2_pen(mgn_g2Device, 0);      /* white */
#endif

  FindMaxExt(NNodes, Coords, &Minx, &Miny, &Maxx, &Maxy);

  d = ((Maxx - Minx) > (Maxy - Miny)) ? (Maxx - Minx) : (Maxy - Miny);

  if (d == 0.0)
    fprintf(stderr, "\nERROR: Extension of mesh is zero !\n");

  mgn_g2PointSize = 1;
  scale = ((h > w) ? w : h) / d;
  /* leave some space around the displayed mesh */
  g2Scale = scale * 0.9;

  /* default tiling percentage */
  g2Tiling = 0.1;

  g2ScreenX = w / 2.0;
  g2ScreenY = h / 2.0;

  g2TargetX = Minx + (Maxx - Minx) / 2.0;
  g2TargetY = Miny + (Maxy - Miny) / 2.0;


  g2Mode = G2_DRAW_NODE | G2_INTERRUPT | G2_DRAW_NTEXT | G2_FILL_ELEM ;

  p_StartTimer();
  timerStart -= temptime;

}


void mgnCloseGraphics(void)
{
  temptime = p_Timer();

  g2_close(mgn_g2Device);

  p_StartTimer();
  timerStart -= temptime;

}

void mgnClearGraphics(void)
{
  g2_clear(mgn_g2Device);
}

void mgnDrawNumber(double x, double y, int n, double offset)
{
  char number[20];

#ifdef WIN32
  sprintf(number, "%d", n);
#else
  snprintf(number, 19, "%d", n);
#endif

#ifdef WIN32
  g2_pen(mgn_g2Device, 1);      /* black */
#else
  g2_pen(mgn_g2Device, 0);      /* white */
#endif
  g2_string(mgn_g2Device, x + offset, y + offset, number);

}


void mgnDrawNode(double x, double y, int n, int op)
{
  if (op & G2_DRAW_NODE)
  {

#ifdef WIN32
    g2_pen(mgn_g2Device, 1);    /* black */
#else
    g2_pen(mgn_g2Device, 0);    /* white */
#endif
    g2_filled_circle(mgn_g2Device, x, y, mgn_g2PointSize);
  }

}

void mgnDrawAllNode(int NCoord, double **Coords, int op)
{
  int i, j;
  double x, y;
  double rx, ry, off;
  double nx, ny;

  for (i = 0; i < NCoord; i++)
  {
    x = Coords[i][0];
    y = Coords[i][1];
    MeshToScreen(x, y, &nx, &ny);

    mgnDrawNode(nx, ny, i + 1, op);

    off = mgn_g2PointSize;
    for (j = 0; j < NCoord; j++)
    {
      if (j == i)
        continue;
      rx = Coords[j][0];
      ry = Coords[j][1];

      if (fabs(rx - x) <= 1e-3 && fabs(ry - y) <= 1e-3 && j < i)
      {
        off = off * -3.0;
        break;
      }
    }

    if (op & G2_DRAW_NTEXT)
    {
      mgnDrawNumber(nx, ny, i + 1, off);
    }

  }
}


void mgnDrawQuad(double **coords, int *nodes, int n, int op, int index)
{
  int n1, n2, n3, n4;
  double x1, y1, x2, y2, x3, y3, x4, y4;
  double xc, yc, xn, yn;

  n1 = nodes[0] + index;
  n2 = nodes[1] + index;
  n3 = nodes[2] + index;
  n4 = nodes[3] + index;

  x1 = coords[n1][0];
  y1 = coords[n1][1];
  MeshToScreen(x1, y1, &x1, &y1);

  x2 = coords[n2][0];
  y2 = coords[n2][1];
  MeshToScreen(x2, y2, &x2, &y2);

  x3 = coords[n3][0];
  y3 = coords[n3][1];
  MeshToScreen(x3, y3, &x3, &y3);

  x4 = coords[n4][0];
  y4 = coords[n4][1];
  MeshToScreen(x4, y4, &x4, &y4);

  xc = (x1 + x2 + x3 + x4) / 4.0;
  yc = (y1 + y2 + y3 + y4) / 4.0;

  if(op & G2_FILL_ELEM)
  {
    g2_pen(mgn_g2Device, 4); /* green filling */
    /* cannot rely on quad display 
       because for that only two nodes can be given */
    g2_filled_triangle(mgn_g2Device,
                       x1 + (xc - x1) * g2Tiling,
                       y1 + (yc - y1) * g2Tiling,
                       x2 + (xc - x2) * g2Tiling,
                       y2 + (yc - y2) * g2Tiling,
                       x3 + (xc - x3) * g2Tiling, 
                       y3 + (yc - y3) * g2Tiling);
    g2_filled_triangle(mgn_g2Device,
                       x1 + (xc - x1) * g2Tiling,
                       y1 + (yc - y1) * g2Tiling,
                       x3 + (xc - x3) * g2Tiling,
                       y3 + (yc - y3) * g2Tiling,
                       x4 + (xc - x4) * g2Tiling, 
                       y4 + (yc - y4) * g2Tiling);
  }

  if(op & G2_OUTLINE_ELEM)
  {
#ifdef WIN32
    g2_pen(mgn_g2Device, 1);  /* black outline */	
#else
    g2_pen(mgn_g2Device, 25); /* yellow outline */
#endif
    xn = x1 + (xc - x1) * g2Tiling;
    yn = y1 + (yc - y1) * g2Tiling;
    g2_move(mgn_g2Device, xn, yn);
    xn = x2 + (xc - x2) * g2Tiling;
    yn = y2 + (yc - y2) * g2Tiling;
    g2_line_to(mgn_g2Device, xn, yn);
    xn = x3 + (xc - x3) * g2Tiling;
    yn = y3 + (yc - y3) * g2Tiling;
    g2_line_to(mgn_g2Device, xn, yn);
    xn = x4 + (xc - x4) * g2Tiling;
    yn = y4 + (yc - y4) * g2Tiling;
    g2_line_to(mgn_g2Device, xn, yn);
    xn = x1 + (xc - x1) * g2Tiling;
    yn = y1 + (yc - y1) * g2Tiling;
    g2_line_to(mgn_g2Device, xn, yn);
  }


  if (op & G2_DRAW_ETEXT)
  {
    mgnDrawNumber(xc, yc, n, 0.0);
  }
}


void mgnDrawAllQuad(double **coords, int **nodes, int n, int op, int index)
{
  int i;

  for (i = 0; i < n; i++)
  {
    mgnDrawQuad(coords, nodes[i], i + 1, op, index);
  }
}


void mgnDrawTriang(double **coords, int *nodes, int n, int op, int index)
{
  int n1, n2, n3;
  double x1, y1, x2, y2, x3, y3;
  double xc, yc;

  n1 = nodes[0] + index;
  n2 = nodes[1] + index;
  n3 = nodes[2] + index;

  x1 = coords[n1][0];
  y1 = coords[n1][1];
  MeshToScreen(x1, y1, &x1, &y1);

  x2 = coords[n2][0];
  y2 = coords[n2][1];
  MeshToScreen(x2, y2, &x2, &y2);

  x3 = coords[n3][0];
  y3 = coords[n3][1];
  MeshToScreen(x3, y3, &x3, &y3);

  xc = (x1 + x2 + x3) / 3.0;
  yc = (y1 + y2 + y3) / 3.0;

  if(op & G2_FILL_ELEM)
  {
    g2_pen(mgn_g2Device, 4); /* green filling */
    g2_filled_triangle(mgn_g2Device,
                       x1 + (xc - x1) * g2Tiling,
                       y1 + (yc - y1) * g2Tiling,
                       x2 + (xc - x2) * g2Tiling,
                       y2 + (yc - y2) * g2Tiling,
                       x3 + (xc - x3) * g2Tiling, 
                       y3 + (yc - y3) * g2Tiling);
  }

  if(op & G2_OUTLINE_ELEM)
  {
#ifdef WIN32
    g2_pen(mgn_g2Device, 1);  /* black outline */	
#else
    g2_pen(mgn_g2Device, 25); /* yellow outline */
#endif
    g2_triangle(mgn_g2Device,
                x1 + (xc - x1) * g2Tiling,
                y1 + (yc - y1) * g2Tiling,
                x2 + (xc - x2) * g2Tiling,
                y2 + (yc - y2) * g2Tiling,
                x3 + (xc - x3) * g2Tiling, 
                y3 + (yc - y3) * g2Tiling);
  }

  if(op & G2_DRAW_ETEXT)
  {
    mgnDrawNumber(xc, yc, n, 0.0);
  }
}

void mgnDrawAllTriang(double **coords, int **nodes, int n, int op, int index)
{
  int i;

  for (i = 0; i < n; i++)
  {
    mgnDrawTriang(coords, nodes[i], i + 1, op, index);
  }
}


void mgnDrawAll(int NNodes, double **NodeCoords,
                int NTriang, int **NodesTriang,
                int NQuad, int **NodesQuad, int index,
                void (*mgnExtDraw) (), void *data, char *title)
{
  int subop, out;

  temptime = p_Timer();
  LocalStep++;

  if(mgn_g2Step > 0)
  {
    mgn_g2Step--;
    p_StartTimer();
    timerStart -= temptime;
    return;
  }

  out = 0;
  do
  {
    subop = g2Mode & (~G2_INTERRUPT);

    mgnClearGraphics();

    if (NQuad > 0)
    {
      mgnDrawAllQuad(NodeCoords, NodesQuad, NQuad, subop, index);
    }
    if (NTriang > 0)
    {
      mgnDrawAllTriang(NodeCoords, NodesTriang, NTriang, subop, index);
    }

    if ((g2Mode & G2_DRAW_NTEXT) || (g2Mode & G2_DRAW_NODE))
    {
      mgnDrawAllNode(NNodes, NodeCoords, subop);
    }

    if (mgnExtDraw != NULL)
    {
      (*mgnExtDraw) (NNodes, NodeCoords, NTriang, NodesTriang,
                     NQuad, NodesQuad, index, data, subop);
    }

    if ((g2Mode & G2_DRAW_TITLE) && (title != NULL))
    {
      g2_string(mgn_g2Device, 10, 10, title);
    }

#ifdef WIN32
    g2_flush(mgn_g2Device);
#endif

    if (g2Mode & G2_INTERRUPT)
      out = UserInput(NNodes, NodeCoords);
  }
  while (out == GR_REDRAW);

  p_StartTimer();
  timerStart -= temptime;

}

#endif /* HAVE_LIBG2 */



