#ifndef OPERATION_H
#define OPERATION_H

#include "pav_main.h"

#define ALLOC_UNDOPOOL_SIZE    64

/* undo operations */
void UndoMarker(void);
void UndoUntilMarker(PAVEDMESH *mesh);
void InitUndo(void);
void EmptyUndoQueue(void);

/* paving mesh operations */
int CreateNode(double x, double y, int flag, PAVEDMESH *mesh, int undo);
int DeleteNode(PAVEDMESH *mesh, int n, int undo);
int ChangeNodeCoord(int n, double dx, double dy, PAVEDMESH *mesh, int undo);
int CreateElement(int n1, int n2, int n3, int n4, PAVEDMESH *mesh, int undo);
int DeleteElement(int elem, PAVEDMESH *mesh, int undo);
int ChangeElementNode(int elem, int old, int new, PAVEDMESH *mesh, int undo);

void SetLoopHead(LOOP *loop, NODE *node, int undo);
void StepForward(LOOP *loop, int undo);
void StepBackward(LOOP *loop, int undo);
void RemoveBehindFrontNode(PAVEDMESH *mesh, LOOP *loop, int undo);
void InsertBehindFrontNode(PAVEDMESH *mesh, NODE *node, LOOP *loop, int undo);
void NextLoop(PAVEDMESH *mesh, int undo);
void PrevLoop(PAVEDMESH *mesh, int undo);
void AddLoop(PAVEDMESH *mesh, LOOP *loop, int undo);
void RemoveLoop(PAVEDMESH *mesh, LOOP *loop, int undo);
void MergeCutLoops(PAVEDMESH *mesh, NODE *new, NODE *old, int undo);
int StageIncr(NODE *node, int undo);
int StageDecr(NODE *node, int undo);

/* no undo for these functions */
PAVEDMESH *InitPavedMesh(void);
LOOP *InitLoop(int type, NODE *node);
void CountLoopNodes(LOOP *loop);
NODE *GetNewNode(PAVEDMESH *mesh);
void ClearNode(NODE *node);
void RecycleNode(NODE *node, PAVEDMESH *mesh);
NODE *DuplicateNode(NODE *Node, PAVEDMESH *mesh);

#endif




