#include "mgn.h"
#include "rgr.h"
#include "dtr.h"
#include "eg_lib.h"

static int compare_sortx(const void *i, const void *j);
static int compare_sorty(const void *i, const void *j);

static double **crd;

/*****************************************************************************/

int make_grid_points(double ***coords, int *np, int *tp,
                     int *stnp, double **mpr,
                     int ***tpnl,
                     double gprm, double umpr,
                     double **fcoords, int fnp, int **ftr, int fntr)
{
  int nnp = 0;                  /* number of new points */
  double left, right, top, bottom; /* extremities of the mesh */
  int n, i;                     /* counters */
  double xpos, ypos;            /* current position for a grid node */
  double pos[3];                /* the same */
  double xdif, ydif;            /* differences between positions */
  int row;                      /* row number */
  int ct;                       /* triangle current position is in */
  int onside;                   /* something for the trianglesearch */
  int sres;                     /* result of the triangle search */
  double ds;                    /* distance */
  int include;

  /* spectacular new global search! */
  int *sortx, *sorty, *rankx, *ranky; /* sort and rank vectors */
  int sxmin, symin, sxmax, symax; /* begin-index end-index of search region */
  int x;                        /* index inside the search region */

  MTRF mtrf;
  mtrf.used = FALSE;

  (*stnp) = (*np);

  /* grid characteristics */
  left = fcoords[0][0];
  right = fcoords[0][0];
  top = fcoords[0][1];
  bottom = fcoords[0][1];
  for (n = 0; n < fnp; n++)
  {
    if (fcoords[n][0] < left)
      left = fcoords[n][0];
    if (fcoords[n][0] > right)
      right = fcoords[n][0];
    if (fcoords[n][1] < bottom)
      bottom = fcoords[n][1];
    if (fcoords[n][1] > top)
      top = fcoords[n][1];
  }

  xdif = umpr;
  ydif = umpr * sqrt(3.0) / 2.0;
  row = 0;
  xpos = left + ((row % 2) + 1) * (xdif / 2.0);
  ypos = bottom + ydif;

  /* global search init */
  sortx = e_alloc_i_vect(fnp);
  sorty = e_alloc_i_vect(fnp);
  rankx = e_alloc_i_vect(fnp);
  ranky = e_alloc_i_vect(fnp);
  for (i = 0; i < fnp; i++)
  {
    sortx[i] = i;
    sorty[i] = i;
    rankx[i] = i;
    ranky[i] = i;
  }
  crd = fcoords;                /* make coords accessible for sorting */
  qsort(sortx, fnp, sizeof(int), compare_sortx);
  qsort(sorty, fnp, sizeof(int), compare_sorty);
  for (i = 0; i < fnp; i++)
  {
    rankx[sortx[i]] = i;
    ranky[sorty[i]] = i;
  }

  symin = 0;
  symax = 0;

  /* start of the grid creation */
  while (ypos < top)
  {
    sxmin = 0;
    sxmax = 0;
    while (fcoords[sorty[symin]][1] < ypos - gprm * umpr)
      symin++;
    while (fcoords[sorty[symax]][1] < ypos + gprm * umpr)
    {
      symax++;
      if (symax == (fnp))
        break;
    }
    symax--;

    while (xpos < right)
    {
      while (fcoords[sortx[sxmin]][0] < xpos - gprm * umpr)
        sxmin++;
      while (fcoords[sortx[sxmax]][0] < xpos + gprm * umpr)
      {
        sxmax++;
        if (sxmax == (fnp))
          break;
      }
      sxmax--;

      pos[0] = xpos;
      pos[1] = ypos;
      pos[2] = 0.0;

/*      sres = trianglesearch(pos, fcoords, ftr, sttr, &ct, &onside, &mtrf); */
			sres = exh_tr_search(pos, fcoords, ftr, fntr, &ct, &onside, &mtrf);
      if (sres == 0)            /* node outside hull */
        include = FALSE;
      else if (sres == -1)      /* duplicate node */
        include = FALSE;
      else if (ftr[ct][10] == -1)
        include = FALSE;
      else                      /* node inside somewhere */
      {
        /* check for grid acceptance parameter */
        include = TRUE;
        for (x = sxmin; x <= sxmax; x++)
          if ((ranky[sortx[x]] >= symin) && (ranky[sortx[x]] <= symax))
          {
            ds = eg_PointDist2D(pos, fcoords[sortx[x]]);
            if (ds < umpr * gprm)
            {
              include = FALSE;
              break;
            }
          }
        if (include)
        {
          include_grid_node(pos, coords, np, tp, mpr, &nnp, tpnl, umpr);
        }
      }
      xpos += xdif;
    }
    ypos += ydif;
    row++;
    xpos = left + ((row % 2) + 1) * (xdif / 2.0);
  }
  return nnp;
}

void include_grid_node(double pos[3], double ***coords, int *np, int *tp,
                       double **mpr, int *nnp, int ***tpnl, double nmpr)
{

    /** temporary check
    for(i=0;i<*np;i++)
	if (eg_PointDist2D((*coords)[i], pos)<1e-6)
	{
	    if (i>(*np)-(*nnp)-1)
	    {
		printf("inserted node no %d\n", (*nnp)+1);
		printf("double with %d out of %d\n\n", i+1, (*np));
	    }
	    else
		e_GeneralWarning("node generated twice");
	} **/

  if ((*tp) - (*np) < 1)
  {
    (*coords) = e_incr_cont_d_matr((*tp), 3, (*coords), (*tp));
    (*tpnl) = e_incr_frag_i_matr((*tp), (*tpnl), (*tp));
    (*mpr) = e_incr_d_vect((*tp), (*mpr), (*tp));
    (*tp) += (*tp);
  }
  (*coords)[(*np)][0] = pos[0];
  (*coords)[(*np)][1] = pos[1];
  (*coords)[(*np)][2] = 0.0;
  (*tpnl)[(*np)] = e_alloc_i_vect(2);
  (*tpnl)[(*np)][0] = 0;
  (*tpnl)[(*np)][1] = 0;
  (*mpr)[(*np)] = nmpr;
  (*np)++;
  (*nnp)++;
}

static int compare_sortx(const void *i, const void *j)
{
  int *pi, *pj;

  pi = (int *) i;
  pj = (int *) j;

  if (crd[*pi][0] > crd[*pj][0])
    return +1;
  if (crd[*pi][0] < crd[*pj][0])
    return -1;
  return 0;
}

static int compare_sorty(const void *i, const void *j)
{
  int *pi, *pj;

  pi = (int *) i;
  pj = (int *) j;

  if (crd[*pi][1] > crd[*pj][1])
    return 1;
  if (crd[*pi][1] < crd[*pj][1])
    return -1;
  return 0;
}


/* end of file */
